package gregapi.code;

import gregapi.lang.LanguageHandler;

import java.util.ArrayList;

/**
 * @author Gregorius Techneticies
 * 
 * Useful for tagging things. It could Tag anything.
 * Better than Strings for tagging Stuff since you can do an == Check rather than needing to use equals.
 */
public final class TagData implements ICondition<ITagDataContainer> {
	private static final ArrayList<TagData> TAGS_INTERNAL = new ArrayListNoNulls<TagData>();
	public static final ArrayList<TagData> TAGS = new ArrayListNoNulls<TagData>();
	
	public final int mSubtagID;
	public final String mName;
	public String mChatFormat = "";
	
	private TagData(String aName) {
		mSubtagID = TAGS_INTERNAL.size();
		mName = aName;
		TAGS_INTERNAL.add(this);
		TAGS.add(this);
	}
	
	public static TagData createTagData(String aName, String aLocalShort, String aLocalLong, String aChatFormat) {
		TagData rTagData = createTagData(aName, aLocalShort, aLocalLong);
		rTagData.mChatFormat = aChatFormat;
		return rTagData;
	}
	
	public static TagData createTagData(String aName, String aLocalShort, String aLocalLong) {
		TagData rTagData = createTagData(aName);
		LanguageHandler.add(rTagData.getTranslatableNameShort(), aLocalShort);
		LanguageHandler.add(rTagData.getTranslatableNameLong(), aLocalLong);
		return rTagData;
	}
	
	public static TagData createTagData(String aName, String aLocal) {
		return createTagData(aName, aLocal, aLocal);
	}
	
	public static TagData createTagData(String aName) {
		aName = aName.toUpperCase();
		for (TagData tSubTag : TAGS_INTERNAL) if (tSubTag.mName.equals(aName)) return tSubTag;
		return new TagData(aName);
	}
	
	public String getTranslatableNameLong() {
		return "gt.td.long."+mName.toLowerCase();
	}
	
	public String getLocalisedNameLong() {
		return LanguageHandler.translate(getTranslatableNameLong(), mName);
	}
	
	public String getTranslatableNameShort() {
		return "gt.td.short."+mName.toLowerCase();
	}
	
	public String getLocalisedNameShort() {
		return LanguageHandler.translate(getTranslatableNameShort(), mName);
	}
	
	public String getChatFormat() {
		return mChatFormat;
	}
	
	@Override
	public String toString() {
		return mName;
	}
	
	@Override
	public boolean isTrue(ITagDataContainer aObject) {
		return aObject.contains(this);
	}
	
	@Override
	public boolean equals(Object aObject) {
		return (aObject instanceof TagData && ((TagData)aObject).mSubtagID == mSubtagID);
	}
	
	@Override
	public int hashCode() {
		return mSubtagID;
	}
}