package gregapi.item;

import static gregapi.data.CS.D1;
import gregapi.GT_API;
import gregapi.data.CS.ModIDs;
import gregapi.lang.LanguageHandler;
import gregapi.log.GT_Log;
import gregapi.util.UT;

import java.util.List;

import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.IIcon;
import net.minecraft.world.World;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidRegistry;
import cpw.mods.fml.common.registry.GameRegistry;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 */
public class ItemFluidDisplay extends Item {
	protected IIcon mIcon;
	private final String mName;
	
	public ItemFluidDisplay() {
		super();
    	mName = "gt.display.fluid";
		LanguageHandler.add(mName + ".name", "Fluid Display");
		GameRegistry.registerItem(this, mName, ModIDs.API);
		if (!D1) UT.Stacks.hide(this);
	}
	
	@Override
    public void addInformation(ItemStack aStack, EntityPlayer aPlayer, List aList, boolean aF3_H) {
		NBTTagCompound aNBT = aStack.getTagCompound();
		if (D1) {
	    	Fluid tFluid = FluidRegistry.getFluid(aStack.getItemDamage());
	    	if (tFluid != null) aList.add("Registry: " + tFluid.getName());
		}
		if (aNBT != null) {
			long tToolTipAmount = aNBT.getLong("mFluidDisplayAmount");
			if (tToolTipAmount > 0)
			aList.add(EnumChatFormatting.BLUE + "Amount: " + tToolTipAmount + EnumChatFormatting.GRAY);
			aList.add(EnumChatFormatting.RED + "Temperature: " + aNBT.getLong("mFluidDisplayHeat") + " K" + EnumChatFormatting.GRAY);
			aList.add(EnumChatFormatting.GREEN + "State: " + (aNBT.getBoolean("mFluidState")?"Gas":"Liquid") + EnumChatFormatting.GRAY);
		}
	}
	
	@Override
    @SideOnly(Side.CLIENT)
    public void registerIcons(IIconRegister aIconRegister) {
		// Useful hack to register Block Icons. That is why the Fluid Display Item has to exist always.
		if (GT_API.sGTPhaseFinishedPostLoad) {
			GT_Log.out.println("GT_Mod: Setting up Icon Register for Blocks");
			GT_API.sBlockIcons = aIconRegister;
			GT_Log.out.println("GT_Mod: Starting Block Icon Load Phase");
	    	for (Runnable tRunnable : GT_API.sBlockIconload) {
	    		try {
	    			tRunnable.run();
	    		} catch(Throwable e) {
	    			e.printStackTrace(GT_Log.err);
	    		}
	    	}
			GT_Log.out.println("GT_Mod: Registering Crop specific Textures");
			try {
				for (gregapi.old.GT_BaseCrop tCrop : gregapi.old.GT_BaseCrop.sCropList) tCrop.registerSprites(aIconRegister);
			} catch(Throwable e) {
				e.printStackTrace(GT_Log.err);
			}
			GT_Log.out.println("GT_Mod: Finished Block Icon Load Phase");
		}
    }
    
	@Override
    @SideOnly(Side.CLIENT)
    public IIcon getIconFromDamage(int aMeta) {
    	Fluid tFluid = FluidRegistry.getFluid(aMeta);
    	return tFluid==null?FluidRegistry.WATER.getStillIcon():tFluid.getStillIcon();
    }
	
    @Override
	@SideOnly(Side.CLIENT)
    public int getColorFromItemStack(ItemStack aStack, int aRenderPass) {
    	Fluid tFluid = FluidRegistry.getFluid(aStack.getItemDamage());
    	return tFluid==null?16777215:tFluid.getColor();
    }
    
	@Override
    public int getSpriteNumber() {
        return 0;
    }
	
	@Override
    public String getUnlocalizedName(ItemStack aStack) {
	    if (aStack != null) return UT.Fluids.name(FluidRegistry.getFluid(aStack.getItemDamage()), false);
        return "";
    }
	
	@Override
    public String getItemStackDisplayName(ItemStack aStack) {
	    if (aStack != null) return UT.Fluids.name(FluidRegistry.getFluid(aStack.getItemDamage()), true);
        return "";
    }
    
	@Override
	@SideOnly(Side.CLIENT)
	public void getSubItems(Item aItem, CreativeTabs aTab, List aList) {
		if (D1) for (int i = 0, j = FluidRegistry.getMaxID(); i < j; i++) {
			ItemStack tStack = UT.Fluids.display(FluidRegistry.getFluid(i));
			if (tStack != null) aList.add(tStack);
		}
        for (String tName : UT.Books.BOOK_LIST) aList.add(UT.Books.getBookWithTitle(tName));
	}
	
	@Override public final Item setUnlocalizedName(String aName) {return this;}
	@Override public final String getUnlocalizedName() {return mName;}
    
	@Override
    public boolean doesSneakBypassUse(World aWorld, int aX, int aY, int aZ, EntityPlayer aPlayer) {
        return true;
    }
	
	@Override
    public ItemStack getContainerItem(ItemStack aStack) {
        return null;
    }
    
	@Override
    public final boolean hasContainerItem(ItemStack aStack) {
        return getContainerItem(aStack) != null;
    }
}