package gregapi.oredict;

import static gregapi.data.CS.D2;
import static gregapi.data.CS.STATE_SOLID;
import static gregapi.data.CS.U;
import gregapi.GT_API;
import gregapi.code.*;
import gregapi.data.*;
import gregapi.data.TC.TC_AspectStack;
import gregapi.log.GT_Log;
import gregapi.render.TextureSet;
import gregapi.util.UT;

import java.util.*;

import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.ItemStack;

/**
 * @author Gregorius Techneticies
 */
public final class OreDictPrefix implements IOreDictListenerEvent, ITagDataContainer<OreDictPrefix> {
	/**
	 * This List is sorted by the length of the Prefixes.
	 * This is to prevent smaller Prefixes from breaking larger ones by just starting with the same few Characters the larger one starts with.
	 */
	private static final List<OreDictPrefix> VALUES_INTERNAL = new ArrayListNoNulls<OreDictPrefix>();
	
	/** The List of all Values in the order they have been added. not really sorted at all. */
	public static final List<OreDictPrefix> VALUES = new ArrayListNoNulls<OreDictPrefix>();
	/** The Map containing all Prefixes by their Name. */
	public static final Map<String, OreDictPrefix> sPrefixes = new HashMap<String, OreDictPrefix>();
	
	private final Set<TagData> mTags = new HashSetNoNulls<TagData>();
	private final Set<OreDictMaterial> mItemGeneratorBlackList = new HashSetNoNulls<OreDictMaterial>(), mIgnoredRegistrations = new HashSetNoNulls<OreDictMaterial>(), mItemGeneratorForced = new HashSetNoNulls<OreDictMaterial>();
	private final Set<IOreDictListenerEvent> mListenersOre = new HashSetNoNulls<IOreDictListenerEvent>();
	public final Set<IOreDictListenerItem> mListenersItem = new HashSetNoNulls<IOreDictListenerItem>();
	public final String mNameInternal;
	public OreDictPrefix mTargetRegistration = this;
	public CreativeTabs mCreativeTab = null;
	
	public ItemStack mContainerItem = null;
	private ICondition<OreDictMaterial> mCondition = ICondition.FALSE;
	/** The Indices of the Icons inside the Texture Sets. -1 if it doesn't have a Set */
	public int mIconIndexBlock = -1, mIconIndexItem = -1;
	public byte mConfigStackSize = 64, mDefaultStackSize = 64, mMinimumStackSize = 1, mState = STATE_SOLID;
	public long mAmount = -1;
	public float mHeatDamage = 0.0F;
	public String mNameLocal, mMaterialPre, mMaterialPost, mNameCategory;
	/** List of Prefixes which are familiar to this Prefix. Like "dust" having "dustSmall" and "dustTiny" and vice versa. Note that this per Default also contains the Prefix itself inside this Set. */
	public final Set<OreDictPrefix> mFamiliarPrefixes = new HashSetNoNulls<OreDictPrefix>();
	/** Secondary Materials of this Prefix. OreDictMaterialStacks are In Material Units */
	public final List<OreDictMaterialStack> mByProducts = new ArrayListNoNulls<OreDictMaterialStack>();
	/** List of ThaumCraft Aspects. */
	public final List<TC_AspectStack> mAspects = new ArrayListNoNulls<TC_AspectStack>(1);
	
	public static OreDictPrefix createPrefix(String aName) {
		if (GT_API.sAPIPhaseStartedLoad) throw new IllegalStateException("Prefixes have to be initialised in PreInit or earlier!");
		String tName = aName.replaceAll(" ", "").replaceAll("-", ""); // Auto-Replace all Spaces and Minuses.
		OreDictPrefix rPrefix = sPrefixes.get(tName);
		return rPrefix == null ? new OreDictPrefix(tName, aName) : rPrefix;
	}
	
	private OreDictPrefix(String aNameInternal, String aNameLocal) {
		mNameInternal = aNameInternal;
		mNameCategory = mNameLocal = aNameLocal;
		if (mNameInternal.contains("|") || mNameInternal.contains("*") || mNameInternal.contains(":") || mNameInternal.contains(".") || mNameInternal.contains("$")) throw new IllegalArgumentException("The Prefix Name contains invalid Characters!");
		if (mNameInternal.length() < 3) throw new IllegalArgumentException("A Prefix must have at least 3 Characters, otherwise it would break other Prefixes way to easily.");
		VALUES.add(this);
		if (VALUES_INTERNAL.isEmpty()) {
			VALUES_INTERNAL.add(this);
		} else for (int i = 0, j = VALUES_INTERNAL.size(); i < j; i++) {
			if (mNameInternal.length() >= VALUES_INTERNAL.get(i).mNameInternal.length()) {
				VALUES_INTERNAL.add(i, this);
				break;
			}
		}
		mFamiliarPrefixes.add(this);
	}
	
	public static OreDictPrefix get(String aOre) {
		for (int i = 0, j = VALUES_INTERNAL.size(); i < j; i++) if (aOre.startsWith(VALUES_INTERNAL.get(i).mNameInternal)) {
			if (VALUES_INTERNAL.get(i) == OP.oreNether && aOre.equals("oreNetherQuartz")) return OP.ore;
			return VALUES_INTERNAL.get(i);
		}
		return null;
	}
	
	/** Adds Identical Names which are getting re-registered to this Prefix */
	public OreDictPrefix addIdenticalNames(String... aNames) {
		for (String aName : aNames) createPrefix(aName).setRegistration(this);
		return this;
	}
	
	public OreDictPrefix addFamiliarPrefix(OreDictPrefix aPrefix) {
		mFamiliarPrefixes.add(aPrefix);
		return this;
	}
	
	/** The Re-Registration for the Ore Dictionary for invalid Prefixes */
	public OreDictPrefix setRegistration(OreDictPrefix aPrefix) {
		if (aPrefix != null) {
			mTargetRegistration = aPrefix;
			add(TD.Prefix.PREFIX_UNUSED);
		}
		return this;
	}
	
	public OreDictPrefix setMaterialStats(long aMaterialAmount) {
		add(TD.Prefix.MATERIAL_BASED);
		mAmount = aMaterialAmount;
		setStacksize(mAmount < U * 2 ? 64 : 64 / (mAmount / U));
		return this;
	}
	
	public OreDictPrefix setConfigStacksize(long aStacksize) {
		mConfigStackSize = (byte)UT.Code.bind(mMinimumStackSize, 64, aStacksize);
		mDefaultStackSize = (byte)UT.Code.bind(mMinimumStackSize, 64, aStacksize);
		return this;
	}
	
	public OreDictPrefix setStacksize(long aStacksize) {
		mDefaultStackSize = (byte)UT.Code.bind(mMinimumStackSize, 64, aStacksize);
		return this;
	}
	
	public OreDictPrefix setStacksize(long aStacksize, long aMinimumStacksize) {
		mMinimumStackSize = (byte)UT.Code.bind(1, 64, aMinimumStacksize);
		mDefaultStackSize = (byte)UT.Code.bind(mMinimumStackSize, 64, aStacksize);
		return this;
	}
	
	public OreDictPrefix setMinStacksize(long aMinimumStacksize) {
		mMinimumStackSize = (byte)UT.Code.bind(1, 64, aMinimumStacksize);
		mDefaultStackSize = (byte)UT.Code.bind(mMinimumStackSize, 64, mDefaultStackSize);
		return this;
	}
	
	public OreDictPrefix setCategoryName(String aCategoryName) {
		mNameCategory = aCategoryName;
		return this;
	}
	
	/** sets the State of things which are of this Prefix. 0 - 3 are from Solid to Plasma. Solid is Default. */
	public OreDictPrefix setState(long aState) {
		mState = (byte)UT.Code.bind(0, 3, aState);
		return this;
	}
	
	public OreDictPrefix setCondition(ICondition<OreDictMaterial> aCondition) {
		mCondition = aCondition==null?ICondition.FALSE:aCondition;
		return this;
	}
	
	public OreDictPrefix setLocalItemName(String aPreMaterial, String aPostMaterial) {
		mMaterialPre = aPreMaterial;
		mMaterialPost = aPostMaterial;
		return this;
	}
	
	public OreDictPrefix setLocalPrefixName(String aLocalName) {
		mNameLocal = aLocalName;
		return this;
	}
	
	public OreDictPrefix ignoreMaterials(OreDictMaterial... aMaterials) {
		if (aMaterials != null) for (OreDictMaterial aMaterial : aMaterials) if (aMaterial != null && !mIgnoredRegistrations.contains(aMaterial)) mIgnoredRegistrations.add(aMaterial);
		return this;
	}
	
	public OreDictPrefix disableItemGeneration(OreDictMaterial... aMaterials) {
		if (aMaterials != null) for (OreDictMaterial aMaterial : aMaterials) if (aMaterial != null && !mItemGeneratorBlackList.contains(aMaterial)) mItemGeneratorBlackList.add(aMaterial);
		return this;
	}
	
	public OreDictPrefix forceItemGeneration(OreDictMaterial... aMaterials) {
		if (aMaterials != null) for (OreDictMaterial aMaterial : aMaterials) if (aMaterial != null && !mItemGeneratorForced.contains(aMaterial)) mItemGeneratorForced.add(aMaterial);
		return this;
	}
	
	public boolean isGeneratingItem(OreDictMaterial aMaterial) {
		return aMaterial != null && (mItemGeneratorForced.contains(aMaterial) || (!mItemGeneratorBlackList.contains(aMaterial) && mCondition.isTrue(aMaterial)));
	}
	
	public boolean canGenerateItem(OreDictMaterial aMaterial) {
		return aMaterial != null && (mItemGeneratorForced.contains(aMaterial) || mCondition.isTrue(aMaterial));
	}
	
	private final List<OreDictRegistrationContainer> mRegistrations = new ArrayListNoNulls<OreDictRegistrationContainer>();
	
	public OreDictPrefix addListener(IOreDictListenerEvent aListener) {
		if (GT_API.sAPIPhaseStartedPostLoad) addListenerInternal(aListener); else mBufferedListeners.add(aListener);
		return this;
	}
	
	private final Set<IOreDictListenerEvent> mBufferedListeners = new HashSetNoNulls<IOreDictListenerEvent>();
	private void addListenerInternal(IOreDictListenerEvent aListener) {
		if (mListenersOre.add(aListener)) for (OreDictRegistrationContainer tEvent : mRegistrations) aListener.onOreRegistration(tEvent);
	}
	
	public void onPostLoad() {
		if (GT_API.sAPIPhaseStartedPostLoad) for (IOreDictListenerEvent aListener : mBufferedListeners) addListenerInternal(aListener);
	}
	
	public OreDictPrefix addListener(IOreDictListenerItem aListener) {
		mListenersItem.add(aListener);
		return this;
	}
	
	/**
	 * Adds the internal Name of this Prefix as Icon Name to all Texture Sets and sets its Icon Index to the Index of the Set.
	 * @param aModID the Mod responsible for adding the Icons (which are located in either assets\aModID.toLowerCase()\textures\items\materialicons\ or assets\aModID.toLowerCase()\textures\blocks\materialicons\).
	 */
	public OreDictPrefix addTextureSet(String aModID, boolean aIsItem) {
		if (aIsItem) mIconIndexItem = TextureSet.addToAll(aModID, aIsItem, mNameInternal); else mIconIndexBlock = TextureSet.addToAll(aModID, aIsItem, mNameInternal);
		return this;
	}
	
	public OreDictPrefix addAspects(TC_AspectStack... aAspects) {
		for (TC_AspectStack tAspect : aAspects) tAspect.addToAspectList(mAspects);
		return this;
	}
	
	public OreDictPrefix aspects(TC aAspect1, long aAmount1) {
		return addAspects(TC.stack(aAspect1, aAmount1));
	}
	public OreDictPrefix aspects(TC aAspect1, long aAmount1, TC aAspect2, long aAmount2) {
		return addAspects(TC.stack(aAspect1, aAmount1), TC.stack(aAspect2, aAmount2));
	}
	public OreDictPrefix aspects(TC aAspect1, long aAmount1, TC aAspect2, long aAmount2, TC aAspect3, long aAmount3) {
		return addAspects(TC.stack(aAspect1, aAmount1), TC.stack(aAspect2, aAmount2), TC.stack(aAspect3, aAmount3));
	}
	public OreDictPrefix aspects(TC aAspect1, long aAmount1, TC aAspect2, long aAmount2, TC aAspect3, long aAmount3, TC aAspect4, long aAmount4) {
		return addAspects(TC.stack(aAspect1, aAmount1), TC.stack(aAspect2, aAmount2), TC.stack(aAspect3, aAmount3), TC.stack(aAspect4, aAmount4));
	}
	public OreDictPrefix aspects(TC aAspect1, long aAmount1, TC aAspect2, long aAmount2, TC aAspect3, long aAmount3, TC aAspect4, long aAmount4, TC aAspect5, long aAmount5) {
		return addAspects(TC.stack(aAspect1, aAmount1), TC.stack(aAspect2, aAmount2), TC.stack(aAspect3, aAmount3), TC.stack(aAspect4, aAmount4), TC.stack(aAspect5, aAmount5));
	}
	public OreDictPrefix aspects(TC aAspect1, long aAmount1, TC aAspect2, long aAmount2, TC aAspect3, long aAmount3, TC aAspect4, long aAmount4, TC aAspect5, long aAmount5, TC aAspect6, long aAmount6) {
		return addAspects(TC.stack(aAspect1, aAmount1), TC.stack(aAspect2, aAmount2), TC.stack(aAspect3, aAmount3), TC.stack(aAspect4, aAmount4), TC.stack(aAspect5, aAmount5), TC.stack(aAspect6, aAmount6));
	}
	public OreDictPrefix aspects(TC aAspect1, long aAmount1, TC aAspect2, long aAmount2, TC aAspect3, long aAmount3, TC aAspect4, long aAmount4, TC aAspect5, long aAmount5, TC aAspect6, long aAmount6, TC aAspect7, long aAmount7) {
		return addAspects(TC.stack(aAspect1, aAmount1), TC.stack(aAspect2, aAmount2), TC.stack(aAspect3, aAmount3), TC.stack(aAspect4, aAmount4), TC.stack(aAspect5, aAmount5), TC.stack(aAspect6, aAmount6), TC.stack(aAspect7, aAmount7));
	}
	public OreDictPrefix aspects(TC aAspect1, long aAmount1, TC aAspect2, long aAmount2, TC aAspect3, long aAmount3, TC aAspect4, long aAmount4, TC aAspect5, long aAmount5, TC aAspect6, long aAmount6, TC aAspect7, long aAmount7, TC aAspect8, long aAmount8) {
		return addAspects(TC.stack(aAspect1, aAmount1), TC.stack(aAspect2, aAmount2), TC.stack(aAspect3, aAmount3), TC.stack(aAspect4, aAmount4), TC.stack(aAspect5, aAmount5), TC.stack(aAspect6, aAmount6), TC.stack(aAspect7, aAmount7), TC.stack(aAspect8, aAmount8));
	}
	public OreDictPrefix aspects(TC aAspect1, long aAmount1, TC aAspect2, long aAmount2, TC aAspect3, long aAmount3, TC aAspect4, long aAmount4, TC aAspect5, long aAmount5, TC aAspect6, long aAmount6, TC aAspect7, long aAmount7, TC aAspect8, long aAmount8, TC aAspect9, long aAmount9) {
		return addAspects(TC.stack(aAspect1, aAmount1), TC.stack(aAspect2, aAmount2), TC.stack(aAspect3, aAmount3), TC.stack(aAspect4, aAmount4), TC.stack(aAspect5, aAmount5), TC.stack(aAspect6, aAmount6), TC.stack(aAspect7, aAmount7), TC.stack(aAspect8, aAmount8), TC.stack(aAspect9, aAmount9));
	}
	public OreDictPrefix aspects(TC aAspect1, long aAmount1, TC aAspect2, long aAmount2, TC aAspect3, long aAmount3, TC aAspect4, long aAmount4, TC aAspect5, long aAmount5, TC aAspect6, long aAmount6, TC aAspect7, long aAmount7, TC aAspect8, long aAmount8, TC aAspect9, long aAmount9, TC aAspect0, long aAmount0) {
		return addAspects(TC.stack(aAspect1, aAmount1), TC.stack(aAspect2, aAmount2), TC.stack(aAspect3, aAmount3), TC.stack(aAspect4, aAmount4), TC.stack(aAspect5, aAmount5), TC.stack(aAspect6, aAmount6), TC.stack(aAspect7, aAmount7), TC.stack(aAspect8, aAmount8), TC.stack(aAspect9, aAmount9), TC.stack(aAspect0, aAmount0));
	}
	
	@Override
	public void onOreRegistration(OreDictRegistrationContainer aEvent) {
		if ((aEvent.mMaterial == MT.NULL && !contains(TD.Prefix.MATERIAL_BASED)) || (!mIgnoredRegistrations.contains(aEvent.mMaterial) && !aEvent.mMaterial.contains(TD.Properties.INVALID_MATERIAL))) {
    	    if (GT_API.sCompatTC != null) {
	    	    List<TC_AspectStack> tAspects = new ArrayListNoNulls<TC_AspectStack>();
	    	    for (TC_AspectStack tAspect : mAspects) tAspect.addToAspectList(tAspects);
	    	    if (mAmount >= U || mAmount < 0) for (TC_AspectStack tAspect : aEvent.mMaterial.mAspects) tAspect.addToAspectList(tAspects);
	    	    GT_API.sCompatTC.registerThaumcraftAspectsToItem(UT.Stacks.copyAmount(1, aEvent.mStack), tAspects, aEvent.mOreDictName);
    	    }
			for (IOreDictListenerEvent tListener : mListenersOre) {
				if (D2) GT_Log.ore.println("Processing '" + aEvent.mOreDictName + "' with the Prefix '" + mNameInternal + "' and the Material '" + aEvent.mMaterial.mNameInternal + "' at " + UT.Reflection.getClassName(tListener));
				tListener.onOreRegistration(aEvent);
			}
			if (aEvent.mMaterial != MT.NULL) aEvent.mMaterial.mRegisteredItems.add(new ItemStackContainer(aEvent.mStack));
			mRegisteredMaterials.add(aEvent.mMaterial);
			mRegisteredItems.add(new ItemStackContainer(aEvent.mStack));
			mRegistrations.add(aEvent);
		}
	}
	
	/** List of all valid Items, which are registered for this Prefix. */
	public final Set<ItemStackContainer> mRegisteredItems = new ItemStackSet<ItemStackContainer>();
	public final Set<OreDictMaterial> mRegisteredMaterials = new HashSetNoNulls<OreDictMaterial>();
	/** This is used to determine if any of the ItemStacks belongs to this Prefix. */
	public boolean contains(ItemStack... aStacks) {
		if (aStacks == null) return false;
		for (ItemStack aStack : aStacks) if (UT.Stacks.inList(aStack, mRegisteredItems)) return true;
		return false;
	}
	
	@Override
	public boolean contains(TagData aTag) {
		return mTags.contains(aTag);
	}
	
	@Override
	public boolean containsAll(TagData... aTags) {
		return mTags.containsAll(Arrays.asList(aTags));
	}
	
	@Override
	public boolean containsAll(Collection<TagData> aTags) {
		return mTags.containsAll(aTags);
	}
	
	@Override
	public OreDictPrefix add(TagData... aTags) {
		if (aTags != null) for (TagData aTag : aTags) mTags.add(aTag);
		return this;
	}
	
	@Override
	public boolean remove(TagData aTag) {
		return mTags.remove(aTag);
	}
	
	public OreDictItemData get(OreDictMaterial aMaterial) {
		return new OreDictItemData(this, aMaterial);
	}
	
	@Override
	public String toString() {
		return mNameInternal;
	}
}