package gregapi.data;

import static gregapi.data.CS.U;
import static gregapi.data.TD.Atomic.*;
import static gregapi.data.TD.Compounds.ALLOY;
import static gregapi.data.TD.Compounds.APPROXIMATE;
import static gregapi.data.TD.Compounds.DECOMPOSABLE;
import static gregapi.data.TD.Creative.HIDDEN;
import static gregapi.data.TD.ItemGenerator.*;
import static gregapi.data.TD.Processing.*;
import static gregapi.data.TD.Properties.*;
import static gregapi.render.TextureSet.*;
import gregapi.code.HashSetNoNulls;
import gregapi.data.CS.ModIDs;
import gregapi.oredict.OreDictConfigurationComponent;
import gregapi.oredict.OreDictMaterial;
import gregapi.render.TextureSet;
import gregapi.util.OM;
import gregapi.util.UT;

import java.util.Collection;

import net.minecraft.enchantment.Enchantment;
import net.minecraft.stats.AchievementList;
import cpw.mods.fml.common.Loader;

/**
 * @author Gregorius Techneticies
 * 
 * List of all Materials. The Short Name is for ease of overview and stands for "MaTerial".
 * 
 * Note: I wrote those shortcuts not only because of overview Reasons. I have hit the 65536 Limit of the static initialiser multiple times now.
 */
public class MT {
	public static final Collection<OreDictMaterial> ALL_MATERIALS_REGISTERED_HERE = new HashSetNoNulls<OreDictMaterial>();
	
	/** Making the Table a little bit more overviewable. DO NOT USE THESE FUNCTIONS YOURSELF!!! Use "OreDictMaterial.createMaterial(YOUR-ID-AS-SPECIFIED-IN-THE-ID-RANGES, OREDICT-NAME, LOCALISED-NAME)" */
	static OreDictMaterial unused		(String aNameOreDict) {return create(-1, aNameOreDict);}
	static OreDictMaterial invalid		(String aNameOreDict) {return unused(aNameOreDict).add(INVALID_MATERIAL);}
	static OreDictMaterial create		(int aID, String aNameOreDict) {OreDictMaterial rMaterial = OreDictMaterial.createMaterial(aID, aNameOreDict, aNameOreDict); ALL_MATERIALS_REGISTERED_HERE.add(rMaterial); return rMaterial;}
	static OreDictMaterial create		(int aID, String aNameOreDict, TextureSet[] aSets) {return create(aID, aNameOreDict).setTextures(aSets);}
	static OreDictMaterial create		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA) {return create(aID, aNameOreDict, aSets).setRGBa(aR, aG, aB, aA).add(aR==256?HIDDEN:null);}
	
	static OreDictMaterial element		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create		(aID, aNameOreDict, aSets, aR, aG, aB, aA).setStats(aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter).add(ELEMENT);}
	static OreDictMaterial metal		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return element		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(METAL, SMITHABLE, MELTING);}
	static OreDictMaterial metalloid	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return element		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(METALLOID, SMITHABLE, MELTING);}
	static OreDictMaterial nonmetal		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return element		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(NONMETAL);}
	static OreDictMaterial diatomic		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return nonmetal	(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(DIATOMIC_NONMETAL);}
	static OreDictMaterial polyatomic	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return nonmetal	(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(POLYATOMIC_NONMETAL);}
	static OreDictMaterial noblegas		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return nonmetal	(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(NOBLE_GAS);}
	static OreDictMaterial alkali		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(ALKALI_METAL);}
	static OreDictMaterial alkaline		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(ALKALINE_EARTH_METAL);}
	static OreDictMaterial lanthanide	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(LANTHANIDE);}
	static OreDictMaterial actinide		(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(ACTINIDE);}
	static OreDictMaterial transmetal	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(TRANSITION_METAL);}
	static OreDictMaterial posttrans	(int aID, String aNameOreDict, long aProtonsAndElectrons, long aNeutrons, long aMeltingPoint, long aBoilingPoint, double aGramPerCubicCentimeter, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal		(aID, aNameOreDict, aProtonsAndElectrons, aNeutrons, aMeltingPoint, aBoilingPoint, aGramPerCubicCentimeter, aSets, aR, aG, aB, aA).add(POST_TRANSITION_METAL);}
	
	static OreDictMaterial metal		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return create			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_INGOT, SMITHABLE, MELTING);}
	static OreDictMaterial metalore		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_INGOT_ORES);}
	static OreDictMaterial metalmachine	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_INGOT_MACHINE);}
	static OreDictMaterial metalmachore	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metalmachine	(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(G_INGOT_MACHINE_ORES);}
	static OreDictMaterial alloy		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metal			(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ALLOY, DECOMPOSABLE);}
	static OreDictMaterial alloyore		(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metalore		(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ALLOY, DECOMPOSABLE);}
	static OreDictMaterial alloymachine	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metalmachine	(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ALLOY, DECOMPOSABLE);}
	static OreDictMaterial alloymachore	(int aID, String aNameOreDict, TextureSet[] aSets, long aR, long aG, long aB, long aA)	{return metalmachore	(aID, aNameOreDict, aSets, aR, aG, aB, aA).add(ALLOY, DECOMPOSABLE);}
	
	/** Technical Material describing a Nullpointer */
	public static final OreDictMaterial NULL = create(-1, "NULL").setStatsElement(0,0,0,0,0).setTextures(SET_NONE).add(INVALID_MATERIAL, DONT_SHOW_THIS_COMPONENT);
	
	/** Material for empty Containers. */
	public static final OreDictMaterial Empty = create(0, "Empty").setStatsElement(0,0,0,0,0).setTextures(SET_NONE).add(EMPTY, AUTO_BLACKLIST, DONT_SHOW_THIS_COMPONENT);
	
	/** Some subatomic Particles and, this might be useful if Projectile Materials have to be determined and you shoot a Laser (Photons in that case) or something Radioactive. */
	public static final OreDictMaterial
	y		, Photon					=	y		= create(	1, "Photon"		).setStatsElement(0,0,0,0,0).setStatsEnergetic(0,0,0).setRGBa(255, 255, 255, 255).setTextures(SET_NONE).add(PARTICLE, HIDDEN),
	v		, Neutrino					=	v		= create(	2, "Neutrino"	).setStatsElement(0,0,0,0,0).setStatsEnergetic(0,0,0).setRGBa(180, 180, 180,   0).setTextures(SET_NONE).add(PARTICLE, HIDDEN),
	n		, Neutron					=	n		= create(	3, "Neutron"	).setStatsElement(0,0,1,0,0).setStatsEnergetic(0,0,0).setRGBa(128, 128, 128,   0).setTextures(SET_NONE).add(PARTICLE, HIDDEN),
	p		, Proton					=	p		= create(	4, "Proton"		).setStatsElement(1,0,0,0,0).setStatsEnergetic(0,0,0).setRGBa(255,   0,   0,   0).setTextures(SET_NONE).add(PARTICLE, HIDDEN),
	e		, Electron					=	e		= create(	5, "Electron"	).setStatsElement(0,1,0,0,0).setStatsEnergetic(0,0,0).setRGBa(  0,   0, 255,   0).setTextures(SET_NONE).add(PARTICLE, HIDDEN);
	
	/** Yes, I consider Magic as some kind of Element. I placed it at the Spot of "Anti-Empty", since Slot 0 is for the "Empty" Material, and "Anti-Empty" = "Magically-Full", or some crazy shit like that. Also Magic has negative Mass. */
	public static final OreDictMaterial
	Ma		, Magic						=	Ma		= create( 4000, "Magic"		).setStatsElement(0,0,0,-1,0).setStatsEnergetic(0,0,0).setTextures(SET_SHINY).add(ELEMENT, MAGICAL, UNBURNABLE, HIDDEN).setQuality(10.0F, 5120, 5);
	
	/**
	 * All the Elements and a few of their Isotopes.
	 * 
	 * in setStatsEnergetic I guessed some of the Values, because they were not listed in Wikipedia at that time.
	 * Those guessed Values are postfixed with an empty Space.
	 */
	public static final OreDictMaterial
	H		, Hydrogen					=	H		= diatomic		(   10,	"Hydrogen"				,   1,   0,    14,    20,  0.00008988, SET_DULL				,   0,   0, 255,  15).add(	CONTAINERS_GAS				, UUM												).aspects(	TC.AQUA, 1															),
	D		, Deuterium					=	D		= diatomic		(   11,	"Deuterium"				,   1,   1,    14,    20,  0.00008988, SET_SHINY			, 255, 255,   0,  15).add(	CONTAINERS_GAS																	).aspects(	TC.AQUA, 2															), H_2=D,
	T		, Tritium					=	T		= diatomic		(   12,	"Tritium"				,   1,   2,    14,    20,  0.00008988, SET_SHINY			, 255,   0,   0,  15).add(	CONTAINERS_GAS																	).aspects(	TC.AQUA, 3															), H_3=T,
	He		, Helium					=	He		= noblegas		(   20,	"Helium"				,   2,   2,     1,     4,  0.0001785 , SET_SHINY			, 255, 255, 120,  15).add(	CONTAINERS_GAS				, FUSION, UUM										).aspects(	TC.AER, 2															),
	He_3	, Helium3					=	He_3	= noblegas		(   21,	"Helium-3"				,   2,   1,	    1,     4,  0.0001785 , SET_SHINY			, 255, 255, 140,  15).add(	CONTAINERS_GAS				, FUSION											).aspects(	TC.AER, 3															),
	Li		, Lithium					=	Li		= alkali		(   30,	"Lithium"				,   3,   4,   453,  1560,  0.534	 , SET_ROUGH			, 225, 220, 255, 255).add(	G_DUST_ORES					, FUSION, UUM										).aspects(	TC.VITREUS, 1, TC.POTENTIA, 2										),
	Li_6	, Lithium6					=	Li_6	= alkali		(   31,	"Lithium-6"				,   3,   3,   453,  1560,  0.534	 , SET_ROUGH			, 230, 225, 255, 255).add(	G_DUST_ORES					, FUSION											).aspects(	TC.VITREUS, 1, TC.POTENTIA, 1, TC.RADIO, 1							),
	Be		, Beryllium					=	Be		= alkaline		(   40,	"Beryllium"				,   4,   5,  1560,  2742,  1.85      , SET_METALLIC			, 100, 180, 100, 255).add(	G_INGOT_ORES				, FUSION, UUM										).aspects(	TC.METALLUM, 2, TC.LUCRUM, 1										).setQuality(14.0F, 64, 2),
	B		, Boron						=	B		= metalloid		(   50,	"Boron"					,   5,   5,  2349,  4200,  2.34      , SET_DULL				, 250, 250, 250, 255).add(	G_INGOT_ORES				, FUSION, UUM										).aspects(	TC.METALLUM, 1, TC.VITREUS, 1, TC.ELECTRUM, 1						),
	C		, Carbon					=	C		= polyatomic	(   60,	"Carbon"				,   6,   6,  3800,  4300,  2.267     , SET_FINE				,  20,  20,  20, 255).add(	G_DUST_ORES					, FUSION, UUM										).aspects(	TC.VITREUS, 1, TC.IGNIS, 1											),
	C_13	, Carbon13					=	C_13	= polyatomic	(   61,	"Carbon-13"				,   6,   7,  3800,  4300,  2.267     , SET_FINE				,  25,  25,  25, 255).add(	G_DUST_ORES					, FUSION											).aspects(	TC.VITREUS, 1, TC.IGNIS, 1, TC.RADIO, 1								),
	C_14	, Carbon14					=	C_14	= polyatomic	(   62,	"Carbon-14"				,   6,   8,  3800,  4300,  2.267     , SET_FINE				,  30,  30,  30, 255).add(	G_DUST_ORES					, FUSION											).aspects(	TC.VITREUS, 1, TC.IGNIS, 1, TC.RADIO, 1								),
	N		, Nitrogen					=	N		= diatomic		(   70,	"Nitrogen"				,   7,   7,    63,    77,  0.0012506 , SET_DULL				,   0, 150, 200,  15).add(	CONTAINERS_GAS				, FUSION, UUM										).aspects(	TC.AER, 1															),
	O		, Oxygen					=	O		= diatomic		(   80,	"Oxygen"				,   8,   8,    54,    90,  0.001429  , SET_DULL				,   0, 100, 200,  15).add(	CONTAINERS_GAS				, FUSION, UUM										).aspects(	TC.AER, 1															),
	F		, Fluorine					=	F		= diatomic		(   90,	"Fluorine"				,   9,   9,    53,    85,  0.001696  , SET_DULL				,  64, 192,   0, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM										).aspects(	TC.PERDITIO, 2														),
	Ne		, Neon						=	Ne		= noblegas		(  100,	"Neon"					,  10,  10,    24,    27,  0.0008999 , SET_SHINY			, 250, 180, 180,  15).add(	CONTAINERS_GAS				, FUSION, UUM										).aspects(	TC.AER, 3															),
	Na		, Sodium					=	Na		= alkali		(  110,	"Sodium"				,  11,  11,   370,  1156,  0.971	 , SET_ROUGH			,   0,   0, 150, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM										).aspects(	TC.VITREUS, 2, TC.LUX, 1											), Natrium=Na.addIdenticalNames("Natrium"),
	Mg		, Magnesium					=	Mg		= alkaline		(  120,	"Magnesium"				,  12,  12,   923,  1363,  1.738	 , SET_METALLIC			, 255, 200, 200, 255).add(	G_INGOT_ORES				, FUSION, UUM										).aspects(	TC.METALLUM, 2, TC.SANO, 1											),
	Al		, Aluminium					=	Al		= posttrans		(  130,	"Aluminium"				,  13,  13,   933,  2792,  2.698	 , SET_METALLIC			, 128, 200, 240, 255).add(	G_INGOT_MACHINE_ORES		, FUSION, UUM										).aspects(	TC.METALLUM, 2, TC.VOLATUS, 1										).setQuality(10.0F, 128, 2).add(WIRES), Aluminum=Al.addIdenticalNames("Aluminum", "NaturalAluminum"),
	Si		, Silicon					=	Si		= metalloid		(  140,	"Silicon"				,  14,  14,  1687,  3538,  2.3296    , SET_METALLIC			,  60,  60,  80, 255).add(	G_INGOT_ORES				, FUSION, UUM										).aspects(	TC.METALLUM, 2, TC.TENEBRAE, 1										),
	P		, Phosphor					=	P		= polyatomic	(  150,	"Phosphor"				,  15,  15,   317,   550,  1.82      , SET_FINE				, 255, 255,   0, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM										).aspects(	TC.IGNIS, 2, TC.POTENTIA, 1											).add(FLAMMABLE, EXPLOSIVE),
	S		, Sulfur					=	S		= polyatomic	(  160,	"Sulfur"				,  16,  16,   388,   717,  2.067	 , SET_FINE				, 200, 200,   0, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM										).aspects(	TC.IGNIS, 1															).add(FLAMMABLE),
	Cl		, Chlorine					=	Cl		= diatomic		(  170,	"Chlorine"				,  17,  18,   171,   239,  0.003214  , SET_DULL				,   0, 240, 255, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM										).aspects(	TC.AQUA, 2, TC.PANNUS, 1											),
	Ar		, Argon						=	Ar		= noblegas		(  180,	"Argon"					,  18,  22,    83,    87,  0.0017837 , SET_SHINY			,   0, 255,   0,  15).add(	CONTAINERS_GAS				, FUSION, UUM										).aspects(	TC.AER, 3															),
	K		, Potassium					=	K		= alkali		(  190,	"Potassium"				,  19,  20,   336,  1032,  0.862	 , SET_ROUGH			, 250, 250, 250, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM										).aspects(	TC.VITREUS, 1, TC.POTENTIA, 1										), Kalium=K.addIdenticalNames("Kalium"),
	Ca		, Calcium					=	Ca		= alkaline		(  200,	"Calcium"				,  20,  20,  1115,  1757,  1.54      , SET_METALLIC			, 255, 245, 245, 255).add(	G_CRYSTAL_ORES				, FUSION, UUM										).aspects(	TC.SANO, 1, TC.TUTAMEN, 1											),
	Sc		, Scandium					=	Sc		= transmetal	(  210,	"Scandium"				,  21,  24,  1814,  3109,  2.989	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, FUSION, UUM										).aspects(	TC.METALLUM, 2, TC.GELUM, 1											),
	Ti		, Titanium					=	Ti		= transmetal	(  220,	"Titanium"				,  22,  26,  1941,  3560,  4.54	     , SET_METALLIC			, 220, 160, 240, 255).add(	G_INGOT_MACHINE_ORES		, FUSION, UUM										).aspects(	TC.METALLUM, 2, TC.TUTAMEN, 1										).setQuality(8.0F, 2560, 3), Titan=Ti.addIdenticalNames("Titan"),
	V		, Vanadium					=	V		= transmetal	(  230,	"Vanadium"				,  23,  28,  2183,  3680,  6.11	     , SET_METALLIC			,  50,  50,  50, 255).add(	G_INGOT_ORES				, FUSION, UUM										).aspects(	TC.METALLUM, 2, TC.ELECTRUM, 1										),
	Cr		, Chromium					=	Cr		= transmetal	(  240,	"Chromium"				,  24,  28,  2180,  2944,  7.15	     , SET_SHINY			, 255, 230, 230, 255).add(	G_INGOT_MACHINE_ORES		, FUSION, UUM										).aspects(	TC.METALLUM, 2, TC.MACHINA, 1										).setQuality(11.0F, 256, 3).add(ELECTROMAGNETIC_SEPARATION_IRON), Chrome=Cr.addIdenticalNames("Chrome"),
	Mn		, Manganese					=	Mn		= transmetal	(  250,	"Manganese"				,  25,  30,  1519,  2334,  7.44	     , SET_DULL				, 250, 250, 250, 255).add(	G_INGOT_ORES				, FUSION, UUM										).aspects(	TC.METALLUM, 3														).setQuality( 7.0F, 512, 2).add(ELECTROMAGNETIC_SEPARATION_IRON),
	Fe		, Iron						=	Fe		= transmetal	(  260,	"Iron"					,  26,  30,  1811,  3134,  7.874	 , SET_METALLIC			, 200, 200, 200, 255).add(	G_INGOT_MACHINE_ORES		, FUSION, UUM										).aspects(	TC.METALLUM, 3														).setQuality( 6.0F, 256, 2).add(WIRES),
	Co		, Cobalt					=	Co		= transmetal	(  270,	"Cobalt"				,  27,  32,  1768,  3200,  8.86	     , SET_METALLIC			,  80,  80, 250, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.INSTRUMENTUM, 1									).setQuality( 5.0F, 256, 3).add(WASHING_SODIUMPERSULFATE),
	Co_60	, Cobalt60					=	Co_60	= transmetal	(  278,	"Cobalt-60"				,  27,  43,  1768,  3200,  8.86	     , SET_SHINY			,  90,  90, 250, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.INSTRUMENTUM, 1, TC.RADIO, 1						).setQuality( 5.0F, 256, 3).add(WASHING_SODIUMPERSULFATE),
	Ni		, Nickel					=	Ni		= transmetal	(  280,	"Nickel"				,  28,  30,  1728,  3186,  8.912	 , SET_METALLIC			, 200, 200, 250, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.IGNIS, 1											).setQuality( 6.0F,  64, 2).add(WASHING_SODIUMPERSULFATE, ELECTROMAGNETIC_SEPARATION_IRON),
	Cu		, Copper					=	Cu		= transmetal	(  290,	"Copper"				,  29,  34,  1357,  2835,  8.96	     , SET_SHINY			, 255, 100,   0, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.PERMUTATIO, 1									).add(WASHING_SODIUMPERSULFATE, FURNACE, WIRES),
	Zn		, Zinc						=	Zn		= transmetal	(  300,	"Zinc"					,  30,  35,   692,  1180,  7.134	 , SET_METALLIC			, 250, 240, 240, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.SANO, 1											).add(WASHING_SODIUMPERSULFATE, FURNACE),
	Ga		, Gallium					=	Ga		= posttrans		(  310,	"Gallium"				,  31,  39,   302,  2477,  5.907	 , SET_DULL				, 220, 220, 255, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.ELECTRUM, 1										),
	Ge		, Germanium					=	Ge		= metalloid		(  320,	"Germanium"				,  32,  40,  1211,  3106,  5.323	 , SET_SHINY			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.ELECTRUM, 1										),
	As		, Arsenic					=	As		= metalloid		(  330,	"Arsenic"				,  33,  42,   887,  1090,  5.776	 , SET_SHINY			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 1, TC.VENENUM, 2										),
	Se		, Selenium					=	Se		= polyatomic	(  340,	"Selenium"				,  34,  45,   453,   958,  4.809	 , SET_DULL				, 256, 256, 256, 255).add(	G_CRYSTAL_ORES				, UUM												).aspects(	TC.VITREUS, 1, TC.SPIRITUS, 2										),
	Br		, Bromine					=	Br		= diatomic		(  350,	"Bromine"				,  35,  45,   265,   332,  3.122	 , SET_DULL				, 256, 256, 256, 255).add(	G_CRYSTAL_ORES				, UUM												).aspects(	TC.VITREUS, 1, TC.TERRA, 2											),
	Kr		, Krypton					=	Kr		= noblegas		(  360,	"Krypton"				,  36,  48,   115,   119,  0.003733  , SET_DIAMOND			, 128, 255, 128,  15).add(	CONTAINERS_GAS				, UUM												).aspects(	TC.AER, 3															),
	Rb		, Rubidium					=	Rb		= alkali		(  370,	"Rubidium"				,  37,  48,   312,   961,  1.532	 , SET_SHINY			, 240,  30,  30, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.VITREUS, 1										),
	Sr		, Strontium					=	Sr		= alkaline		(  380,	"Strontium"				,  38,  49,  1050,  1655,  2.64	     , SET_METALLIC			, 200, 200, 200, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.STRONTIO, 1										),
	Y		, Yttrium					=	Y		= transmetal	(  390,	"Yttrium"				,  39,  50,  1799,  3609,  4.469	 , SET_METALLIC			, 220, 250, 220, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3 														),
	Zr		, Zirconium					=	Zr		= transmetal	(  400,	"Zirconium"				,  40,  51,  2128,  4682,  6.506	 , SET_DIAMOND			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.VITREUS, 1										),
	Nb		, Niobium					=	Nb		= transmetal	(  410,	"Niobium"				,  41,  53,  2750,  5017,  8.57	     , SET_METALLIC			, 190, 180, 200, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.IGNIS, 1											),
	Mo		, Molybdenum				=	Mo		= transmetal	(  420,	"Molybdenum"			,  42,  53,  2896,  4912, 10.22	     , SET_SHINY			, 180, 180, 220, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.INSTRUMENTUM, 1									).setQuality(7.0F, 512, 2),
	Tc		, Technetium				=	Tc		= transmetal	(  430,	"Technetium"			,  43,  55,  2430,  4538, 11.5	     , SET_SHINY			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											), Gregorium=Tc.addIdenticalNames("Gregorium"),
	Ru		, Ruthenium					=	Ru		= transmetal	(  440,	"Ruthenium"				,  44,  57,  2607,  4423, 12.37	     , SET_SHINY			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3 														),
	Rh		, Rhodium					=	Rh		= transmetal	(  450,	"Rhodium"				,  45,  58,  2237,  3968, 12.41	     , SET_SHINY			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3 														),
	Pd		, Palladium					=	Pd		= transmetal	(  460,	"Palladium"				,  46,  60,  1828,  3236, 12.02	     , SET_SHINY			, 128, 128, 128, 255).add(	G_INGOT_MACHINE_ORES		, UUM												).aspects(	TC.METALLUM, 3 														).setQuality(8.0F, 512, 2),
	Ag		, Silver					=	Ag		= transmetal	(  470,	"Silver"				,  47,  60,  1234,  2435, 10.501	 , SET_SHINY			, 220, 220, 255, 255).add(	G_INGOT_MACHINE_ORES		, UUM												).aspects(	TC.METALLUM, 2, TC.LUCRUM, 1										).setQuality(10.0F, 64, 2).add(WIRES),
	Cd		, Cadmium					=	Cd		= transmetal	(  480,	"Cadmium"				,  48,  64,   594,  1040,  8.69	     , SET_SHINY			,  50,  50,  60, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 1, TC.POTENTIA, 1, TC.VENENUM, 1						),
	In		, Indium					=	In		= posttrans		(  490,	"Indium"				,  49,  65,   429,  2345,  7.31	     , SET_SHINY			,  64,   0, 128, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Sn		, Tin						=	Sn		= posttrans		(  500,	"Tin"					,  50,  68,   505,  2875,  7.287	 , SET_DULL				, 220, 220, 220, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.VITREUS, 1										).add(SOLDERING_MATERIAL, ELECTROMAGNETIC_SEPARATION_IRON, FURNACE, WIRES),
	Sb		, Antimony					=	Sb		= metalloid		(  510,	"Antimony"				,  51,  70,   903,  1860,  6.685	 , SET_SHINY			, 220, 220, 240, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.AQUA, 1											).add(ELECTROMAGNETIC_SEPARATION_IRON),
	Te		, Tellurium					=	Te		= metalloid		(  520,	"Tellurium"				,  52,  75,   722,  1261,  6.232	 , SET_SHINY			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.ELECTRUM, 1										),
	I		, Iodine					=	I		= diatomic		(  530,	"Iodine"				,  53,  74,   386,   457,  4.93	     , SET_DULL				, 255, 240, 240, 255).add(	G_CRYSTAL_ORES				, UUM												).aspects(	TC.VITREUS, 2, TC.AQUA, 1											),
	Xe		, Xenon						=	Xe		= noblegas		(  540,	"Xenon"					,  54,  77,   161,   165,  0.005887  , SET_DULL				,   0, 255, 255,  15).add(	CONTAINERS_GAS				, UUM												).aspects(	TC.AER, 3															),
	Cs		, Caesium					=	Cs		= alkali		(  550,	"Caesium"				,  55,  77,   301,   944,  1.873	 , SET_SHINY			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Ba		, Barium					=	Ba		= alkaline		(  560,	"Barium"				,  56,  81,  1000,  2170,  3.594	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.VINCULUM, 3														),
	La		, Lantanium					=	La		= lanthanide	(  570,	"Lantanium"				,  57,  81,  1193,  3737,  6.145	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Ce		, Cerium					=	Ce		= lanthanide	(  580,	"Cerium"				,  58,  82,  1068,  3716,  6.77      , SET_SHINY			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Pr		, Praseodymium				=	Pr		= lanthanide	(  590,	"Praseodymium"			,  59,  81,  1208,  3793,  6.773	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Nd		, Neodymium					=	Nd		= lanthanide	(  600,	"Neodymium"				,  60,  84,  1297,  3347,  7.007	 , SET_SHINY			, 100, 100, 100, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.MAGNETO, 2										).setQuality(7.0F, 512, 2),
	Pm		, Promethium				=	Pm		= lanthanide	(  610,	"Promethium"			,  61,  83,  1315,  3273,  7.26	     , SET_SHINY			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Sm		, Samarium					=	Sm		= lanthanide	(  620,	"Samarium"				,  62,  88,  1345,  2067,  7.52	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Eu		, Europium					=	Eu		= lanthanide	(  630,	"Europium"				,  63,  88,  1099,  1802,  5.243	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Gd		, Gadolinium				=	Gd		= lanthanide	(  640,	"Gadolinium"			,  64,  93,  1585,  3546,  7.895	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Tb		, Terbium					=	Tb		= lanthanide	(  650,	"Terbium"				,  65,  93,  1629,  3503,  8.229	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Dy		, Dysprosium				=	Dy		= lanthanide	(  660,	"Dysprosium"			,  66,  96,  1680,  2840,  8.55	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Ho		, Holmium					=	Ho		= lanthanide	(  670,	"Holmium"				,  67,  97,  1734,  2993,  8.795	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Er		, Erbium					=	Er		= lanthanide	(  680,	"Erbium"				,  68,  99,  1802,  3141,  9.066	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Tm		, Thulium					=	Tm		= lanthanide	(  690,	"Thulium"				,  69,  99,  1818,  2223,  9.321	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Yb		, Ytterbium					=	Yb		= lanthanide	(  700,	"Ytterbium"				,  70, 103,  1097,  1469,  6.965	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Lu		, Lutetium					=	Lu		= lanthanide	(  710,	"Lutetium"				,  71, 103,  1925,  3675,  9.84      , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Hf		, Hafnium					=	Hf		= transmetal	(  720,	"Hafnium"				,  72, 106,  2506,  4876, 13.31      , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Ta		, Tantalum					=	Ta		= transmetal	(  730,	"Tantalum"				,  73, 107,  3290,  5731, 16.654	 , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 2, TC.VINCULUM, 1										),
	W		, Tungsten					=	W		= transmetal	(  740,	"Tungsten"				,  74, 109,  3695,  5828, 19.25	     , SET_METALLIC			,  50,  50,  50, 255).add(	G_INGOT_MACHINE_ORES		, UUM												).aspects(	TC.METALLUM, 3, TC.TUTAMEN, 1										).setQuality(8.0F, 5120, 3).add(WIRES), Wolframium=W.addIdenticalNames("Wolframium"), Wolfram=W.addIdenticalNames("Wolfram"),
	Re		, Rhenium					=	Re		= transmetal	(  750,	"Rhenium"				,  75, 111,  3459,  5869, 21.02	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Os		, Osmium					=	Os		= transmetal	(  760,	"Osmium"				,  76, 114,  3306,  5285, 22.61	     , SET_METALLIC			,  50,  50, 255, 255).add(	G_INGOT_MACHINE_ORES		, UUM												).aspects(	TC.METALLUM, 2, TC.MACHINA, 1, TC.NEBRISUM, 1						).setQuality(16.0F, 1280, 4).add(WIRES),
	Ir		, Iridium					=	Ir		= transmetal	(  770,	"Iridium"				,  77, 115,  2719,  4701, 22.56	     , SET_DULL				, 240, 240, 245, 255).add(	G_INGOT_MACHINE_ORES		, UUM												).aspects(	TC.METALLUM, 2, TC.MACHINA, 1										).setQuality(6.0F, 5120, 4).setRGBaLiquid(255, 128, 200, 255),
	Pt		, Platinum					=	Pt		= transmetal	(  780,	"Platinum"				,  78, 117,  2041,  4098, 21.46	     , SET_SHINY			, 255, 255, 200, 255).add(	G_INGOT_MACHINE_ORES		, UUM												).aspects(	TC.METALLUM, 2, TC.NEBRISUM, 1										).setQuality(15.0F, 64, 2).add(WASHING_MERCURY, WIRES),
	Au		, Gold						=	Au		= transmetal	(  790,	"Gold"					,  79, 117,  1337,  3129, 19.282	 , SET_SHINY			, 255, 230,  80, 255).add(	G_INGOT_MACHINE_ORES		, UUM												).aspects(	TC.METALLUM, 2, TC.LUCRUM, 2										).setQuality(12.5F, 64, 2).add(WASHING_MERCURY, WIRES),
	Hg		, Mercury					=	Hg		= transmetal	(  800,	"Mercury"				,  80, 120,   234,   629, 13.5336    , SET_SHINY			, 230, 220, 220, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 1, TC.AQUA, 1, TC.VENENUM, 1							).add(FURNACE), Quicksilver=Hg.addIdenticalNames("Quicksilver").addIdenticalNames("QuickSilver"),
	Tl		, Thallium					=	Tl		= posttrans		(  810,	"Thallium"				,  81, 123,   577,  1746, 11.85	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES				, UUM												).aspects(	TC.METALLUM, 3														),
	Pb		, Lead						=	Pb		= posttrans		(  820,	"Lead"					,  82, 125,   600,  2022, 11.342     , SET_DULL				, 140, 100, 140, 255).add(	G_INGOT_MACHINE_ORES		, UUM												).aspects(	TC.METALLUM, 2, TC.ORDO, 1											).setQuality(8.0F, 64, 1).add(SOLDERING_MATERIAL, SOLDERING_MATERIAL_BAD, FURNACE, WIRES),
	Bi		, Bismuth					=	Bi		= posttrans		(  830,	"Bismuth"				,  83, 125,   544,  1837,  9.807     , SET_METALLIC			, 100, 160, 160, 255).add(	G_INGOT_MACHINE_ORES															).aspects(	TC.METALLUM, 2, TC.RADIO, 1											).setQuality(6.0F, 64, 1).add(FURNACE),
	Po		, Polonium					=	Po		= posttrans		(  840,	"Polonium"				,  84, 124,   527,  1235,  9.32	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	At		, Astatine					=	At		= metalloid		(  850,	"Astatine"				,  85, 124,   575,   610,  7.0	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Rn		, Radon						=	Rn		= noblegas		(  860,	"Radon"					,  86, 134,   202,   211,  0.00973   , SET_DULL				, 255,   0, 255,  15).add(	CONTAINERS_GAS																	).aspects(	TC.AER, 2, TC.RADIO, 1												),
	Fr		, Francium					=	Fr		= alkali		(  870,	"Francium"				,  87, 134,   300,   950,  1.87      , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Ra		, Radium					=	Ra		= alkaline		(  880,	"Radium"				,  88, 136,   973,  2010,  5.5	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Ac		, Actinium					=	Ac		= actinide		(  890,	"Actinium"				,  89, 136,  1323,  3471, 10.07      , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Th		, Thorium					=	Th		= actinide		(  900,	"Thorium"				,  90, 140,  2115,  5061, 11.72      , SET_SHINY			,   0,  30,   0, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											).setQuality(6.0F, 512, 2),
	Pa		, Protactinium				=	Pa		= actinide		(  910,	"Protactinium"			,  91, 138,  1841,  4300, 15.37      , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	U_238	, Uranium					=	U_238	= actinide		(  920,	"Uranium"				,  92, 146,  1405,  4404, 18.95      , SET_METALLIC			,  50, 240,  50, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											).setQuality(6.0F, 512, 3), Uranium238=U_238.addIdenticalNames("Uranium238"), Uran=U_238.addIdenticalNames("Uran"), // No, I could not make a Variable named "U" here. That would collide with the Material Unit.
	U_235	, Uranium235				=	U_235	= actinide		(  921,	"Uranium-235"			,  92, 143,  1405,  4404, 18.95      , SET_SHINY			,  70, 250,  70, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											).setQuality(6.0F, 512, 3),
	Np		, Neptunium					=	Np		= actinide		(  930,	"Neptunium"				,  93, 144,   917,  4273, 20.45      , SET_DULL				, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Pu		, Plutonium					=	Pu		= actinide		(  940,	"Plutonium"				,  94, 150,   912,  3501, 19.84      , SET_METALLIC			, 240,  50,  50, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											).setQuality(6.0F, 512, 3), Plutonium244=Pu.addIdenticalNames("Plutonium244"),
	Pu_241	, Plutonium241				=	Pu_241	= actinide		(  943,	"Plutonium-241"			,  94, 147,   912,  3501, 19.84      , SET_SHINY			, 245,  70,  70, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 3											).setQuality(6.0F, 512, 3),
	Pu_243	, Plutonium243				=	Pu_243	= actinide		(  945,	"Plutonium-243"			,  94, 149,   912,  3501, 19.84      , SET_SHINY			, 250,  70,  70, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											).setQuality(6.0F, 512, 3),
	Am		, Americium					=	Am		= actinide		(  950,	"Americium"				,  95, 150,  1449,  2880, 13.69      , SET_METALLIC			, 200, 200, 200, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											).setQuality(4.0F, 256, 2),
	Am_241	, Americium241				=	Am_241	= actinide		(  951,	"Americium-241"			,  95, 146,  1449,  2880, 13.69      , SET_SHINY			, 210, 210, 210, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 3											).setQuality(4.0F, 256, 2),
	Cm		, Curium					=	Cm		= actinide		(  960,	"Curium"				,  96, 153,  1613,  3383, 13.51      , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Bk		, Berkelium					=	Bk		= actinide		(  970,	"Berkelium"				,  97, 152,  1259,  2900, 14.79      , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Cf		, Californium				=	Cf		= actinide		(  980,	"Californium"			,  98, 153,  1173,  1743, 15.1	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Es		, Einsteinium				=	Es		= actinide		(  990,	"Einsteinium"			,  99, 153,  1133,  1269,  8.84      , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 2, TC.RADIO, 1											),
	Fm		, Fermium					=	Fm		= actinide		( 1000,	"Fermium"				, 100, 157,  1125, 3000 ,		    0, SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Md		, Mendelevium				=	Md		= actinide		( 1010,	"Mendelevium"			, 101, 157,  1100, 3000 ,		    0, SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	No		, Nobelium					=	No		= actinide		( 1020,	"Nobelium"				, 102, 157,  1100, 3000 ,		    0, SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Lr		, Lawrencium				=	Lr		= actinide		( 1030,	"Lawrencium"			, 103, 159,  1900, 3000 ,		    0, SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Rf		, Rutherfordium				=	Rf		= transmetal	( 1040,	"Rutherfordium"			, 104, 161,  2400,  5800, 23.2	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Db		, Dubnium					=	Db		= transmetal	( 1050,	"Dubnium"				, 105, 163, 1000 , 3000 , 29.3	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Sg		, Seaborgium				=	Sg		= transmetal	( 1060,	"Seaborgium"			, 106, 165, 1000 , 3000 , 35.0	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Bh		, Bohrium					=	Bh		= transmetal	( 1070,	"Bohrium"				, 107, 163, 1000 , 3000 , 37.1	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Hs		, Hassium					=	Hs		= transmetal	( 1080,	"Hassium"				, 108, 169, 1000 , 3000 , 40.7	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Mt		, Meitnerium				=	Mt		= element		( 1090,	"Meitnerium"			, 109, 167, 1000 , 3000 , 37.4	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Ds		, Darmstadtium				=	Ds		= element		( 1100,	"Darmstadtium"			, 110, 171, 1000 , 3000 , 34.8	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Rg		, Roentgenium				=	Rg		= element		( 1110,	"Roentgenium"			, 111, 169, 1000 , 3000 , 28.7	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Cn		, Copernicium				=	Cn		= transmetal	( 1120,	"Copernicium"			, 112, 173,  150 ,   357, 23.7	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Uut		, Ununtrium					=	Uut		= element		( 1130,	"Ununtrium"				, 113, 171,   700,  1400, 16.0	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Fl		, Flerovium					=	Fl		= posttrans		( 1140,	"Flerovium"				, 114, 175,   340,   420, 14.0	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 3											),
	Fl_298	, Flerovium298				=	Fl_298	= posttrans		( 1148,	"Flerovium-298"			, 114, 184,   340,   420, 14.0	     , SET_SHINY			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Uup		, Ununpentium				=	Uup		= element		( 1150,	"Ununpentium"			, 115, 174,   700,  1400, 13.5	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Lv		, Livermorium				=	Lv		= element		( 1160,	"Livermorium"			, 116, 177,   708,  1085, 12.9	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Fa		, Farnsium					=	Fa		= element		( 1170,	"Farnsium"				, 117, 177,   673,   823,  7.2	     , SET_SHINY			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											), Uus=Fa, Ununseptium=Fa.addIdenticalNames("Ununseptium"),
	Uuo		, Ununoctium				=	Uuo		= element		( 1180,	"Ununoctium"			, 118, 176,   258,   263,  5.0	     , SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Uue		, Ununennium				=	Uue		= element		( 1190,	"Ununennium"			, 119, 178,   290,   903,		    0, SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	Ubn		, Unbinilium				=	Ubn		= element		( 1200,	"Unbinilium"			, 120, 180,   953,  1973,		    0, SET_METALLIC			, 256, 256, 256, 255).add(	G_INGOT_ORES																	).aspects(	TC.METALLUM, 1, TC.RADIO, 2											),
	
	// The following Data is in no way reliable, but it is there in case someone wants to use it.
	Ubu		, Unbiunium					=	Ubu		= element		( 1210,	"Unbiunium"				, 121, 182,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ubb		, Unbibium					=	Ubb		= element		( 1220,	"Unbibium"				, 122, 184,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ubt		, Unbitrium					=	Ubt		= element		( 1230,	"Unbitrium"				, 123, 186,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ubq		, Unbiquadium				=	Ubq		= element		( 1240,	"Unbiquadium"			, 124, 188,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ubp		, Unbipentium				=	Ubp		= element		( 1250,	"Unbipentium"			, 125, 190,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ubh		, Unbihexium				=	Ubh		= element		( 1260,	"Unbihexium"			, 126, 192,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ubs		, Unbiseptium				=	Ubs		= element		( 1270,	"Unbiseptium"			, 127, 194,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ubo		, Unbioctium				=	Ubo		= element		( 1280,	"Unbioctium"			, 128, 196,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ube		, Unbiennium				=	Ube		= element		( 1290,	"Unbiennium"			, 129, 198,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Utn		, Untrinilium				=	Utn		= element		( 1300,	"Untrinilium"			, 130, 200,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Utu		, Untriunium				=	Utu		= element		( 1310,	"Untriunium"			, 131, 203,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Utb		, Untribium					=	Utb		= element		( 1320,	"Untribium"				, 132, 206,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Utt		, Untritrium				=	Utt		= element		( 1330,	"Untritrium"			, 133, 209,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Utq		, Untriquadium				=	Utq		= element		( 1340,	"Untriquadium"			, 134, 212,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Utp		, Untripentium				=	Utp		= element		( 1350,	"Untripentium"			, 135, 215,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uth		, Untrihexium				=	Uth		= element		( 1360,	"Untrihexium"			, 136, 218,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uts		, Untriseptium				=	Uts		= element		( 1370,	"Untriseptium"			, 137, 221,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uto		, Untrioctium				=	Uto		= element		( 1380,	"Untrioctium"			, 138, 224,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ute		, Untriennium				=	Ute		= element		( 1390,	"Untriennium"			, 139, 227,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uqn		, Unquadnilium				=	Uqn		= element		( 1400,	"Unquadnilium"			, 140, 230,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uqu		, Unquadunium				=	Uqu		= element		( 1410,	"Unquadunium"			, 141, 233,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uqb		, Unquadbium				=	Uqb		= element		( 1420,	"Unquadbium"			, 142, 236,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uqt		, Unquadtrium				=	Uqt		= element		( 1430,	"Unquadtrium"			, 143, 239,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uqq		, Unquadquadium				=	Uqq		= element		( 1440,	"Unquadquadium"			, 144, 242,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uqp		, Unquadpentium				=	Uqp		= element		( 1450,	"Unquadpentium"			, 145, 245,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uqh		, Unquadhexium				=	Uqh		= element		( 1460,	"Unquadhexium"			, 146, 248,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uqs		, Unquadseptium				=	Uqs		= element		( 1470,	"Unquadseptium"			, 147, 251,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uqo		, Unquadoctium				=	Uqo		= element		( 1480,	"Unquadoctium"			, 148, 254,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uqe		, Unquadennium				=	Uqe		= element		( 1490,	"Unquadennium"			, 149, 257,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Upn		, Unpentnilium				=	Upn		= element		( 1500,	"Unpentnilium"			, 150, 260,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Upu		, Unpentunium				=	Upu		= element		( 1510,	"Unpentunium"			, 151, 263,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Vb		, Vibranium					=	Vb		= element		( 1520,	"Vibranium"				, 152, 266,  4852,  9415,  3.23978365, SET_EMERALD			, 200, 128, 255, 100).add(	G_GEM_ORES_TRANSPARENT, GLOWING													).aspects(TC.VITREUS, 10, TC.SENSUS, 10), Upb = Vb, Unpentbium = Vb.addIdenticalNames("Unpentbium").setQuality(1000.0F, 512, 15),
	Upt		, Unpenttrium				=	Upt		= element		( 1530,	"Unpenttrium"			, 153, 269,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Upq		, Unpentquadium				=	Upq		= element		( 1540,	"Unpentquadium"			, 154, 272,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Upp		, Unpentpentium				=	Upp		= element		( 1550,	"Unpentpentium"			, 155, 276,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uph		, Unpenthexium				=	Uph		= element		( 1560,	"Unpenthexium"			, 156, 280,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ups		, Unpentseptium				=	Ups		= element		( 1570,	"Unpentseptium"			, 157, 284,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Upo		, Unpentoctium				=	Upo		= element		( 1580,	"Unpentoctium"			, 158, 288,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Upe		, Unpentennium				=	Upe		= element		( 1590,	"Unpentennium"			, 159, 292,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uhn		, Unhexnilium				=	Uhn		= element		( 1600,	"Unhexnilium"			, 160, 296,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uhu		, Unhexunium				=	Uhu		= element		( 1610,	"Unhexunium"			, 161, 300,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uhb		, Unhexbium					=	Uhb		= element		( 1620,	"Unhexbium"				, 162, 304,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uht		, Unhextrium				=	Uht		= element		( 1630,	"Unhextrium"			, 163, 308,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uhq		, Unhexquadium				=	Uhq		= element		( 1640,	"Unhexquadium"			, 164, 312,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uhp		, Unhexpentium				=	Uhp		= element		( 1650,	"Unhexpentium"			, 165, 316,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uhh		, Unhexhexium				=	Uhh		= element		( 1660,	"Unhexhexium"			, 166, 320,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uhs		, Unhexseptium				=	Uhs		= element		( 1670,	"Unhexseptium"			, 167, 324,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uho		, Unhexoctium				=	Uho		= element		( 1680,	"Unhexoctium"			, 168, 328,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uhe		, Unhexennium				=	Uhe		= element		( 1690,	"Unhexennium"			, 169, 332,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Usn		, Unseptnilium				=	Usn		= element		( 1700,	"Unseptnilium"			, 170, 336,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Usu		, Unseptunium				=	Usu		= element		( 1710,	"Unseptunium"			, 171, 340,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Usb		, Unseptbium				=	Usb		= element		( 1720,	"Unseptbium"			, 172, 344,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ust		, Unsepttrium				=	Ust		= element		( 1730,	"Unsepttrium"			, 173, 348,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Nq		, Naquadah					=	Nq		= element		( 1740,	"Naquadah"				, 174, 352,   500,  2000, 21.0	     , SET_METALLIC			,  50,  50,  50, 255).add(	G_INGOT_MACHINE_ORES, SMITHABLE, MELTING										).aspects(TC.METALLUM, 3, TC.RADIO, 1, TC.NEBRISUM, 1).setRGBaLiquid(  0, 255,   0, 255), Usq = Nq, Unseptquadium = Nq.addIdenticalNames("Unseptquadium").setQuality(6.0F, 1280, 4),
	Nq_528	, NaquadahEnriched			=	Nq_528	= element		( 1741,	"Naquadah-Enriched"		, 174, 354,   500,  2000, 22.0	     , SET_METALLIC			,  60,  60,  60, 255).add(	G_INGOT_ORES, SMITHABLE, MELTING, EXPLOSIVE										).aspects(TC.METALLUM, 3, TC.RADIO, 2, TC.NEBRISUM, 2).setRGBaLiquid( 64, 255,  64, 255).setLocal("Enriched Naquadah").setQuality(6.0F, 1280, 4),
	Nq_522	, Naquadria					=	Nq_522	= element		( 1742,	"Naquadria"				, 174, 348,   500,  2000, 20.0	     , SET_SHINY			,  30,  30,  30, 255).add(	G_INGOT_ORES, SMITHABLE, MELTING, EXPLOSIVE										).aspects(TC.METALLUM, 4, TC.RADIO, 3, TC.NEBRISUM, 3).setRGBaLiquid(128, 255, 128, 255).setQuality(1.0F, 512, 4),
	Usp		, Unseptpentium				=	Usp		= element		( 1750,	"Unseptpentium"			, 175, 356,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ush		, Unsepthexium				=	Ush		= element		( 1760,	"Unsepthexium"			, 176, 360,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uss		, Unseptseptium				=	Uss		= element		( 1770,	"Unseptseptium"			, 177, 364,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uso		, Unseptoctium				=	Uso		= element		( 1780,	"Unseptoctium"			, 178, 368,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Use		, Unseptennium				=	Use		= element		( 1790,	"Unseptennium"			, 179, 372,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uon		, Unoctnilium				=	Uon		= element		( 1800,	"Unoctnilium"			, 180, 376,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uou		, Unoctunium				=	Uou		= element		( 1810,	"Unoctunium"			, 181, 380,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uob		, Unoctbium					=	Uob		= element		( 1820,	"Unoctbium"				, 182, 384,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uot		, Unocttrium				=	Uot		= element		( 1830,	"Unocttrium"			, 183, 388,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uoq		, Unoctquadium				=	Uoq		= element		( 1840,	"Unoctquadium"			, 184, 392,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uop		, Unoctpentium				=	Uop		= element		( 1850,	"Unoctpentium"			, 185, 396,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uoh		, Unocthexium				=	Uoh		= element		( 1860,	"Unocthexium"			, 186, 400,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uos		, Unoctseptium				=	Uos		= element		( 1870,	"Unoctseptium"			, 187, 405,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uoo		, Unoctoctium				=	Uoo		= element		( 1880,	"Unoctoctium"			, 188, 410,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uoe		, Unoctennium				=	Uoe		= element		( 1890,	"Unoctennium"			, 189, 415,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uen		, Unennilium				=	Uen		= element		( 1900,	"Unennilium"			, 190, 420,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ueu		, Unennunium				=	Ueu		= element		( 1910,	"Unennunium"			, 191, 425,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ueb		, Unennbium					=	Ueb		= element		( 1920,	"Unennbium"				, 192, 430,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uet		, Unenntrium				=	Uet		= element		( 1930,	"Unenntrium"			, 193, 435,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ueq		, Unennquadium				=	Ueq		= element		( 1940,	"Unennquadium"			, 194, 440,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uep		, Unennpentium				=	Uep		= element		( 1950,	"Unennpentium"			, 195, 445,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ueh		, Unennhexium				=	Ueh		= element		( 1960,	"Unennhexium"			, 196, 450,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ues		, Unennseptium				=	Ues		= element		( 1970,	"Unennseptium"			, 197, 455,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ueo		, Unennoctium				=	Ueo		= element		( 1980,	"Unennoctium"			, 198, 460,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Uee		, Unennennium				=	Uee		= element		( 1990,	"Unennennium"			, 199, 465,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bnn		, Binilnilium				=	Bnn		= element		( 2000,	"Binilnilium"			, 200, 470,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bnu		, Binilunium				=	Bnu		= element		( 2010,	"Binilunium"			, 201, 475,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bnb		, Binilbium					=	Bnb		= element		( 2020,	"Binilbium"				, 202, 480,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bnt		, Biniltrium				=	Bnt		= element		( 2030,	"Biniltrium"			, 203, 485,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bnq		, Binilquadium				=	Bnq		= element		( 2040,	"Binilquadium"			, 204, 490,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bnp		, Binilpentium				=	Bnp		= element		( 2050,	"Binilpentium"			, 205, 495,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bnh		, Binilhexium				=	Bnh		= element		( 2060,	"Binilhexium"			, 206, 500,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bns		, Binilseptium				=	Bns		= element		( 2070,	"Binilseptium"			, 207, 505,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bno		, Biniloctium				=	Bno		= element		( 2080,	"Biniloctium"			, 208, 510,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bne		, Binilennium				=	Bne		= element		( 2090,	"Binilennium"			, 209, 515,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bun		, Biunnilium				=	Bun		= element		( 2100,	"Biunnilium"			, 210, 520,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Buu		, Biununium					=	Buu		= element		( 2110,	"Biununium"				, 211, 525,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bub		, Biunbium					=	Bub		= element		( 2120,	"Biunbium"				, 212, 530,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	But		, Biuntrium					=	But		= element		( 2130,	"Biuntrium"				, 213, 535,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Buq		, Biunquadium				=	Buq		= element		( 2140,	"Biunquadium"			, 214, 540,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bup		, Biunpentium				=	Bup		= element		( 2150,	"Biunpentium"			, 215, 545,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Buh		, Biunhexium				=	Buh		= element		( 2160,	"Biunhexium"			, 216, 550,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bus		, Biunseptium				=	Bus		= element		( 2170,	"Biunseptium"			, 217, 555,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Buo		, Biunoctium				=	Buo		= element		( 2180,	"Biunoctium"			, 218, 560,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bue		, Biunennium				=	Bue		= element		( 2190,	"Biunennium"			, 219, 565,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bbn		, Bibinilium				=	Bbn		= element		( 2200,	"Bibinilium"			, 220, 570,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bbu		, Bibiunium					=	Bbu		= element		( 2210,	"Bibiunium"				, 221, 575,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ad		, Adamantium				= 	Ad		= element		( 2220,	"Adamantium"			, 222, 580,  5425, 14528, 13.35624762, SET_SHINY			, 255, 255, 255, 255).add(G_INGOT_MACHINE_ORES, SMITHABLE, MELTING, MAGICAL									).aspects(TC.METALLUM, 10, TC.PRAECANTIO, 10).setQuality(10.0F, 5120, 5).addIdenticalNames("Adamantine", "Adamant"), Bbb = Ad, Bibibium = Ad.addIdenticalNames("Bibibium"),
	Bbt		, Bibitrium					=	Bbt		= element		( 2230,	"Bibitrium"				, 223, 585,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bbq		, Bibiquadium				=	Bbq		= element		( 2240,	"Bibiquadium"			, 224, 590,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bbp		, Bibipentium				=	Bbp		= element		( 2250,	"Bibipentium"			, 225, 595,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bbh		, Bibihexium				=	Bbh		= element		( 2260,	"Bibihexium"			, 226, 600,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bbs		, Bibiseptium				=	Bbs		= element		( 2270,	"Bibiseptium"			, 227, 605,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bbo		, Bibioctium				=	Bbo		= element		( 2280,	"Bibioctium"			, 228, 610,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bbe		, Bibiennium				=	Bbe		= element		( 2290,	"Bibiennium"			, 229, 615,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Btn		, Bitrinilium				=	Btn		= element		( 2300,	"Bitrinilium"			, 230, 620,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Btu		, Bitriunium				=	Btu		= element		( 2310,	"Bitriunium"			, 231, 625,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Btb		, Bitribium					=	Btb		= element		( 2320,	"Bitribium"				, 232, 630,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Btt		, Bitritrium				=	Btt		= element		( 2330,	"Bitritrium"			, 233, 635,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Btq		, Bitriquadium				=	Btq		= element		( 2340,	"Bitriquadium"			, 234, 640,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Btp		, Bitripentium				=	Btp		= element		( 2350,	"Bitripentium"			, 235, 645,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bth		, Bitrihexium				=	Bth		= element		( 2360,	"Bitrihexium"			, 236, 650,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bts		, Bitriseptium				=	Bts		= element		( 2370,	"Bitriseptium"			, 237, 655,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bto		, Bitrioctium				=	Bto		= element		( 2380,	"Bitrioctium"			, 238, 660,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Mcg		, MacGuffium				=	Mcg		= element		( 2390,	"Mac-Guffium"			, 239, 665,   200,  1000,  3.122     , SET_SHINY			, 200,  50, 150, 255).add(	CONTAINERS																		).aspects(TC.ALIENIS, 8, TC.PERMUTATIO, 8, TC.SPIRITUS, 8, TC.AURAM, 8, TC.VITIUM, 8, TC.RADIO, 8, TC.MAGNETO, 8, TC.ELECTRUM, 8, TC.NEBRISUM, 8, TC.STRONTIO, 8), Bte = Mcg, Bitriennium = Mcg.addIdenticalNames("Bitriennium"),
	Bqn		, Biquadnilium				=	Bqn		= element		( 2400,	"Biquadnilium"			, 240, 670,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bqu		, Biquadunium				=	Bqu		= element		( 2410,	"Biquadunium"			, 241, 675,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bqb		, Biquadbium				=	Bqb		= element		( 2420,	"Biquadbium"			, 242, 680,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bqt		, Biquadtrium				=	Bqt		= element		( 2430,	"Biquadtrium"			, 243, 685,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bqq		, Biquadquadium				=	Bqq		= element		( 2440,	"Biquadquadium"			, 244, 690,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bqp		, Biquadpentium				=	Bqp		= element		( 2450,	"Biquadpentium"			, 245, 695,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bqh		, Biquadhexium				=	Bqh		= element		( 2460,	"Biquadhexium"			, 246, 700,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bqs		, Biquadseptium				=	Bqs		= element		( 2470,	"Biquadseptium"			, 247, 705,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bqo		, Biquadoctium				=	Bqo		= element		( 2480,	"Biquadoctium"			, 248, 710,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bqe		, Biquadennium				=	Bqe		= element		( 2490,	"Biquadennium"			, 249, 715,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bpn		, Bipentnilium				=	Bpn		= element		( 2500,	"Bipentnilium"			, 250, 720,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bpu		, Bipentunium				=	Bpu		= element		( 2510,	"Bipentunium"			, 251, 725,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bpb		, Bipentbium				=	Bpb		= element		( 2520,	"Bipentbium"			, 252, 730,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bpt		, Bipenttrium				=	Bpt		= element		( 2530,	"Bipenttrium"			, 253, 735,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bpq		, Bipentquadium				=	Bpq		= element		( 2540,	"Bipentquadium"			, 254, 740,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bpp		, Bipentpentium				=	Bpp		= element		( 2550,	"Bipentpentium"			, 255, 745,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bph		, Bipenthexium				=	Bph		= element		( 2560,	"Bipenthexium"			, 256, 750,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bps		, Bipentseptium				=	Bps		= element		( 2570,	"Bipentseptium"			, 257, 755,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bpo		, Bipentoctium				=	Bpo		= element		( 2580,	"Bipentoctium"			, 258, 760,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bpe		, Bipentennium				=	Bpe		= element		( 2590,	"Bipentennium"			, 259, 765,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bhn		, Bihexnilium				=	Bhn		= element		( 2600,	"Bihexnilium"			, 260, 770,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bhu		, Bihexunium				=	Bhu		= element		( 2610,	"Bihexunium"			, 261, 775,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bhb		, Bihexbium					=	Bhb		= element		( 2620,	"Bihexbium"				, 262, 780,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bht		, Bihextrium				=	Bht		= element		( 2630,	"Bihextrium"			, 263, 785,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bhq		, Bihexquadium				=	Bhq		= element		( 2640,	"Bihexquadium"			, 264, 790,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bhp		, Bihexpentium				=	Bhp		= element		( 2650,	"Bihexpentium"			, 265, 795,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bhh		, Bihexhexium				=	Bhh		= element		( 2660,	"Bihexhexium"			, 266, 800,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bhs		, Bihexseptium				=	Bhs		= element		( 2670,	"Bihexseptium"			, 267, 805,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bho		, Bihexoctium				=	Bho		= element		( 2680,	"Bihexoctium"			, 268, 810,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bhe		, Bihexennium				=	Bhe		= element		( 2690,	"Bihexennium"			, 269, 815,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bsn		, Biseptnilium				=	Bsn		= element		( 2700,	"Biseptnilium"			, 270, 820,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bsu		, Biseptunium				=	Bsu		= element		( 2710,	"Biseptunium"			, 271, 825,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bsb		, Biseptbium				=	Bsb		= element		( 2720,	"Biseptbium"			, 272, 830,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bst		, Bisepttrium				=	Bst		= element		( 2730,	"Bisepttrium"			, 273, 835,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bsq		, Biseptquadium				=	Bsq		= element		( 2740,	"Biseptquadium"			, 274, 840,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bsp		, Biseptpentium				=	Bsp		= element		( 2750,	"Biseptpentium"			, 275, 845,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bsh		, Bisepthexium				=	Bsh		= element		( 2760,	"Bisepthexium"			, 276, 850,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bss		, Biseptseptium				=	Bss		= element		( 2770,	"Biseptseptium"			, 277, 855,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bso		, Biseptoctium				=	Bso		= element		( 2780,	"Biseptoctium"			, 278, 860,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bse		, Biseptennium				=	Bse		= element		( 2790,	"Biseptennium"			, 279, 865,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bon		, Bioctnilium				=	Bon		= element		( 2800,	"Bioctnilium"			, 280, 870,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bou		, Bioctunium				=	Bou		= element		( 2810,	"Bioctunium"			, 281, 875,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bob		, Bioctbium					=	Bob		= element		( 2820,	"Bioctbium"				, 282, 880,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bot		, Biocttrium				=	Bot		= element		( 2830,	"Biocttrium"			, 283, 885,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Boq		, Bioctquadium				=	Boq		= element		( 2840,	"Bioctquadium"			, 284, 890,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bop		, Bioctpentium				=	Bop		= element		( 2850,	"Bioctpentium"			, 285, 895,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Boh		, Biocthexium				=	Boh		= element		( 2860,	"Biocthexium"			, 286, 900,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bos		, Bioctseptium				=	Bos		= element		( 2870,	"Bioctseptium"			, 287, 905,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Boo		, Bioctoctium				=	Boo		= element		( 2880,	"Bioctoctium"			, 288, 910,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Boe		, Bioctennium				=	Boe		= element		( 2890,	"Bioctennium"			, 289, 915,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ben		, Biennilium				=	Ben		= element		( 2900,	"Biennilium"			, 290, 920,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Beu		, Biennunium				=	Beu		= element		( 2910,	"Biennunium"			, 291, 925,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Beb		, Biennbium					=	Beb		= element		( 2920,	"Biennbium"				, 292, 930,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bet		, Bienntrium				=	Bet		= element		( 2930,	"Bienntrium"			, 293, 935,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Beq		, Biennquadium				=	Beq		= element		( 2940,	"Biennquadium"			, 294, 940,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bep		, Biennpentium				=	Bep		= element		( 2950,	"Biennpentium"			, 295, 945,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Beh		, Biennhexium				=	Beh		= element		( 2960,	"Biennhexium"			, 296, 950,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bes		, Biennseptium				=	Bes		= element		( 2970,	"Biennseptium"			, 297, 955,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Beo		, Biennoctium				=	Beo		= element		( 2980,	"Biennoctium"			, 298, 960,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Bee		, Biennennium				=	Bee		= element		( 2990,	"Biennennium"			, 299, 965,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tnn		, Trinilnilium				=	Tnn		= element		( 3000,	"Trinilnilium"			, 300, 970,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tnu		, Trinilunium				=	Tnu		= element		( 3010,	"Trinilunium"			, 301, 975,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tnb		, Trinilbium				=	Tnb		= element		( 3020,	"Trinilbium"			, 302, 980,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tnt		, Triniltrium				=	Tnt		= element		( 3030,	"Triniltrium"			, 303, 985,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tnq		, Trinilquadium				=	Tnq		= element		( 3040,	"Trinilquadium"			, 304, 990,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tnp		, Trinilpentium				=	Tnp		= element		( 3050,	"Trinilpentium"			, 305, 995,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tnh		, Trinilhexium				=	Tnh		= element		( 3060,	"Trinilhexium"			, 306,1000,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tns		, Trinilseptium				=	Tns		= element		( 3070,	"Trinilseptium"			, 307,1005,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tno		, Triniloctium				=	Tno		= element		( 3080,	"Triniloctium"			, 308,1010,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tne		, Trinilennium				=	Tne		= element		( 3090,	"Trinilennium"			, 309,1015,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tun		, Triunnilium				=	Tun		= element		( 3100,	"Triunnilium"			, 310,1020,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tuu		, Triununium				=	Tuu		= element		( 3110,	"Triununium"			, 311,1025,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tub		, Triunbium					=	Tub		= element		( 3120,	"Triunbium"				, 312,1030,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tut		, Triuntrium				=	Tut		= element		( 3130,	"Triuntrium"			, 313,1035,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tuq		, Triunquadium				=	Tuq		= element		( 3140,	"Triunquadium"			, 314,1040,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tup		, Triunpentium				=	Tup		= element		( 3150,	"Triunpentium"			, 315,1045,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tuh		, Triunhexium				=	Tuh		= element		( 3160,	"Triunhexium"			, 316,1050,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tus		, Triunseptium				=	Tus		= element		( 3170,	"Triunseptium"			, 317,1055,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tuo		, Triunoctium				=	Tuo		= element		( 3180,	"Triunoctium"			, 318,1060,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tue		, Triunennium				=	Tue		= element		( 3190,	"Triunennium"			, 319,1065,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tbn		, Tribinilium				=	Tbn		= element		( 3200,	"Tribinilium"			, 320,1070,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tbu		, Tribiunium				=	Tbu		= element		( 3210,	"Tribiunium"			, 321,1075,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tbb		, Tribibium					=	Tbb		= element		( 3220,	"Tribibium"				, 322,1080,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tbt		, Tribitrium				=	Tbt		= element		( 3230,	"Tribitrium"			, 323,1085,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tbq		, Tribiquadium				=	Tbq		= element		( 3240,	"Tribiquadium"			, 324,1090,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tbp		, Tribipentium				=	Tbp		= element		( 3250,	"Tribipentium"			, 325,1095,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tbh		, Tribihexium				=	Tbh		= element		( 3260,	"Tribihexium"			, 326,1100,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tbs		, Tribiseptium				=	Tbs		= element		( 3270,	"Tribiseptium"			, 327,1105,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tbo		, Tribioctium				=	Tbo		= element		( 3280,	"Tribioctium"			, 328,1110,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tbe		, Tribiennium				=	Tbe		= element		( 3290,	"Tribiennium"			, 329,1115,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ttn		, Tritrinilium				=	Ttn		= element		( 3300,	"Tritrinilium"			, 330,1120,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ttu		, Tritriunium				=	Ttu		= element		( 3310,	"Tritriunium"			, 331,1125,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ttb		, Tritribium				=	Ttb		= element		( 3320,	"Tritribium"			, 332,1130,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ttt		, Tritritrium				=	Ttt		= element		( 3330,	"Tritritrium"			, 333,1135,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ttq		, Tritriquadium				=	Ttq		= element		( 3340,	"Tritriquadium"			, 334,1140,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ttp		, Tritripentium				=	Ttp		= element		( 3350,	"Tritripentium"			, 335,1145,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tth		, Tritrihexium				=	Tth		= element		( 3360,	"Tritrihexium"			, 336,1150,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tts		, Tritriseptium				=	Tts		= element		( 3370,	"Tritriseptium"			, 337,1155,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tto		, Tritrioctium				=	Tto		= element		( 3380,	"Tritrioctium"			, 338,1160,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tte		, Tritriennium				=	Tte		= element		( 3390,	"Tritriennium"			, 339,1165,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tqn		, Triquadnilium				=	Tqn		= element		( 3400,	"Triquadnilium"			, 340,1170,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tqu		, Triquadunium				=	Tqu		= element		( 3410,	"Triquadunium"			, 341,1175,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tqb		, Triquadbium				=	Tqb		= element		( 3420,	"Triquadbium"			, 342,1180,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tqt		, Triquadtrium				=	Tqt		= element		( 3430,	"Triquadtrium"			, 343,1185,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tqq		, Triquadquadium			=	Tqq		= element		( 3440,	"Triquadquadium"		, 344,1190,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tqp		, Triquadpentium			=	Tqp		= element		( 3450,	"Triquadpentium"		, 345,1195,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tqh		, Triquadhexium				=	Tqh		= element		( 3460,	"Triquadhexium"			, 346,1200,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tqs		, Triquadseptium			=	Tqs		= element		( 3470,	"Triquadseptium"		, 347,1205,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tqo		, Triquadoctium				=	Tqo		= element		( 3480,	"Triquadoctium"			, 348,1210,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tqe		, Triquadennium				=	Tqe		= element		( 3490,	"Triquadennium"			, 349,1215,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tpn		, Tripentnilium				=	Tpn		= element		( 3500,	"Tripentnilium"			, 350,1220,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tpu		, Tripentunium				=	Tpu		= element		( 3510,	"Tripentunium"			, 351,1225,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tpb		, Tripentbium				=	Tpb		= element		( 3520,	"Tripentbium"			, 352,1230,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tpt		, Tripenttrium				=	Tpt		= element		( 3530,	"Tripenttrium"			, 353,1235,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tpq		, Tripentquadium			=	Tpq		= element		( 3540,	"Tripentquadium"		, 354,1240,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tpp		, Tripentpentium			=	Tpp		= element		( 3550,	"Tripentpentium"		, 355,1245,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tph		, Tripenthexium				=	Tph		= element		( 3560,	"Tripenthexium"			, 356,1250,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tps		, Tripentseptium			=	Tps		= element		( 3570,	"Tripentseptium"		, 357,1255,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tpo		, Tripentoctium				=	Tpo		= element		( 3580,	"Tripentoctium"			, 358,1260,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tpe		, Tripentennium				=	Tpe		= element		( 3590,	"Tripentennium"			, 359,1265,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Thn		, Trihexnilium				=	Thn		= element		( 3600,	"Trihexnilium"			, 360,1270,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Thu		, Trihexunium				=	Thu		= element		( 3610,	"Trihexunium"			, 361,1275,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Thb		, Trihexbium				=	Thb		= element		( 3620,	"Trihexbium"			, 362,1280,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tht		, Trihextrium				=	Tht		= element		( 3630,	"Trihextrium"			, 363,1285,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Thq		, Trihexquadium				=	Thq		= element		( 3640,	"Trihexquadium"			, 364,1290,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Thp		, Trihexpentium				=	Thp		= element		( 3650,	"Trihexpentium"			, 365,1295,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Thh		, Trihexhexium				=	Thh		= element		( 3660,	"Trihexhexium"			, 366,1300,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ths		, Trihexseptium				=	Ths		= element		( 3670,	"Trihexseptium"			, 367,1305,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tho		, Trihexoctium				=	Tho		= element		( 3680,	"Trihexoctium"			, 368,1310,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	The		, Trihexennium				=	The		= element		( 3690,	"Trihexennium"			, 369,1315,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tsn		, Triseptnilium				=	Tsn		= element		( 3700,	"Triseptnilium"			, 370,1320,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tsu		, Triseptunium				=	Tsu		= element		( 3710,	"Triseptunium"			, 371,1325,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Gt		, Gravitonium				=	Gt		= element		( 3720,	"Gravitonium"			, 372,1330,   112,  1275, 1768.866761, SET_SHINY			,   0,  50,   0, 255).add(	CONTAINERS																		).aspects(TC.TERRA, 10, TC.POTENTIA, 10), Tsb = Gt, Triseptbium = Gt.addIdenticalNames("Triseptbium"),
	Tst		, Trisepttrium				=	Tst		= element		( 3730,	"Trisepttrium"			, 373,1335,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tsq		, Triseptquadium			=	Tsq		= element		( 3740,	"Triseptquadium"		, 374,1340,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tsp		, Triseptpentium			=	Tsp		= element		( 3750,	"Triseptpentium"		, 375,1345,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tsh		, Trisepthexium				=	Tsh		= element		( 3760,	"Trisepthexium"			, 376,1350,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tss		, Triseptseptium			=	Tss		= element		( 3770,	"Triseptseptium"		, 377,1355,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tso		, Triseptoctium				=	Tso		= element		( 3780,	"Triseptoctium"			, 378,1360,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tse		, Triseptennium				=	Tse		= element		( 3790,	"Triseptennium"			, 379,1365,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ton		, Trioctnilium				=	Ton		= element		( 3800,	"Trioctnilium"			, 380,1370,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tou		, Trioctunium				=	Tou		= element		( 3810,	"Trioctunium"			, 381,1375,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tob		, Trioctbium				=	Tob		= element		( 3820,	"Trioctbium"			, 382,1380,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tot		, Triocttrium				=	Tot		= element		( 3830,	"Triocttrium"			, 383,1385,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Toq		, Trioctquadium				=	Toq		= element		( 3840,	"Trioctquadium"			, 384,1390,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Top		, Trioctpentium				=	Top		= element		( 3850,	"Trioctpentium"			, 385,1395,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Toh		, Triocthexium				=	Toh		= element		( 3860,	"Triocthexium"			, 386,1400,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tos		, Trioctseptium				=	Tos		= element		( 3870,	"Trioctseptium"			, 387,1405,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Too		, Trioctoctium				=	Too		= element		( 3880,	"Trioctoctium"			, 388,1410,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Toe		, Trioctennium				=	Toe		= element		( 3890,	"Trioctennium"			, 389,1415,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Ten		, Triennilium				=	Ten		= element		( 3900,	"Triennilium"			, 390,1420,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Teu		, Triennunium				=	Teu		= element		( 3910,	"Triennunium"			, 391,1425,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Teb		, Triennbium				=	Teb		= element		( 3920,	"Triennbium"			, 392,1430,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tet		, Trienntrium				=	Tet		= element		( 3930,	"Trienntrium"			, 393,1435,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Teq		, Triennquadium				=	Teq		= element		( 3940,	"Triennquadium"			, 394,1440,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tep		, Triennpentium				=	Tep		= element		( 3950,	"Triennpentium"			, 395,1445,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Teh		, Triennhexium				=	Teh		= element		( 3960,	"Triennhexium"			, 396,1450,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tes		, Triennseptium				=	Tes		= element		( 3970,	"Triennseptium"			, 397,1455,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Teo		, Triennoctium				=	Teo		= element		( 3980,	"Triennoctium"			, 398,1460,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3),
	Tee		, Triennennium				=	Tee		= element		( 3990,	"Triennennium"			, 399,1465,  1000,  3000,           0, SET_SHINY			, 256, 256, 256, 255).add(	HIDDEN																			).aspects(TC.RADIO, 3);
	
	public static final OreDictMaterial
	Neutronium				= unused		("Neutronium"		).setQuality(6.0F, 81920, 6),
	
	Primitive				= unused		("Primitive"		).aspects(TC.MACHINA	, 1)	.add(DONT_SHOW_THIS_COMPONENT).setAllToTheOutputOf(null, 0, 1),
	Basic					= unused		("Basic"			).aspects(TC.MACHINA	, 2)	.add(DONT_SHOW_THIS_COMPONENT).setAllToTheOutputOf(null, 0, 1),
	Good					= unused		("Good"				).aspects(TC.MACHINA	, 3)	.add(DONT_SHOW_THIS_COMPONENT).setAllToTheOutputOf(null, 0, 1),
	Advanced				= unused		("Advanced"			).aspects(TC.MACHINA	, 4)	.add(DONT_SHOW_THIS_COMPONENT).setAllToTheOutputOf(null, 0, 1),
	Data					= unused		("Data"				).aspects(TC.MACHINA	, 5)	.add(DONT_SHOW_THIS_COMPONENT).setAllToTheOutputOf(null, 0, 1),
	Elite					= unused		("Elite"			).aspects(TC.MACHINA	, 6)	.add(DONT_SHOW_THIS_COMPONENT).setAllToTheOutputOf(null, 0, 1),
	Master					= unused		("Master"			).aspects(TC.MACHINA	, 7)	.add(DONT_SHOW_THIS_COMPONENT).setAllToTheOutputOf(null, 0, 1),
	Ultimate				= unused		("Ultimate"			).aspects(TC.MACHINA	, 8)	.add(DONT_SHOW_THIS_COMPONENT).setAllToTheOutputOf(null, 0, 1),
	Quantum					= unused		("Quantum"			).aspects(TC.ORDO		, 8)	.add(DONT_SHOW_THIS_COMPONENT).setAllToTheOutputOf(null, 0, 1),
	Superconductor			= unused		("Superconductor"	).aspects(TC.ELECTRUM	, 8)	.add(DONT_SHOW_THIS_COMPONENT).setAllToTheOutputOf(null, 0, 1),
	Infinite				= unused		("Infinite"			)								.add(DONT_SHOW_THIS_COMPONENT).setAllToTheOutputOf(null, 0, 1);
	
	public static final OreDictMaterial
	Water					= create		( 9800, "Water"					, SET_FLUID				,   0,   0, 255, 255).add(CONTAINERS, FOOD, UNRECYCLABLE, MELTING, LIQUID)													  										.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.AQUA, 2).setLiquid(UT.Fluids.water(1000)).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0),
	Snow					= create		( 9801, "Snow"					, SET_FINE				, 250, 250, 250, 255).add(G_DUST, FOOD, UNRECYCLABLE, MELTING)																										.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.GELUM, 1).setSmelting(Water, U).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0),
	Ice						= create		( 9802, "Ice"					, SET_SHINY				, 200, 200, 255, 255).add(G_GEM_TRANSPARENT, FOOD, CONTAINERS, UNRECYCLABLE, MELTING)																				.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.GELUM, 2).setSmelting(Water, U).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0).setQuality( 2.0F,	 24,  0),
	FreshWater				= create		( 9803, "Fresh Water"			, SET_FLUID				,   0,   0, 255, 255).add(CONTAINERS, LIQUID, FOOD, UNRECYCLABLE)																									.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.AQUA, 2).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0),
	SaltWater				= create		( 9804, "Salt Water"			, SET_FLUID				,   0,   0, 255, 255).add(CONTAINERS, LIQUID, UNRECYCLABLE)																											.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.AQUA, 2).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0),
	HolyWater				= create		( 9805, "Holy Water"			, SET_FLUID				,   0,   0, 255, 255).add(CONTAINERS, LIQUID, GLOWING)																												.setMcfg( 0, H				, 2*U, O				, 1*U)																									.aspects(TC.AQUA, 2, TC.AURAM, 1).setStatsEnergetic(CS.C, CS.C+100).setDensity(1.0),
	Lava					= create		( 9810,	"Lava"					, SET_STONE				, 255,  64,   0, 255).add(CONTAINERS, LIQUID, MELTING, GLOWING, LIGHTING)																																																														.setStatsEnergetic(1000, 4000).setLiquid(UT.Fluids.lava(1000)),
	NitroCarbon				= create		( 9820, "Nitro Carbon"			, SET_FLUID				,   0,  75, 100, 255).add(CONTAINERS, LIQUID, ELECTROLYSER, DECOMPOSABLE, EXPLOSIVE, FLAMMABLE)													   					.setMcfg( 0, N				, 1*U, C				, 1*U)																									,
	Glyceryl				= create		( 9821, "Glyceryl"				, SET_FLUID				,   0, 150, 150, 255).add(CONTAINERS, LIQUID, ELECTROLYSER, DECOMPOSABLE, EXPLOSIVE, FLAMMABLE)																		.setMcfg( 0, C				, 3*U, H				, 5*U, N				, 3*U, O				, 9*U)													,
	SodiumPersulfate		= create		( 9822, "Sodium Persulfate"		, SET_FLUID				, 256, 256, 256, 255).add(CONTAINERS, LIQUID, ELECTROLYSER, DECOMPOSABLE)																							.setMcfg( 0, Na				, 1*U, S				, 1*U, O				, 4*U)																			,
	SodiumSulfide			= create		( 9823, "Sodium Sulfide"		, SET_FLUID				, 256, 256, 256, 255).add(CONTAINERS, LIQUID, ELECTROLYSER, DECOMPOSABLE)																							.setMcfg( 0, Na				, 1*U, S				, 1*U)																									,
	SulfuricAcid			= create		( 9824, "Sulfuric Acid"			, SET_FLUID				, 255, 128,   0, 255).add(CONTAINERS, LIQUID, ELECTROLYSER, DECOMPOSABLE)																							.setMcfg( 0, H				, 2*U, S				, 1*U, O				, 4*U)																			,
	CobaltHexahydrate		= create		( 9825,	"Cobalt Hexahydrate"	, SET_METALLIC			,  80,  80, 250, 255).add(CONTAINERS, LIQUID)																																																																					,
	Air						= create		( 9830, "Air"					, SET_FLUID				, 169, 208, 245,  15).add(CONTAINERS, GASEOUS, TRANSPARENT)										  																	.setMcfg( 0, N				,40*U, O				,11*U, Ar				, 1*U)																			,
	NitrogenDioxide			= create		( 9831, "Nitrogen Dioxide"		, SET_FLUID				, 100, 175, 255, 255).add(CONTAINERS, GASEOUS, ELECTROLYSER, DECOMPOSABLE)													   										.setMcfg( 0, N				, 1*U, O				, 2*U)																									,
	Methane					= create		( 9832, "Methane"				, SET_FLUID				, 250, 200, 250, 255).add(CONTAINERS, GASEOUS, FLAMMABLE, ELECTROLYSER, DECOMPOSABLE)																				.setMcfg( 0, C				, 1*U, H				, 4*U)																									,
	MethaneIce				= create		( 9833, "Methane Ice"			, SET_SHINY				, 225, 200, 250, 255).add(G_DUST, CONTAINERS, FLAMMABLE, ELECTROLYSER, DECOMPOSABLE)																				.setMcfg( 2, Methane		, 1*U, Ice				, 2*U)																									,
	Biomass					= create		( 9840, "Biomass"				, SET_FLUID				,   0, 255,   0, 255).add(CONTAINERS, LIQUID, FLAMMABLE)																																																																		.addIdenticalNames("BioMass"),
	BioFuel					= create		( 9841, "Bio Fuel"				, SET_FLUID				, 200, 128,   0, 255).add(CONTAINERS, LIQUID, FLAMMABLE)																																																																		.setLocal("Bio Diesel"),
	Ethanol					= create		( 9842, "Ethanol"				, SET_FLUID				, 255, 128,   0, 255).add(CONTAINERS, LIQUID, FLAMMABLE)																																																																		,
	Oil						= create		( 9850, "Oil"					, SET_FLUID				,  10,  10,  10, 255).add(CONTAINERS, LIQUID, FLAMMABLE)																																																																		,
	Oilsands				= create		( 9851,	"Oilsands"				, SET_SAND				,  10,  10,  10, 255).add(G_DUST_ORES, FLAMMABLE)																																																																				,
	CrudeOil				= create		( 9852,	"Crude Oil"				, SET_DULL				,  10,  10,  10, 255).add(G_DUST_ORES, CONTAINERS, FLAMMABLE)																																																																	,
	Fuel					= create		( 9860, "Fuel"					, SET_FLUID				, 255, 255,   0, 255).add(CONTAINERS, LIQUID, EXPLOSIVE, FLAMMABLE)																																																																.setLocal("Diesel").addIdenticalNames("Diesel"),
	NitroFuel				= create		( 9861, "Nitro-Fuel"			, SET_FLUID				, 200, 255,   0, 255).add(CONTAINERS, LIQUID, EXPLOSIVE, FLAMMABLE)																									.setMcfg( 0, Glyceryl		, 1*U, Fuel				, 4*U)																									,
	Creosote				= create		( 9870, "Creosote Oil"			, SET_FLUID				, 128,  64,   0, 255).add(CONTAINERS, LIQUID, FLAMMABLE)																																																																		.addIdenticalNames("Creosote"),
	FishOil					= create		( 9871, "Fish Oil"				, SET_FLUID				, 255, 196,   0, 255).add(CONTAINERS, LIQUID, FLAMMABLE, FOOD)																																																																	.aspects(TC.CORPUS, 2),
	SeedOil					= create		( 9872, "Seed Oil"				, SET_FLUID				, 196, 255,   0, 255).add(CONTAINERS, LIQUID, FLAMMABLE, FOOD)																																																																	.aspects(TC.GRANUM, 2),
	SeedOilHemp				= create		( 9873, "SeedOilHemp"			, SET_FLUID				, 196, 255,   0, 255).add(CONTAINERS, LIQUID, FLAMMABLE, FOOD)																																																																	.aspects(TC.GRANUM, 2).setLocal("Hemp Seed Oil"),
	SeedOilLin				= create		( 9874, "SeedOilLin"			, SET_FLUID				, 196, 255,   0, 255).add(CONTAINERS, LIQUID, FLAMMABLE, FOOD)																																																																	.aspects(TC.GRANUM, 2).setLocal("Lin Seed Oil"),
	FryingOilHot			= create		( 9880, "FryingOilHot"			, SET_FLUID				, 200, 196,   0, 255).add(CONTAINERS, LIQUID, FLAMMABLE, FOOD)																																																																	.aspects(TC.AQUA, 1, TC.IGNIS, 1).setLocal("Hot Frying Oil"),
	Glue					= create		( 9881, "Glue"					, SET_FLUID				, 200, 196,   0, 255).add(CONTAINERS, LIQUID)																																																																					.aspects(TC.LIMUS, 2),
	Lubricant				= create		( 9882, "Lubricant"				, SET_FLUID				, 255, 196,   0, 255).add(CONTAINERS, LIQUID)																																																																					.aspects(TC.AQUA, 2, TC.MACHINA, 1),
	ConstructionFoam		= create		( 9883,	"Construction Foam"		, SET_DULL				, 128, 128, 128, 255).add(CONTAINERS, LIQUID, STONE, BRITTLE, MELTING)																																																															,
	UUAmplifier				= create		( 9884,	"UU-Amplifier"			, SET_FLUID				,  96,   0, 128, 255).add(CONTAINERS, LIQUID)																																																																					,
	UUMatter				= create		( 9885,	"UU-Matter"				, SET_FLUID				, 128,   0, 196, 255).add(CONTAINERS, LIQUID, GLOWING)																																																																			;
	
	public static final OreDictMaterial
	SiliconDioxide			= create		( 8000, "Silicon Dioxide"		, SET_QUARTZ			, 200, 200, 200, 255).add(G_DUST, ELECTROLYSER, DECOMPOSABLE, BRITTLE, QUARTZ, CRYSTALLISABLE)																		.setMcfg( 0, Si				, 1*U, O 				, 2*U)																									,
	Glass					= create		( 8001, "Glass"					, SET_GLASS				, 250, 250, 250,  35).add(G_GEM_TRANSPARENT, CENTRIFUGE, DECOMPOSABLE, FURNACE, CRYSTAL, UNRECYCLABLE, MELTING, BRITTLE)											.setMcfg( 0, SiliconDioxide	, 1*U)																															.aspects(TC.VITREUS, 2).setQuality( 1.0F,	  4,  0).setMeltingPoint(800),
	Flint					= create		( 8002, "Flint"					, SET_FLINT				,   0,  32,  64, 255).add(G_GEM, STONE, BRITTLE, DIRTY_DUSTS, CENTRIFUGE, DECOMPOSABLE, MORTAR)																		.setMcfg( 0, SiliconDioxide	, 1*U)																															.aspects(TC.TERRA, 1, TC.INSTRUMENTUM, 1).setQuality( 2.5F,	 48,  1);
	
	public static final OreDictMaterial
	Ash						= create		( 8200, "Ashes"					, SET_DULL				, 150, 150, 150, 255).add(G_DUST, CENTRIFUGE, DECOMPOSABLE)														   													.setMcfg( 0, C 				, 1*U)																															.aspects(TC.PERDITIO, 1).addIdenticalNames("Ash"),
	DarkAsh					= create		( 8201, "Dark Ashes"			, SET_DULL				,  50,  50,  50, 255).add(G_DUST, ELECTROLYSER, DECOMPOSABLE)																										.setMcfg( 1, C 				, 2*U)																															.aspects(TC.IGNIS, 1, TC.PERDITIO, 1).addIdenticalNames("DarkAsh"),
	VolcanicAsh				= create		( 8202, "Volcanic Ashes"		, SET_FLINT				,  60,  50,  50, 255).add(G_DUST, CENTRIFUGE, DECOMPOSABLE)																											.setMcfg( 0, Flint			, 6*U, Fe				, 1*U, Mg				, 1*U)																			.aspects(TC.IGNIS, 2, TC.PERDITIO, 1).addIdenticalNames("VolcanicAsh"),
	RockSalt				= create		( 8203, "Rock Salt"				, SET_FINE				, 240, 200, 200, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE)													  										.setMcfg( 0, K 				, 1*U, Cl				, 1*U)																									,
	Salt					= create		( 8204, "Salt"					, SET_FINE				, 250, 250, 250, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FOOD)													   								.setMcfg( 0, Na				, 1*U, Cl				, 1*U)																									,
	Saltpeter				= create		( 8205, "Saltpeter"				, SET_FINE				, 230, 230, 230, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, FLAMMABLE)										   												.setMcfg( 0, K 				, 1*U, N 				, 1*U, O				, 3*U)																			,
	Niter					= create		( 8206, "Niter"					, SET_FLINT				, 255, 200, 200, 255).add(G_GEM_ORES, CENTRIFUGE, DECOMPOSABLE, CRYSTAL, BRITTLE)																					.setMcfg( 0, Saltpeter		, 1*U)																															,
	Phosphate				= create		( 8207, "Phosphate"				, SET_DULL				, 255, 255,   0, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, FLAMMABLE, EXPLOSIVE)							  												.setMcfg( 0, P 				, 1*U, O 				, 4*U)																									,
	Phosphorus				= create		( 8208, "Phosphorus"			, SET_FLINT				, 255, 255,   0, 255).add(G_GEM_ORES, CENTRIFUGE, DECOMPOSABLE, FLAMMABLE, EXPLOSIVE)																				.setMcfg( 0, Ca				, 3*U, Phosphate		, 2*U)																									,
	Apatite					= create		( 8209, "Apatite"				, SET_DIAMOND			, 200, 200, 255, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, FLAMMABLE, CRYSTAL, BRITTLE, CRYSTALLISABLE)														.setMcfg( 0, Ca				, 5*U, Phosphate		, 3*U, Cl				, 1*U)																			.aspects(TC.MESSIS, 2),
	Blizz					= create		( 8210,	"Blizz"					, SET_SHINY				, 220, 233, 255, 255).add(G_DUST)																																																																								,
	Blaze					= create		( 8211, "Blaze"					, SET_POWDER			, 255, 200,   0, 255).add(G_BLAZE, CENTRIFUGE, DECOMPOSABLE, MAGICAL, UNBURNABLE, BURNING, MORTAR, MELTING, BRITTLE)												.setMcfg( 2, DarkAsh		, 1*U, S				, 1*U, Ma				, 1*U)																			.aspects(TC.PRAECANTIO, 2, TC.IGNIS, 4).setQuality(2.0F, 16, 1).setMeltingPoint(4000),
	Obsidian				= create		( 8214, "Obsidian"				, SET_STONE				,  80,  50, 100, 255).add(G_STONE, STONE, BRITTLE, ELECTROLYSER, DECOMPOSABLE)														   								.setMcfg( 0, Mg				, 1*U, Fe				, 1*U, Si				, 2*U, O 				, 8*U)													.setSmelting(Lava, U).setStatsEnergetic(1000, 4000).setQuality( 3.0F,	128,  3),
	Clay					= create		( 8215, "Clay"					, SET_ROUGH				, 200, 200, 220, 255).add(G_DUST, ELECTROLYSER, DECOMPOSABLE)														   												.setMcfg( 0, Na				, 2*U, Li				, 1*U, Al				, 2*U, Si				, 2*U)													.setMeltingPoint(2000),
	Paper					= create		( 8216, "Paper"					, SET_PAPER				, 250, 250, 250, 255).add(G_DUST, MULTIPLATES)																																																																					.aspects(TC.COGNITO, 1).setBurning(Ash, U / 9),
	Rubber					= create		( 8217, "Rubber"				, SET_SHINY				,   0,   0,   0, 255).add(G_INGOT_MACHINE, APPROXIMATE, FLAMMABLE, MELTING, BOUNCY, STRETCHY, FURNACE)																.setMcfg( 0, C 				, 5*U, H 				, 8*U)																									.aspects(TC.MOTUS, 2).setMeltingPoint(410).setBurning(Ash, U / 9).setSmelting(null, 2*U/3).setQuality(1.5F, 16, 0),
	Plastic					= create		( 8218, "Plastic"				, SET_DULL				, 200, 200, 200, 255).add(G_INGOT_MACHINE, APPROXIMATE, FLAMMABLE, MELTING, BOUNCY, BRITTLE, FURNACE)																.setMcfg( 0, C 				, 1*U, H 				, 2*U)																									.aspects(TC.MOTUS, 2).setMeltingPoint(440).setBurning(Ash, U / 9).setSmelting(null, 2*U/3).setQuality(3.0F, 32, 1),
	Bone					= create		( 8219, "Bone"					, SET_DULL				, 250, 250, 250, 255).add(G_DUST)														   																			.setMcfg( 0, Ca				, 1*U)																															.aspects(TC.MORTUUS, 2, TC.CORPUS, 1),
	Gunpowder				= create		( 8220, "Gunpowder"				, SET_DULL				, 128, 128, 128, 255).add(G_DUST, EXPLOSIVE, FLAMMABLE)																																																																			.aspects(TC.PERDITIO, 3, TC.IGNIS, 4).setBurning(Ash, U / 9),
	Wood					= create		( 8221, "Wood"					, SET_WOOD				, 100,  50,   0, 255).add(G_WOOD, WOOD, FLAMMABLE, APPROXIMATE, AUTO_BLACKLIST)																						.setMcfg( 0, C 				, 1*U, O 				, 1*U, H 				, 1*U)																			.aspects(TC.ARBOR, 2).setBurning(Ash, U / 9).setSmelting(Ash, U / 4).setQuality(2.0F, 16, 0),
	WoodSealed				= create		( 8222, "WoodSealed"			, SET_WOOD				,  80,  40,   0, 255).add(G_WOOD, WOOD, FLAMMABLE)																													.setMcfg( 0, Wood			, 1*U)																															.aspects(TC.ARBOR, 2, TC.FABRICO, 1).setAllToTheOutputOf(Wood).setQuality(3.0F, 24, 0).setLocal("Sealed Wood"),
	LiveRoot				= create		( 8223, "LiveRoot"				, SET_WOOD				, 220, 200,   0, 255).add(G_DUST, WOOD, CENTRIFUGE, DECOMPOSABLE, MAGICAL)																							.setMcfg( 3, Wood			, 3*U, Ma				, 1*U)																									.aspects(TC.ARBOR, 2, TC.VICTUS, 2, TC.PRAECANTIO, 1).setLocal("Liveroot").setBurning(Ash, U / 9).setStatsEnergetic(1178, 2465),
	Peanutwood				= create		(-8224,	"Peanut Wood"			, SET_NONE				, 256, 256, 256, 255).add(G_WOOD, WOOD, FLAMMABLE)																													.setMcfg( 0, Wood			, 1*U)																															.aspects(TC.ARBOR, 2).setAllToTheOutputOf(Wood).steal(Wood).addIdenticalNames("Peanutwood"),
	Ceramic					= create		( 8225, "Ceramic"				, SET_ROUGH				, 196, 128,  64, 255).add(G_DUST, PLATES, BRITTLE)														   															.setMcfg( 0, Clay			, 1*U)																															.steal(Clay).setMeltingPoint(2000).setPulver(Clay, U/2).setCompressing(null, 0).setBending(null, 0).setForging(null, 0).setSmashing(null, 0);
	
	static {
		Water.setSolidifying(Ice, U);
		Lava.setSolidifying(Obsidian, U).setDensity(Obsidian.mGramPerCubicCentimeter);
	}
	
	public static final OreDictMaterial
	MeatCooked				= create		( 9701, "MeatCooked"			, SET_FINE				, 150,  60,  20, 255).add(G_DUST, FOOD, FURNACE)																																																																				.setMeltingPoint(477).setLocal("Cooked Meat"),
	MeatRaw					= create		( 9700, "MeatRaw"				, SET_FINE				, 255, 100, 100, 255).add(G_DUST, FOOD, FURNACE)																																																																				.setMeltingPoint(477).setLocal("Raw Meat").setSmelting(MeatCooked, U).addIdenticalNames("Meat"),
	Wheat					= create		( 9702, "Wheat"					, SET_POWDER			, 255, 255, 196, 255).add(G_DUST, FOOD, FLAMMABLE)																																																																				.aspects(TC.MESSIS, 2).addIdenticalNames("Flour").setBurning(Ash, U / 9),
	Sugar					= create		( 9703, "Sugar"					, SET_FINE				, 250, 250, 250, 255).add(G_DUST, FOOD, ELECTROLYSER, DECOMPOSABLE, MELTING)																						.setMcfg( 0, C				, 2*U, Water			, 5*U, O				,25*U)																			.setMeltingPoint(340).aspects(TC.HERBA, 1, TC.AQUA, 1, TC.AER, 1).setMeltingPoint(320),
	Cheese					= create		( 9780, "Cheese"				, SET_FINE				, 255, 255,   0, 255).add(G_DUST_ORES, FOOD, MELTING)																																																																			.setMeltingPoint(320),
	Chili					= create		( 9781, "Chili"					, SET_FINE				, 200,   0,   0, 255).add(G_DUST, FOOD)																																																																							,
	Cocoa					= create		( 9782, "Cocoa"					, SET_FINE				, 190,  95,   0, 255).add(G_DUST, FOOD)																																																																							,
	Chocolate				= create		( 9783, "Chocolate"				, SET_FINE				, 190,  95,   0, 255).add(G_DUST, FOOD)																																																																							,
	Coffee					= create		( 9784, "Coffee"				, SET_FINE				, 150,  75,   0, 255).add(G_DUST, FOOD)																																																																							.addIdenticalNames("CoffeeDust"),
	Milk					= create		( 9790, "Milk"					, SET_FINE				, 254, 254, 254, 255).add(CONTAINERS, FOOD)																																																																						.aspects(TC.SANO, 2),
	Honey					= create		( 9791, "Honey"					, SET_FLUID				, 210, 200,   0, 255).add(CONTAINERS, FOOD)																																																																						;
	
	public static final OreDictMaterial
	Sand					= create		( 8100,	"Sand"					, SET_SAND				, 250, 250, 200, 255).add(FURNACE, UNRECYCLABLE)																																																																				.setSmelting(Glass, U),
	SoulSand				= create		( 8101,	"Soulsand"				, SET_SAND				, 100, 100,  80, 255)																																																																											;
	
	public static final OreDictMaterial
	RareEarth				= create		( 9100, "Rare Earth"			, SET_FINE				, 128, 128, 100, 255).add(G_DUST_ORES																	)																																																						.aspects(TC.VITREUS, 1, TC.LUCRUM, 1),
	Almandine				= create		( 9101, "Almandine"				, SET_ROUGH				, 255,   0,   0, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Al				, 2*U, Fe				, 3*U, Si				, 3*U, O				,12*U)													,
	Andradite				= create		( 9102, "Andradite"				, SET_ROUGH				, 150, 120,   0, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_IRON		)						 									.setMcfg( 0, Ca				, 3*U, Fe				, 2*U, Si				, 3*U, O				,12*U)													,
	Asbestos				= create		( 9103, "Asbestos"				, SET_DULL				, 230, 230, 230, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Mg				, 3*U, Si				, 2*U, H 				, 4*U, O				, 9*U)													, // Mg3Si2O5(OH)4
	BandedIron				= create		( 9104, "Banded Iron"			, SET_DULL				, 145,  90,  90, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_IRON		)															.setMcfg( 0, Fe				, 2*U, O 				, 3*U)																									.setMeltingPoint(2 * Fe.mMeltingPoint / 3),
	Bauxite					= create		( 9105, "Bauxite"				, SET_DULL				, 200, 100,   0, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Al,   U/2)									.setMcfg( 0, Ti				, 1*U, Al				,16*U, H				,10*U, O				,12*U)													.setMeltingPoint(2800),
	BrownLimonite			= create		( 9106, "Brown Limonite"		, SET_METALLIC			, 200, 100,   0, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_IRON		)															.setMcfg( 0, Fe				, 1*U, H 				, 1*U, O				, 2*U)																			.setMeltingPoint(2 * Fe.mMeltingPoint / 3), // FeO(OH)
	Calcite					= create		( 9107, "Calcite"				, SET_DULL				, 250, 230, 220, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Ca				, 1*U, C 				, 1*U, O				, 3*U)																			.setMeltingPoint(3 * Fe.mMeltingPoint / 5),
	Cassiterite				= create		( 9108, "Cassiterite"			, SET_METALLIC			, 220, 220, 220, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, FURNACE								).setSmelting(Sn, 3*U/4)					  				.setMcfg( 1, Sn				, 1*U, O 				, 2*U)																									,
	CassiteriteSand			= create		( 9109, "Cassiterite Sand"		, SET_SAND				, 220, 220, 220, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, FURNACE								).setSmelting(Sn, 3*U/4)									.setMcfg( 1, Sn				, 1*U, O 				, 2*U)																									,
	Celestine				= create		( 9110, "Celestine"				, SET_DULL				, 200, 205, 240, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Sr,   U/9)									.setMcfg( 0, Sr				, 1*U, S 				, 1*U, O				, 4*U)																			,
	Chalcopyrite			= create		( 9111, "Chalcopyrite"			, SET_DULL				, 160, 120,  40, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, FURNACE								).setSmelting(Cu, 2*U/9)									.setMcfg( 0, Cu				, 1*U, Fe				, 1*U, S				, 2*U)																			,
	Chalk					= create		( 9112, "Chalk"					, SET_FINE				, 250, 250, 250, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Calcite, 2*U/3)								.setMcfg( 0, Ca				, 1*U, C 				, 1*U, O				, 3*U)																			,
	Chromite				= create		( 9113, "Chromite"				, SET_METALLIC			,  35,  20,  15, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_IRON		).setSmelting(Cr, 2*U/9)									.setMcfg( 0, Fe				, 1*U, Cr				, 2*U, O				, 4*U)																			,
	Cinnabar				= create		( 9114, "Cinnabar"				, SET_ROUGH				, 150,   0,   0, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE, FURNACE								).setSmelting(Hg,   U/3)									.setMcfg( 0, Hg				, 1*U, S 				, 1*U)																									,
	Cobaltite				= create		( 9115, "Cobaltite"				, SET_METALLIC			,  80,  80, 250, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Co,   U/4)									.setMcfg( 0, Co				, 1*U, As				, 1*U, S				, 1*U)																			,
	Cooperite				= create		( 9116, "Cooperite"				, SET_METALLIC			, 255, 255, 200, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE, WASHING_MERCURY						).setSmelting(Pt,   U/3)									.setMcfg( 0, Pt				, 3*U, Ni				, 1*U, S				, 1*U, Pd				, 1*U)													.setLocal("Sheldonite").addIdenticalNames("Sheldonite"),
	Galena					= create		( 9117, "Galena"				, SET_DULL				, 100,  60, 100, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, FURNACE								).setSmelting(Pb,   U/3)									.setMcfg( 0, Pb				, 3*U, Ag				, 3*U, S				, 2*U)																			,
	Garnierite				= create		( 9118, "Garnierite"			, SET_METALLIC			,  50, 200,  70, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Ni,   U/3)									.setMcfg( 0, Ni				, 1*U, O 				, 1*U)																									, 
	Grossular				= create		( 9119, "Grossular"				, SET_ROUGH				, 200, 100,   0, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Ca				, 3*U, Al				, 2*U, Si				, 3*U, O				,12*U)													,
	Ilmenite				= create		( 9120, "Ilmenite"				, SET_METALLIC			,  70,  55,  50, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_IRON		).setSmelting(Ti,   U/4)									.setMcfg( 5, Fe				, 2*U, Ti				, 2*U, O 				, 6*U)																			,
	Magnesite				= create		( 9121, "Magnesite"				, SET_METALLIC			, 250, 250, 180, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Mg,   U/5)									.setMcfg( 0, Mg				, 1*U, C 				, 1*U, O 				, 3*U)																			,
	Magnetite				= create		( 9122, "Magnetite"				, SET_METALLIC			,  30,  30,  30, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_GOLD		)															.setMcfg( 0, Fe				, 3*U, O 				, 4*U)																									.setMeltingPoint(Fe.mMeltingPoint).aspects(TC.METALLUM, 2, TC.MAGNETO, 1),
	Molybdenite				= create		( 9123, "Molybdenite"			, SET_METALLIC			,  25,  25,  25, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Mo,   U/4)							   		.setMcfg( 0, Mo				, 1*U, S 				, 2*U)																									, // MoS2 (also source of Re)
	Powellite				= create		( 9124, "Powellite"				, SET_DULL				, 255, 255,   0, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE											).setSmelting(Mo,   U/9)							  		.setMcfg( 0, Ca				, 1*U, Mo				, 1*U, O				, 4*U)																			,
	Pyrite					= create		( 9125, "Pyrite"				, SET_ROUGH				, 150, 120,  40, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Fe,   U/4)							  		.setMcfg( 0, Fe				, 1*U, S 				, 2*U)																									,
	Pyrolusite				= create		( 9126, "Pyrolusite"			, SET_DULL				, 150, 150, 170, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Mn,   U/4)							  		.setMcfg( 0, Mn				, 1*U, O 				, 2*U)																									,
	Pyrope					= create		( 9127, "Pyrope"				, SET_METALLIC			, 120,  50, 100, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_IRON		)													  		.setMcfg( 0, Al				, 2*U, Mg				, 3*U, Si				, 3*U, O				,12*U)													,
	Scheelite				= create		( 9128, "Scheelite"				, SET_DULL				, 200, 140,  20, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(W ,   U/9)							   		.setMcfg( 0, W 				, 1*U, Ca				, 2*U, O 				, 4*U)																			,
	Spessartine				= create		( 9129, "Spessartine"			, SET_DULL				, 255, 100, 100, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)													   		.setMcfg( 0, Al				, 2*U, Mn				, 3*U, Si				, 3*U, O				,12*U)													,
	Sphalerite				= create		( 9130, "Sphalerite"			, SET_DULL				, 256, 256, 256, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Zn,   U/3)							   		.setMcfg( 0, Zn				, 1*U, S 				, 1*U)																									,
	Stibnite				= create		( 9131, "Stibnite"				, SET_METALLIC			,  70,  70,  70, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE											).setSmelting(Sb,   U/4)							   		.setMcfg( 0, Sb				, 2*U, S 				, 3*U)																									,
	Tetrahedrite			= create		( 9132, "Tetrahedrite"			, SET_DULL				, 200,  32,   0, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE, FURNACE								).setSmelting(Cu,   U/4)							   		.setMcfg( 0, Cu				, 3*U, Sb				, 1*U, S				, 3*U, Fe				, 1*U)													, //Cu3SbS3 + x(Fe,Zn)6Sb2S9
	Tungstate				= create		( 9133, "Tungstate"				, SET_DULL				,  55,  50,  35, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(W ,   U/9)							 		.setMcfg( 0, W 				, 1*U, Li				, 2*U, O				, 4*U)																			,
	Uraninite				= create		( 9134, "Uraninite"				, SET_METALLIC			,  35,  35,  35, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE											).setSmelting(U_238,U/3)							 		.setMcfg( 1, U_238			, 1*U, O 				, 2*U)																									,
	Uvarovite				= create		( 9135, "Uvarovite"				, SET_DIAMOND			, 180, 255, 180, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)													 		.setMcfg( 0, Ca				, 3*U, Cr				, 2*U, Si				, 3*U, O				,12*U)													,
	Wulfenite				= create		( 9136, "Wulfenite"				, SET_DULL				, 255, 128,   0, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE											).setSmelting(Mo,   U/9)									.setMcfg( 0, Pb				, 1*U, Mo				, 1*U, O 				, 4*U)																			,
	YellowLimonite			= create		( 9137, "Yellow Limonite"		, SET_METALLIC			, 200, 200,   0, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_IRON		)															.setMcfg( 0, Fe				, 1*U, H 				, 1*U, O 				, 2*U)																			.setMeltingPoint(2 * Fe.mMeltingPoint / 3), // FeO(OH) + a bit Ni and Co
	Perlite					= create		( 9138, "Perlite"				, SET_DULL				,  30,  20,  30, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE											)															.setMcfg( 0, Obsidian		, 2*U, Water			, 1*U)																									,
	Borax					= create		( 9139, "Borax"					, SET_FINE				, 250, 250, 250, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Na				, 2*U, B				, 4*U, Water			,10*U, O				, 7*U)													,
	PotassiumFeldspar		= create		( 9140, "Potassium Feldspar"	, SET_FINE				, 120,  40,  40, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, K				, 1*U, Al				, 1*U, Si				, 3*U, O				, 8*U)													,
	Biotite					= create		( 9141, "Biotite"				, SET_METALLIC			,  20,  30,  20, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, K				, 1*U, Mg				, 3*U, Al				, 3*U, F				, 2*U, Si				, 3*U, O				,10*U)	,
	Chrysotile				= create		( 9142, "Chrysotile"			, SET_DULL				, 110, 140, 110, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE											)															.setMcfg( 0, Asbestos		, 1*U)																															,
	VanadiumMagnetite		= create		( 9143, "Vanadium Magnetite"	, SET_METALLIC			,  35,  35,  60, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_GOLD		).setSmelting(V ,   U/3)									.setMcfg( 0, Magnetite		, 1*U, V				, 1*U)																									.aspects(TC.METALLUM, 2, TC.MAGNETO, 1), // Mixture of Fe3O4 and V2O5
	Bastnasite				= create		( 9144, "Bastnasite"			, SET_FINE				, 200, 110,  45, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_NEODYMIUM	).setSmelting(Ce,   U/9)									.setMcfg( 0, Ce				, 1*U, C				, 1*U, F				, 1*U, O				, 3*U)													, // (Ce, La, Y)CO3F
	Pentlandite				= create		( 9145, "Pentlandite"			, SET_DULL				, 165, 150,   5, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_IRON		).setSmelting(Ni,   U/3)									.setMcfg( 0, Ni				, 9*U, S				, 8*U)																									, // (Fe,Ni)9S8
	Spodumene				= create		( 9146, "Spodumene"				, SET_DULL				, 190, 170, 170, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Li				, 1*U, Al				, 1*U, Si				, 2*U, O				, 6*U)													, // LiAl(SiO3)2
	Pollucite				= create		( 9147, "Pollucite"				, SET_DULL				, 240, 210, 210, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Cs				, 2*U, Al				, 2*U, Si				, 4*U, Water			, 2*U, O				,12*U)							, // (Cs,Na)2Al2Si4O12 2H2O (also a source of Rb)
	Tantalite				= create		( 9148, "Tantalite"				, SET_METALLIC			, 145,  80,  40, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Ta,   U/9)									.setMcfg( 0, Mn				, 1*U, Ta				, 2*U, O				, 6*U)																			, // (Fe, Mn)Ta2O6 (also source of Nb)
	Lepidolite				= create		( 9149, "Lepidolite"			, SET_FINE				, 240,  50, 140, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, K				, 1*U, Li				, 3*U, Al				, 4*U, F				, 2*U, O				,10*U)							, // K(Li,Al,Rb)3(Al,Si)4O10(F,OH)2
	Glauconite				= create		( 9150, "Glauconite"			, SET_DULL				, 130, 180,  60, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_IRON		)															.setMcfg( 0, K				, 1*U, Mg				, 2*U, Al				, 4*U, H				, 2*U, O				,12*U)							, // (K,Na)(Fe3+,Al,Mg)2(Si,Al)4O10(OH)2
	GlauconiteSand			= create		( 9151, "Glauconite Sand"		, SET_DULL				, 130, 180,  60, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_IRON		)															.setMcfg( 0, K				, 1*U, Mg				, 2*U, Al				, 4*U, H				, 2*U, O				,12*U)							, // (K,Na)(Fe3+,Al,Mg)2(Si,Al)4O10(OH)2
	Vermiculite				= create		( 9152, "Vermiculite"			, SET_METALLIC			, 200, 180,  15, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_IRON		)															.setMcfg( 0, Fe				, 3*U, Al				, 4*U, Si				, 4*U, H				, 2*U, Water			, 4*U, O				,12*U)	, // (Mg+2, Fe+2, Fe+3)3 [(AlSi)4O10] (OH)2 4H2O)
	Bentonite				= create		( 9153, "Bentonite"				, SET_ROUGH				, 245, 215, 210, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Na				, 1*U, Mg				, 6*U, Si				,12*U, H				, 6*U, Water			, 5*U, O				,36*U)	, // (Na,Ca)0.33(Al,Mg)2(Si4O10)(OH)2 nH2O
	FullersEarth			= create		( 9154, "Fullers Earth"			, SET_FINE				, 160, 160, 120, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Mg				, 1*U, Si				, 4*U, H				, 1*U, Water			, 4*U, O				,11*U)							, // (Mg,Al)2Si4O10(OH) 4(H2O)
	Pitchblende				= create		( 9155, "Pitchblende"			, SET_DULL				, 200, 210,   0, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE											).setSmelting(U_238,U/5)									.setMcfg( 0, Uraninite		, 3*U, Th				, 1*U, Pb				, 1*U)																			,
	Malachite				= create		( 9156, "Malachite"				, SET_LAPIS				,   5,  95,   5, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, FURNACE								).setSmelting(Cu,   U/6)									.setMcfg( 0, Cu				, 2*U, C				, 1*U, H				, 2*U, O				, 5*U)													, // Cu2CO3(OH)2
	Mirabilite				= create		( 9157, "Mirabilite"			, SET_DULL				, 240, 250, 210, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Na				, 2*U, S				, 1*U, Water			,10*U, O				, 4*U)													, // Na2SO4 10H2O
	Mica					= create		( 9158, "Mica"					, SET_FINE				, 195, 195, 205, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, K				, 1*U, Al				, 3*U, Si				, 3*U, F				, 2*U, O				,10*U)							, // KAl2(AlSi3O10)(F,OH)2
	Trona					= create		( 9159, "Trona"					, SET_METALLIC			, 135, 135,  95, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Na				, 3*U, C				, 2*U, H				, 1*U, Water			, 2*U, O				, 6*U)							, // Na3(CO3)(HCO3) 2H2O
	Barite					= create		( 9160, "Barite"				, SET_DULL				, 230, 235, 255, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Ba,   U/9)									.setMcfg( 0, Ba				, 1*U, S				, 1*U, O				, 4*U)																			,
	Gypsum					= create		( 9161, "Gypsum"				, SET_DULL				, 230, 230, 250, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Ca				, 1*U, S				, 1*U, Water			, 2*U, O				, 4*U)													, // CaSO4 2H2O
	Alunite					= create		( 9162, "Alunite"				, SET_METALLIC			, 225, 180,  65, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, K				, 1*U, Al				, 3*U, Si				, 2*U, H				, 6*U, O				,14*U)							, // KAl3(SO4)2(OH)6
	Dolomite				= create		( 9163, "Dolomite"				, SET_FLINT				, 225, 205, 205, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										).setSmelting(Calcite, U/2)									.setMcfg( 0, Ca				, 1*U, Mg				, 1*U, C				, 2*U, O				, 6*U)													, // CaMg(CO3)2
	Wollastonite			= create		( 9164, "Wollastonite"			, SET_DULL				, 240, 240, 240, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Ca				, 1*U, Si				, 1*U, O				, 3*U)																			, // CaSiO3
	Zeolite					= create		( 9165, "Zeolite"				, SET_DULL				, 240, 230, 230, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Na				, 1*U, Ca				, 4*U, Si				,27*U, Al				, 9*U, Water			,28*U, O				,72*U)	, // NaCa4(Si27Al9)O72 28(H2O)
	Kyanite					= create		( 9166, "Kyanite"				, SET_FLINT				, 110, 110, 250, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Al				, 2*U, Si				, 1*U, O				, 5*U)																			, // Al2SiO5
	Kaolinite				= create		( 9167, "Kaolinite"				, SET_DULL				, 245, 235, 235, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Al				, 2*U, Si				, 2*U, H				, 4*U, O				, 9*U)													, // Al2Si2O5(OH)4
	Talc					= create		( 9168, "Talc"					, SET_DULL				,  90, 180,  90, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Mg				, 3*U, Si				, 4*U, H				, 2*U, O				,12*U)													, // H2Mg3(SiO3)4
	Soapstone				= create		( 9169, "Soapstone"				, SET_DULL				,  95, 145,  95, 255).add(G_DUST_ORES, ELECTROLYSER, DECOMPOSABLE										)															.setMcfg( 0, Mg				, 3*U, Si				, 4*U, H				, 2*U, O				,12*U)													, // H2Mg3(SiO3)4
	Gneiss					= create		( 9170,	"Gneiss"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Greenschist				= create		( 9171,	"Greenschist"			, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						.setLocal("Green Schist"),
	Greenstone				= create		( 9172,	"Greenstone"			, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Greywacke				= create		( 9173,	"Greywacke"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Graphite				= create		( 9174,	"Graphite"				, SET_DULL				, 128, 128, 128, 255).add(G_DUST_ORES, BRITTLE															)															.setMcfg( 0, C				, 1*U)																															.setQuality( 5.0F,	 32,  2).setSmelting(C, U/2).setBurning(Ash, U/4).setStatsEnergetic(2000, C.mBoilingPoint),
	Graphene				= create		( 9175,	"Graphene"				, SET_DULL				, 128, 128, 128, 255).add(G_DUST_ORES																	)															.setMcfg( 0, C				, 1*U)																															.setQuality( 6.0F,	 32,  1).setSmelting(C, U/2).setBurning(Ash, U/4).setStatsEnergetic(4300, 4400),
	Gabbro					= create		( 9176,	"Gabbro"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Komatiite				= create		( 9177,	"Komatiite"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Siltstone				= create		( 9178,	"Siltstone"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Rhyolite				= create		( 9179,	"Rhyolite"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Quartzite				= create		( 9180,	"Quartzite"				, SET_QUARTZ			, 210, 230, 210, 255).add(G_QUARTZ_ORES, CRYSTALLISABLE, QUARTZ											).setSmelting(Si, U/9)																																																	.setQuality( 3.75F, 32, 1),
	Migmatite				= create		( 9181,	"Migmatite"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Epidote					= create		( 9182,	"Epidote"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Emery					= create		( 9183,	"Emery"					, SET_DULL				, 256, 256, 256, 255).add(G_DUST_ORES																	)																																																						,
	Blueschist				= create		( 9184,	"Blueschist"			, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						.setLocal("Blue Schist"),
	Bluestone				= create		( 9185,	"Bluestone"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Chert					= create		( 9186,	"Chert"					, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Dacite					= create		( 9187,	"Dacite"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Andesite				= create		( 9188,	"Andesite"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Limestone				= create		( 9189,	"Limestone"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		).setSmelting(Calcite, U/2)																																																,
	Shale					= create		( 9190,	"Shale"					, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						,
	Eclogite				= create		( 9191,	"Eclogite"				, SET_DULL				, 256, 256, 256, 255).add(G_DUST																		)																																																						;
	
	public static final OreDictMaterial
	Diamond					= create		( 8300, "Diamond"				, SET_DIAMOND			, 200, 255, 255, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE)																					.setMcfg( 1, C 				, 4*U)																															.aspects(TC.VITREUS, 3, TC.LUCRUM, 4).steal(C).setQuality( 8.0F, 1280,  3).setSmelting(C, U*2).setBurning(Ash, U).setStatsEnergetic(4200, C.mBoilingPoint),
	Emerald					= create		( 8301, "Emerald"				, SET_EMERALD			,  80, 255,  80, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																		.setMcfg( 0, Be				, 3*U, Al				, 2*U, Si				, 6*U, O				,18*U)													.aspects(TC.VITREUS, 3, TC.LUCRUM, 5).setQuality( 7.0F,  256,  2).setSmelting(null, 0),
	Ruby					= create		( 8302, "Ruby"					, SET_RUBY				, 255, 100, 100, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																		.setMcfg( 0, Cr				, 1*U, Al				, 2*U, O				, 3*U)																			.aspects(TC.LUCRUM, 6, TC.VITREUS, 4).setQuality( 7.0F,  256,  2).setSmelting(null, 0),
	FoolsRuby				= create		( 8303,	"Fools Ruby"			, SET_RUBY				, 255, 100, 100, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE)																																																																.aspects(TC.LUCRUM, 2, TC.VITREUS, 2).setLocal("Ruby").steal(Ruby).setSmelting(null, 0),
	Sapphire				= create		( 8304, "Sapphire"				, SET_GEM_VERTICAL		, 100, 100, 200, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																		.setMcfg( 0, Al				, 2*U, O 				, 3*U)																									.aspects(TC.LUCRUM, 5, TC.VITREUS, 3).setQuality( 7.0F,  256,  2).setSmelting(null, 0),
	GreenSapphire			= create		( 8305, "Green Sapphire"		, SET_GEM_HORIZONTAL	, 100, 200, 130, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																		.setMcfg( 0, Al				, 2*U, O 				, 3*U)																									.aspects(TC.LUCRUM, 5, TC.VITREUS, 3).setQuality( 7.0F,  256,  2).setSmelting(null, 0),
	Topaz					= create		( 8306, "Topaz"					, SET_GEM_HORIZONTAL	, 255, 128,   0, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																		.setMcfg( 0, Al				, 2*U, Si				, 1*U, F 				, 2*U, H				, 2*U, O				, 6*U)							.aspects(TC.LUCRUM, 6, TC.VITREUS, 4).setQuality( 7.0F,  256,  3).setSmelting(null, 0),
	BlueTopaz				= create		( 8307, "Blue Topaz"			, SET_GEM_HORIZONTAL	,   0,   0, 255, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																		.setMcfg( 0, Al				, 2*U, Si				, 1*U, F 				, 2*U, H				, 2*U, O				, 6*U)							.aspects(TC.LUCRUM, 6, TC.VITREUS, 4).setQuality( 7.0F,  256,  3).setSmelting(null, 0),
	Tanzanite				= create		( 8308, "Tanzanite"				, SET_GEM_VERTICAL		,  64,   0, 200, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																		.setMcfg( 0, Ca				, 2*U, Al				, 3*U, Si				, 3*U, H				, 1*U, O				,13*U)							.aspects(TC.LUCRUM, 5, TC.VITREUS, 3).setQuality( 7.0F,  256,  2).setSmelting(null, 0),
	Jasper					= create		( 8309,	"Jasper"				, SET_EMERALD			, 200,  80,  80, 150).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE)																																																																.aspects(TC.LUCRUM, 4, TC.VITREUS, 2).setQuality( 7.0F,  256,  2).setSmelting(null, 0),
	Amber					= create		( 8310,	"Amber"					, SET_RUBY				, 255, 128,   0, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE)																																																																.aspects(TC.VINCULUM, 2, TC.VITREUS, 1).setQuality( 4.0F,  128,  2).setSmelting(null, 0),
	Olivine					= create		( 8311, "Olivine"				, SET_RUBY				, 150, 255, 150, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																		.setMcfg( 0, Mg				, 2*U, Fe				, 1*U, SiliconDioxide	, 2*U)																			.aspects(TC.LUCRUM, 4, TC.VITREUS, 2).setQuality( 7.0F,	256,  2).setSmelting(null, 0).addIdenticalNames("Peridot"),
	Opal					= create		( 8312, "Opal"					, SET_OPAL				,   0,   0, 255, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																		.setMcfg( 0, SiliconDioxide	, 1*U)																															.aspects(TC.LUCRUM, 5, TC.VITREUS, 3).setQuality( 7.0F,	256,  2).setSmelting(null, 0),
	Amethyst				= create		( 8313, "Amethyst"				, SET_FLINT				, 210,  50, 210, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																		.setMcfg( 0, SiliconDioxide	, 4*U, Fe				, 1*U)																									.aspects(TC.LUCRUM, 6, TC.VITREUS, 4).setQuality( 7.0F,	256,  3).setSmelting(null, 0),
	GarnetRed				= create		( 8314, "GarnetRed"				, SET_RUBY				, 200,  80,  80, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, CENTRIFUGE)																		.setMcfg( 0, Pyrope			, 3*U, Almandine		, 5*U, Spessartine		, 8*U)																			.aspects(TC.VITREUS, 3).setQuality( 7.0F,	128,  2).setLocal("Red Garnet").setSmelting(null, 0),
	GarnetYellow			= create		( 8315, "GarnetYellow"			, SET_RUBY				, 200, 200,  80, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, CENTRIFUGE)																		.setMcfg( 0, Andradite		, 5*U, Grossular		, 8*U, Uvarovite		, 3*U)																			.aspects(TC.VITREUS, 3).setQuality( 7.0F,	128,  2).setLocal("Yellow Garnet").setSmelting(null, 0),
	Vinteum					= create		( 8316, "Vinteum"				, SET_EMERALD			, 100, 200, 255, 255).add(G_GEM_ORES, MAGICAL	, CRYSTAL, BRITTLE, DECOMPOSABLE, CENTRIFUGE, UNBURNABLE)															.setMcfg( 0, Ma				, 1*U)																															.aspects(TC.VITREUS, 2, TC.PRAECANTIO, 1).setQuality(10.0F,	128,  3),
	Dilithium				= create		( 8317,	"Dilithium"				, SET_DIAMOND			, 255, 250, 250, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, CRYSTALLISABLE, QUARTZ)																																																										.setSmelting(null, 0),
	EnderPearl				= create		( 8318, "EnderPearl"			, SET_SHINY				, 108, 220, 200, 255).add(G_PEARL_TRANSPARENT	, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER, MAGICAL, PEARL)														.setMcfg(10, Be				, 1*U, K				, 4*U, N				, 5*U, Ma				, 6*U)													.aspects(TC.ALIENIS, 4, TC.ITER, 4, TC.PRAECANTIO, 2).setQuality( 1.0F,	 16,  1).setLocal("Enderpearl"),
	EnderEye				= create		( 8319, "EnderEye"				, SET_SHINY				, 160, 250, 230, 255).add(G_PEARL_TRANSPARENT	, CRYSTAL, BRITTLE, DECOMPOSABLE, CENTRIFUGE, MAGICAL, PEARL)														.setMcfg( 1, EnderPearl		, 1*U, Blaze			, 1*U)																									.aspects(TC.SENSUS, 4, TC.ALIENIS, 4, TC.ITER, 4, TC.PRAECANTIO, 3, TC.IGNIS, 2).setQuality( 1.0F,	 16,  1).setLocal("Endereye"),
	NetherStar				= create		( 8320,	"Nether Star"			, SET_NETHERSTAR		, 255, 255, 255, 255).add(G_GEM					, CRYSTAL, BRITTLE, UNBURNABLE, MAGICAL, GLOWING)																																																								.setQuality( 1.0F,   5120,  4),
	Jade					= create		( 8321,	"Jade"					, SET_SHINY				,   0, 100,   0, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE)																																																																.aspects(TC.LUCRUM, 6, TC.VITREUS, 3).setSmelting(null, 0),
	Craponite				= create		( 8322, "Craponite"				, SET_FLINT				, 255, 170, 185, 127).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, DECOMPOSABLE, ELECTROLYSER)																																																									.aspects(TC.LUCRUM, 3, TC.VITREUS, 3).setQuality( 7.0F,	256,  2).setSmelting(null, 0);
	
	public static final OreDictMaterial
	Lazurite				= create		( 8330, "Lazurite"				, SET_LAPIS				, 100, 120, 255, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, CRYSTAL, CRYSTALLISABLE)												   					.setMcfg( 0, Al				, 6*U, Si				, 6*U, Ca				, 8*U, Na				, 8*U)													,
	Sodalite				= create		( 8331, "Sodalite"				, SET_LAPIS				,  20,  20, 255, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, CRYSTAL, CRYSTALLISABLE)												 					.setMcfg( 0, Al				, 3*U, Si				, 3*U, Na				, 4*U, Cl				, 1*U)													,
	Lapis					= create		( 8332, "Lapis"					, SET_LAPIS				,  70,  70, 220, 255).add(G_GEM_ORES, CENTRIFUGE, DECOMPOSABLE, BRITTLE, CRYSTAL, CRYSTALLISABLE)																	.setMcfg( 0, Lazurite		,12*U, Sodalite			, 2*U, Pyrite			, 1*U, Calcite			, 1*U)													.aspects(TC.SENSUS, 1),
	Redstone				= create		( 8333, "Redstone"				, SET_ROUGH				, 200,   0,   0, 255).add(G_DUST_ORES, PLATES, CENTRIFUGE, DECOMPOSABLE, BRITTLE, MELTING, UNBURNABLE, PULVERIZING_CINNABAR)										.setMcfg( 0, Si				, 1*U, Pyrite			, 5*U, Ruby				, 1*U, Hg				, 3*U)													.aspects(TC.MACHINA, 1, TC.POTENTIA, 2).setStatsEnergetic(500, 1500),
	Coal					= create		( 8334, "Coal"					, SET_LIGNITE			,  70,  70,  70, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE).setBurning(DarkAsh, U/4)														.setMcfg( 1, C 				, 2*U)																															.aspects(TC.POTENTIA, 2, TC.IGNIS, 2),
	CoalCoke				= create		( 8349, "Coal Coke"				, SET_LIGNITE			, 140, 140, 170, 255).add(G_GEM, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE).setBurning(DarkAsh, U/4)															.setMcfg( 1, C 				, 2*U)																															.aspects(TC.POTENTIA, 3, TC.IGNIS, 1).steal(Coal),
	HydratedCoal			= create		( 8335, "Hydrated Coal"			, SET_LIGNITE			,  70,  70, 100, 255).add(G_DUST, CENTRIFUGE, DECOMPOSABLE, BRITTLE)																								.setMcfg( 8, Coal			, 8*U, Water			, 1*U)																									,
	Charcoal				= create		( 8336, "Charcoal"				, SET_LIGNITE			, 100,  70,  70, 255).add(G_GEM, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE).setBurning(Ash, U/4)																.setMcfg( 0, C 				, 1*U)																															.aspects(TC.POTENTIA, 2, TC.IGNIS, 2),
	Lignite					= create		( 8337, "Lignite"				, SET_LIGNITE			, 100,  70,  70, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, BRITTLE, FLAMMABLE).setBurning(DarkAsh, U/9)														.setMcfg( 0, C				, 2*U, Water			, 4*U, DarkAsh			, 1*U)																			.setLocal("Lignite Coal"),
	Monazite				= create		( 8338, "Monazite"				, SET_DIAMOND			,  50,  70,  50, 255).add(G_GEM_ORES, ELECTROLYSER, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_NEODYMIUM, CRYSTAL, BRITTLE, CRYSTALLISABLE)							.setMcfg( 0, RareEarth		, 1*U, Phosphate		, 1*U)																									, // Wikipedia: (Ce, La, Nd, Th, Sm, Gd)PO4 Monazite also smelt-extract to Helium, it is brown like the rare earth Item Monazite sand deposits are inevitably of the monazite-(Ce) composition. Typically, the lanthanides in such monazites contain about 4548% cerium, about 24% lanthanum, about 17% neodymium, about 5% praseodymium, and minor quantities of samarium, gadolinium, and yttrium. Europium concentrations tend to be low, about 0.05% Thorium content of monazite is variable and sometimes can be up to 2030%
	Teslatite				= create		( 8339,	"Teslatite"				, SET_SHINY				,  60, 180, 200, 255).add(G_DUST_ORES, PLATES, BRITTLE, UNBURNABLE, MELTING)																																																													.aspects(TC.ELECTRUM, 2).setStatsEnergetic(1500, 3000),
	Nikolite				= create		( 8340,	"Nikolite"				, SET_SHINY				,  60, 180, 200, 255).add(G_DUST_ORES, PLATES, BRITTLE, UNBURNABLE, MELTING)																																																													.aspects(TC.ELECTRUM, 2).setStatsEnergetic(1500, 3000),
	Glowstone				= create		( 8341,	"Glowstone"				, SET_SHINY				, 255, 255,   0, 255).add(G_DUST_ORES, PLATES, BRITTLE, UNBURNABLE, MELTING, CRYSTAL, GLOWING, LIGHTING)																																																						.aspects(TC.LUX, 2, TC.SENSUS, 1),
	Firestone				= create		( 8342,	"Firestone"				, SET_QUARTZ			, 200,  20,   0, 255).add(G_QUARTZ_ORES, CRYSTAL, BRITTLE, CRYSTALLISABLE, MAGICAL, QUARTZ, UNBURNABLE, BURNING).setQuality( 6.0F, 1280, 3)																																														,
	Force					= create		( 8343,	"Force"					, SET_DIAMOND			, 255, 255,   0, 255).add(G_GEM_ORES, G_INGOT_MACHINE_ORES).add(CRYSTAL, MAGICAL, UNBURNABLE, GLOWING).setQuality(10.0F, 128, 3)																																																.aspects(TC.POTENTIA, 4),
	Forcicium				= create		( 8344,	"Forcicium"				, SET_DIAMOND			,  50,  50,  70, 255).add(G_QUARTZ_ORES, ELECTROMAGNETIC_SEPARATION_NEODYMIUM, MAGICAL, CRYSTAL, BRITTLE, CRYSTALLISABLE)																																																		.aspects(TC.POTENTIA, 2),
	Forcillium				= create		( 8345,	"Forcillium"			, SET_DIAMOND			,  50,  50,  70, 255).add(G_QUARTZ_ORES, ELECTROMAGNETIC_SEPARATION_NEODYMIUM, MAGICAL, CRYSTAL, BRITTLE, CRYSTALLISABLE)																																																		.aspects(TC.POTENTIA, 2),
	NetherQuartz			= create		( 8346,	"Nether Quartz"			, SET_QUARTZ			, 230, 210, 210, 255).add(G_QUARTZ_ORES, BRITTLE, QUARTZ, CRYSTALLISABLE)																																																														.aspects(TC.POTENTIA, 1, TC.VITREUS, 1).setQuality( 2.5F, 32, 1).setSmelting(Si, U/9),
	CertusQuartz			= create		( 8347,	"Certus Quartz"			, SET_QUARTZ			, 210, 210, 230, 255).add(G_QUARTZ_ORES, BRITTLE, QUARTZ, CRYSTALLISABLE)																																																														.aspects(TC.POTENTIA, 1, TC.VITREUS, 1).setQuality( 5.0F, 32, 1).setSmelting(Si, U/9),
	ChargedCertusQuartz		= create		( 8348,	"Charged Certus Quartz"	, SET_QUARTZ			, 210, 210, 230, 255).add(G_QUARTZ_ORES, BRITTLE, QUARTZ, GLOWING)																									.setMcfg( 0, CertusQuartz	, 1*U)																															.aspects(TC.POTENTIA, 2, TC.VITREUS, 1).steal(CertusQuartz).setSmelting(Si, U/9).setPulver(CertusQuartz, U),
	
	InfusedDull				= create		(-8350,	"Infused Dull"			, SET_SHARDS			, 100, 100, 100, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE)																																																										.aspects(TC.PRAECANTIO, 1, TC.VACUOS	, 2).setQuality(32.0F,   64,  3),
	InfusedVis				= create		(-8351,	"Infused Vis"			, SET_SHARDS			, 255,   0, 255, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.AURAM		, 2).setQuality( 8.0F,   64,  3),
	InfusedAir				= create		( 8352,	"Infused Air"			, SET_SHARDS			, 255, 255,   0, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.AER		, 2).setQuality( 8.0F,   64,  3),
	InfusedFire				= create		( 8353,	"Infused Fire"			, SET_SHARDS			, 255,   0,   0, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.IGNIS		, 2).setQuality( 8.0F,   64,  3),
	InfusedEarth			= create		( 8354,	"Infused Earth"			, SET_SHARDS			,   0, 255,   0, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.TERRA		, 2).setQuality( 8.0F,  256,  3),
	InfusedWater			= create		( 8355,	"Infused Water"			, SET_SHARDS			,   0,   0, 255, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.AQUA		, 2).setQuality( 8.0F,   64,  3),
	InfusedEntropy			= create		( 8356,	"Infused Entropy"		, SET_SHARDS			,  62,  62,  62, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.PERDITIO	, 2).setQuality(32.0F,   64,  4),
	InfusedOrder			= create		( 8357,	"Infused Order"			, SET_SHARDS			, 252, 252, 252, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.ORDO		, 2).setQuality( 8.0F,   64,  3),
	InfusedBalance			= create		(-8358,	"Infused Balance"		, SET_SHARDS			, 252, 252, 252, 255).add(G_GEM_ORES_TRANSPARENT, CRYSTAL, BRITTLE, MAGICAL, UNBURNABLE, GLOWING)																																																								.aspects(TC.PRAECANTIO, 1, TC.ORDO, 1, TC.PERDITIO, 1, TC.IGNIS, 1, TC.AER, 1, TC.AQUA, 1, TC.TERRA, 1).setQuality(32.0F,  256,  4);
	
	public static final OreDictMaterial
	Pyrotheum				= create		( 8212, "Pyrotheum"				, SET_FIERY				, 255, 128,   0, 255).add(G_DUST, CONTAINERS, CENTRIFUGE, DECOMPOSABLE)																								.setMcfg( 1, Coal			, 1*U, Redstone			, 1*U, Blaze			, 1*U)																			.aspects(TC.PRAECANTIO, 2, TC.IGNIS, 1),
	Cryotheum				= create		( 8213, "Cryotheum"				, SET_SHINY				, 128, 128, 255, 255).add(G_DUST, CONTAINERS)																														.setMcfg( 1, N				, 3*U)																															.aspects(TC.PRAECANTIO, 2, TC.GELUM, 1);
	
	public static final OreDictMaterial
	Stone					= create		( 8500, "Stone"					, SET_STONE				, 205, 205, 205, 255).add(G_STONE, STONE, BRITTLE, FURNACE, UNRECYCLABLE)																																																														.aspects(TC.TERRA, 1).setQuality(4.0F, 16, 1).setMeltingPoint(1000).setRGBaLiquid( 64,  64,  64, 255),
	Concrete				= create		( 8501, "Concrete"				, SET_STONE				, 100, 100, 100, 255).add(G_STONE, STONE, BRITTLE, FURNACE, MELTING)																								.setMcfg( 0, Stone			, 1*U)																															.aspects(TC.TERRA, 1).setQuality(3.0F, 32, 0).setMeltingPoint(500),
	Netherrack				= create		( 8502,	"Netherrack"			, SET_STONE				, 200,   0,   0, 255).add(G_STONE, STONE, BRITTLE, FURNACE, UNBURNABLE, FLAMMABLE)																																																												.aspects(TC.IGNIS, 1).setMeltingPoint(1500),
	NetherBrick				= create		( 8503,	"Nether Brick"			, SET_STONE				, 100,   0,   0, 255).add(G_DUST , STONE, BRITTLE, FURNACE, UNBURNABLE)																																																															.aspects(TC.IGNIS, 1).setPulver(Netherrack, U).addIdenticalNames("BrickNether").setMeltingPoint(1800),
	Endstone				= create		( 8504,	"Endstone"				, SET_STONE				, 256, 256, 256, 255).add(G_STONE, STONE, BRITTLE)																																																																				.aspects(TC.TENEBRAE, 1),
	Basalt					= create		( 8505, "Basalt"				, SET_STONE				,  30,  20,  20, 255).add(G_STONE, STONE, BRITTLE, CENTRIFUGE, DECOMPOSABLE)																						.setMcfg( 0, Olivine		, 1*U, Calcite			, 3*U, Flint			, 8*U, DarkAsh			, 4*U)													.aspects(TC.TENEBRAE, 1).setMeltingPoint(1673),
	Marble					= create		( 8506, "Marble"				, SET_STONE				, 200, 200, 200, 255).add(G_STONE, STONE, BRITTLE, CENTRIFUGE, DECOMPOSABLE)																						.setMcfg( 0, Mg				, 1*U, Calcite			, 7*U)																									.aspects(TC.PERFODIO, 1).setSmelting(Calcite, 2*U/3),
	GraniteRed				= create		( 8507, "GraniteRed"			, SET_STONE				, 255,   0, 128, 255).add(G_STONE, STONE, BRITTLE, ELECTROLYSER, DECOMPOSABLE)																						.setMcfg( 0, Al				, 2*U, PotassiumFeldspar, 1*U, O				, 3*U)																			.aspects(TC.TUTAMEN, 1).setQuality(4.0F, 64, 3).setLocal("Red Granite").setMeltingPoint(1500),
	GraniteBlack			= create		( 8508, "GraniteBlack"			, SET_STONE				,  10,  10,  10, 255).add(G_STONE, STONE, BRITTLE, CENTRIFUGE, DECOMPOSABLE)																						.setMcfg( 0, SiliconDioxide	, 4*U, Biotite			, 1*U)																									.aspects(TC.TUTAMEN, 1).setQuality(4.0F, 64, 3).setLocal("Black Granite").setMeltingPoint(1500),
	Redrock					= create		( 8509, "Redrock"				, SET_STONE				, 255,  80,  50, 255).add(G_STONE, STONE, BRITTLE, CENTRIFUGE, DECOMPOSABLE)																						.setMcfg( 0, Calcite		, 2*U, Flint			, 1*U, Clay				, 1*U)																			.aspects(TC.TERRA, 1).addIdenticalNames("RedRock"),
	Gravel					= create		( 8510,	"Gravel"				, SET_ROUGH				, 205, 205, 205, 255).add(G_DUST , STONE, BRITTLE)																																																																				.aspects(TC.PERDITIO, 1);
	
	static {Netherrack.setSmelting(NetherBrick, U);}
	
	public static final OreDictMaterial
	Electrum				= alloymachore	( 8600, "Electrum"				, SET_SHINY				, 255, 255, 100, 255).add(CENTRIFUGE, WIRES).setQuality(12.0F,   64,  2)																							.setMcfg( 0, Ag				, 1*U, Au				, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 2).makeIntoSimpleAlloy(),
	SterlingSilver			= alloymachine	( 8601, "Sterling Silver"		, SET_SHINY				, 250, 220, 225, 255).add(CENTRIFUGE).setQuality(13.0F,  128,  2)																									.setMcfg( 0, Cu				, 1*U, Ag				, 4*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 2).makeIntoSimpleAlloy(),
	RoseGold				= alloymachine	( 8602, "Rose Gold"				, SET_SHINY				, 255, 230,  30, 255).add(CENTRIFUGE).setQuality(14.0F,  128,  2)																									.setMcfg( 0, Cu				, 1*U, Au				, 4*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 2).makeIntoSimpleAlloy(),
	
	Brass					= alloymachine	( 8620, "Brass"					, SET_METALLIC			, 255, 180,   0, 255).add(CENTRIFUGE).setQuality( 7.0F,	  96,  1)																									.setMcfg( 0, Cu				, 3*U, Zn				, 1*U)																									.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).setStatsEnergetic(1160, Cu.mBoilingPoint).makeIntoSimpleAlloy(),
	CobaltBrass				= alloymachine	( 8621, "Cobalt Brass"			, SET_METALLIC			, 180, 180, 160, 255).add(CENTRIFUGE).setQuality( 8.0F,  256,  2)																									.setMcfg( 0, Brass			, 7*U, Al				, 1*U, Co				, 1*U)																			.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).makeIntoSimpleAlloy(),
	
	Bronze					= alloymachine	( 8610, "Bronze"				, SET_METALLIC			, 255, 128,   0, 255).add(CENTRIFUGE).setQuality( 6.0F,  192,  2)																									.setMcfg( 0, Cu				, 3*U, Sn				, 1*U)																									.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).setStatsEnergetic(Cu.mMeltingPoint, Cu.mBoilingPoint).makeIntoSimpleAlloy(),
	BlackBronze				= alloymachine	( 8611, "Black Bronze"			, SET_DULL				, 100,  50, 125, 255).add(CENTRIFUGE).setQuality(12.0F,	 256,  2)																									.setMcfg( 0, Cu				, 3*U, Electrum			, 2*U)																									.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).makeIntoSimpleAlloy(),
	BismuthBronze			= alloymachine	( 8612, "Bismuth Bronze"		, SET_DULL				, 100, 125, 125, 255).add(CENTRIFUGE).setQuality( 8.0F,  256,  2)																									.setMcfg( 0, Bi				, 1*U, Brass			, 4*U)																									.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).makeIntoSimpleAlloy(),
	
	WroughtIron				= metalmachine	( 8643, "Wrought Iron"			, SET_METALLIC			, 200, 180, 180, 255).add()																																			.setMcfg( 0, Fe				, 1*U)																															.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).setPulver(Fe		, U)																																						.steal(Fe).setQuality( 6.0F,  384,  2).addIdenticalNames("WrougtIron").setStatsEnergetic(Fe.mMeltingPoint + 200, Fe.mBoilingPoint).makeIntoSimpleAlloy(),
	Steel					= alloymachine	( 8630, "Steel"					, SET_METALLIC			, 130, 130, 130, 255).add(CENTRIFUGE, WIRES).setQuality( 6.0F,  512,  2)																							.setMcfg(50, WroughtIron	,50*U, C 				, 1*U)																									.aspects(TC.METALLUM, 2, TC.ORDO, 1).makeIntoSimpleAlloy(),
	BlackSteel				= alloymachine	( 8631, "Black Steel"			, SET_METALLIC			, 100, 100, 100, 255).add(CENTRIFUGE).setQuality( 6.5F,  768,  2)																									.setMcfg( 0, Ni				, 1*U, BlackBronze		, 1*U, Steel			, 3*U)																			.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.INSTRUMENTUM, 1).makeIntoSimpleAlloy(),
	RedSteel				= alloymachine	( 8632, "Red Steel"				, SET_METALLIC			, 140, 100, 100, 255).add(CENTRIFUGE).setQuality( 7.0F,  896,  2)																									.setMcfg( 0, SterlingSilver	, 1*U, BismuthBronze	, 1*U, Steel			, 2*U, BlackSteel		, 4*U)													.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.INSTRUMENTUM, 1).makeIntoSimpleAlloy(),
	BlueSteel				= alloymachine	( 8633, "Blue Steel"			, SET_METALLIC			, 100, 100, 140, 255).add(CENTRIFUGE).setQuality( 7.5F, 1024,  2)																									.setMcfg( 0, RoseGold		, 1*U, Brass			, 1*U, Steel			, 2*U, BlackSteel		, 4*U)													.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.INSTRUMENTUM, 1).makeIntoSimpleAlloy(),
	DamascusSteel			= alloymachine	( 8634, "Damascus Steel"		, SET_METALLIC			, 110, 110, 110, 255).add(CENTRIFUGE).setQuality( 8.0F, 1280,  2)																									.setMcfg(50, Steel			,50*U, V 				, 1*U, W 				, 1*U)																			.aspects(TC.METALLUM, 2, TC.ORDO, 2, TC.INSTRUMENTUM, 1),
	TungstenSteel			= alloymachine	( 8635, "Tungstensteel"			, SET_METALLIC			, 100, 100, 160, 255).add(CENTRIFUGE).setQuality(10.0F, 5120,  4)																									.setMcfg( 0, Steel			, 1*U, W				, 1*U)																									.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.TUTAMEN, 1).addIdenticalNames("TungstenSteel", "Wolframsteel", "WolframSteel").makeIntoSimpleAlloy(),
	HSLA					= alloymachine	( 8637,	"HSLA-Steel"			, SET_METALLIC			, 128, 128, 128, 255).add()																																			.setMcfg(50, WroughtIron	,50*U, C 				, 1*U)																									.aspects(TC.METALLUM, 2, TC.ORDO, 1).steal(Steel).addIdenticalNames("HSLA"),
	
	AnnealedCopper			= metal			( 8640, "Annealed Copper"		, SET_SHINY				, 255, 120,  20, 255).add(FURNACE, WIRES)																															.setMcfg( 0, Cu				, 1*U)																															.aspects(TC.METALLUM, 2, TC.PERMUTATIO, 1			).setPulver(Cu		, U)																																						.steal(Cu).setStatsEnergetic(2800, Cu.mBoilingPoint).makeIntoSimpleAlloy(),
	DeepIron				= metalmachore	( 8641, "Deep Iron"				, SET_METALLIC			, 150, 140, 140, 255).add(CENTRIFUGE, DECOMPOSABLE)																													.setMcfg( 0, Fe				, 1*U)																															.aspects(TC.METALLUM, 2, TC.MAGNETO, 1				)																																												.steal(Fe).setQuality( 6.0F,  384,  2),
	PigIron					= metalmachore	( 8642, "Pig Iron"				, SET_METALLIC			, 200, 180, 180, 255).add()																																			.setMcfg( 0, WroughtIron	, 1*U)																															.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1			).setPulver(WroughtIron, U).setSmelting(WroughtIron, U)																															.steal(WroughtIron).setQuality( 6.0F,  384,  2),
	IronCompressed			= alloymachine	( 8644, "IronCompressed"		, SET_METALLIC			, 128, 128, 128, 255).add(CENTRIFUGE)																																.setMcfg( 0, Fe				, 1*U)																															.aspects(TC.METALLUM, 2, TC.TERRA, 1).steal(Fe).setLocal("Compressed Iron"),
	IronMagnetic			= metalmachine	( 8645, "IronMagnetic"			, SET_MAGNETIC			, 200, 200, 200, 255).add()																																			.setMcfg( 0, Fe				, 1*U)																															.aspects(TC.METALLUM, 2, TC.MAGNETO, 1				).setBending(Fe		, U).setCompressing(Fe		, U).setPulver(Fe		, U).setSmashing(Fe		, U).setSmelting(Fe		, U).setWorking(Fe		, U).setForging(Fe		, U)	.steal(Fe).setLocal("Magnetic Iron"),
	SteelMagnetic			= metalmachine	( 8646, "SteelMagnetic"			, SET_MAGNETIC			, 128, 128, 128, 255).add()																																			.setMcfg( 0, Steel			, 1*U)																															.aspects(TC.METALLUM, 1, TC.ORDO, 1, TC.MAGNETO, 1	).setBending(Steel	, U).setCompressing(Steel	, U).setPulver(Steel	, U).setSmashing(Steel	, U).setSmelting(Steel	, U).setWorking(Steel	, U).setForging(Steel	, U)	.steal(Steel).setLocal("Magnetic Steel"),
	NeodymiumMagnetic		= metalmachine	( 8647, "NeodymiumMagnetic"		, SET_MAGNETIC			, 100, 100, 100, 255).add()																																			.setMcfg( 0, Nd				, 1*U)																															.aspects(TC.METALLUM, 1, TC.MAGNETO, 3				).setBending(Fe		, U).setCompressing(Nd		, U).setPulver(Nd		, U).setSmashing(Nd		, U).setSmelting(Nd		, U).setWorking(Nd		, U).setForging(Nd		, U)	.steal(Nd).setLocal("Magnetic Neodymium"),
	DarkIron				= metalmachore	( 8648,	"Dark Iron"				, SET_DULL				,  55,  40,  60, 255).add()																																																																										.aspects(TC.METALLUM, 2, TC.IGNIS, 1).steal(Fe).setQuality( 7.0F,	384,  3).setStatsEnergetic(Steel.mMeltingPoint + 200, Fe.mBoilingPoint).addIdenticalNames("FzDarkIron", "FZDarkIron"),
	MeteoricIron			= metalmachore	( 8649,	"Meteoric Iron"			, SET_METALLIC			, 100,  50,  80, 255).add()																																			.setMcfg( 0, Fe				, 1*U)																															.aspects(TC.METALLUM, 2, TC.MAGNETO, 1).steal(Fe).setQuality( 6.0F,	384,  2).setStatsEnergetic(Fe.mMeltingPoint + 200, Fe.mBoilingPoint + 200),
	MeteoricSteel			= metalmachine	( 8650,	"Meteoric Steel"		, SET_METALLIC			,  50,  25,  40, 255).add()																																			.setMcfg( 0, Steel			, 1*U)																															.aspects(TC.METALLUM, 2, TC.MAGNETO, 1, TC.ORDO, 1).steal(Steel).setQuality( 6.0F,	768,  2).setStatsEnergetic(Steel.mMeltingPoint + 200, Steel.mBoilingPoint + 200),
	
	BlueAlloy				= alloy			( 8659, "Blue Alloy"			, SET_DULL				, 100, 180, 255, 255).add(CENTRIFUGE, WIRES)																														.setMcfg( 1, Ag				, 1*U, Nikolite			, 4*U)																									.aspects(TC.ELECTRUM, 3).setStatsEnergetic(1400, Ag.mBoilingPoint).makeIntoSimpleAlloy(),
	RedAlloy				= alloy			( 8660, "Red Alloy"				, SET_DULL				, 200,   0,   0, 255).add(CENTRIFUGE, WIRES)																														.setMcfg( 1, Cu				, 1*U, Redstone			, 4*U)																									.aspects(TC.MACHINA, 3).setStatsEnergetic(1400, Cu.mBoilingPoint).makeIntoSimpleAlloy(),
	Invar					= alloymachine	( 8661, "Invar"					, SET_METALLIC			, 180, 180, 120, 255).add(CENTRIFUGE).setQuality( 6.0F,	256,  2)																									.setMcfg( 0, WroughtIron	, 2*U, Ni				, 1*U)																									.aspects(TC.METALLUM, 2, TC.GELUM, 1).makeIntoSimpleAlloy(),
	Cupronickel				= alloymachine	( 8662, "Cupronickel"			, SET_METALLIC			, 227, 150, 128, 255).add(CENTRIFUGE).setQuality( 6.0F,	 64,  1)																									.setMcfg( 0, Cu				, 1*U, Ni				, 1*U)																									.makeIntoSimpleAlloy(),
	Nichrome				= alloymachine	( 8663, "Nichrome"				, SET_METALLIC			, 205, 206, 246, 255).add(CENTRIFUGE).setQuality( 6.0F,	 64,  2)																									.setMcfg( 0, Ni				, 4*U, Cr				, 1*U)																									.makeIntoSimpleAlloy(),
	Kanthal					= alloymachine	( 8664, "Kanthal"				, SET_METALLIC			, 194, 210, 223, 255).add(CENTRIFUGE).setQuality( 6.0F,	 64,  2)																									.setMcfg( 0, WroughtIron	, 1*U, Al				, 1*U, Cr				, 1*U)																			.makeIntoSimpleAlloy(),
	Magnalium				= alloymachine	( 8665, "Magnalium"				, SET_DULL				, 200, 190, 255, 255).add(CENTRIFUGE).setQuality( 6.0F,	256,  2)																									.setMcfg( 0, Mg				, 1*U, Al				, 2*U)																									.makeIntoSimpleAlloy(),
	StainlessSteel			= alloymachine	( 8636, "Stainless Steel"		, SET_SHINY				, 200, 200, 220, 255).add(CENTRIFUGE).setQuality( 7.0F,  480,  2)																									.setMcfg( 0, WroughtIron	, 4*U, Invar			, 3*U, Cr				, 1*U, Mn				, 1*U)													.aspects(TC.METALLUM, 2, TC.ORDO, 1, TC.VITREUS, 1).makeIntoSimpleAlloy(),
	Ultimet					= alloymachine	( 8666, "Ultimet"				, SET_SHINY				, 180, 180, 230, 255).add(ELECTROLYSER).setQuality( 6.0F,	512,  3)																								.setMcfg( 0, Co				, 5*U, Cr				, 2*U, Ni				, 1*U, Mo				, 1*U)													.makeIntoSimpleAlloy(), // 54% Co, 26% Cr, 9% Ni, 5% Mo, 3% Fe, 2% W, 0.8% Mn, 0.3% Si, 0.08% Ni and 0.06% C
	TinAlloy				= alloymachine	( 8667, "Tin Alloy"				, SET_METALLIC			, 200, 200, 200, 255).add(CENTRIFUGE).setQuality( 6.5F,	 96,  2)																									.setMcfg( 0, Sn				, 1*U, WroughtIron		, 1*U)																									.aspects(TC.METALLUM, 2, TC.INSTRUMENTUM, 1).makeIntoSimpleAlloy(),
	BatteryAlloy			= alloy			( 8668, "Battery Alloy"			, SET_DULL				, 156, 124, 160, 255).add(CENTRIFUGE)														  																		.setMcfg( 0, Pb				, 4*U, Sb				, 1*U)																									.makeIntoSimpleAlloy(),
	SolderingAlloy			= alloy			( 8669, "Soldering Alloy"		, SET_DULL				, 220, 220, 230, 255).add(BRITTLE, CENTRIFUGE, SOLDERING_MATERIAL, SOLDERING_MATERIAL_GOOD, WIRES)																	.setMcfg( 0, Sn				, 9*U, Sb				, 1*U)																									.makeIntoSimpleAlloy(),
	ShadowIron				= metalmachore	( 8670, "Shadow Iron"			, SET_METALLIC			, 120, 120, 120, 255).add(MAGICAL, CENTRIFUGE, DECOMPOSABLE).setQuality( 6.0F,  384,  2)																			.setMcfg( 3, Fe				, 3*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.TENEBRAE, 1),
	ShadowSteel				= alloymachine	( 8671, "Shadow Steel"			, SET_METALLIC			,  90,  90,  90, 255).add(MAGICAL, CENTRIFUGE).setQuality( 6.0F,  768,  2)																							.setMcfg( 3, Steel			, 3*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.TENEBRAE, 1, TC.ORDO, 1),
	IronWood				= alloymachine	( 8672, "IronWood"				, SET_WOOD				, 150, 140, 110, 255).add(MAGICAL, WOOD, CENTRIFUGE, FURNACE).setQuality( 6.0F,  384,  2).setLocal("Ironwood")														.setMcfg(18, WroughtIron	, 9*U, LiveRoot			, 9*U, Au				, 1*U)																			.aspects(TC.METALLUM, 2, TC.ARBOR, 1, TC.PRAECANTIO, 1).makeIntoSimpleAlloy(),
	Steeleaf				= alloymachine	( 8673, "Steeleaf"				, SET_LEAF				,  50, 127,  50, 255).add(MAGICAL, WOOD, CENTRIFUGE).setQuality( 8.0F,  768,  3).setSmelting(Steel, U / 4)															.setMcfg( 1, Steel			, 1*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.HERBA, 1, TC.PRAECANTIO, 1),
	Knightmetal				= alloymachine	( 8674, "Knightmetal"			, SET_METALLIC			, 210, 240, 200, 255).add(MAGICAL, CENTRIFUGE).setQuality( 8.0F, 1024,  3).addIdenticalNames("KnightMetal")															.setMcfg( 2, Steel			, 2*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.TUTAMEN, 1, TC.PRAECANTIO, 1).setStatsEnergetic(Steel.mMeltingPoint+100, Steel.mBoilingPoint+100),
	FierySteel				= alloymachine	( 8675,	"Fiery Steel"			, SET_FIERY				,  64,   0,   0, 255).add(MAGICAL, UNBURNABLE, BURNING, GLOWING).setQuality( 8.0F,  256,  3)																		.setMcfg( 1, Steel			, 1*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.IGNIS, 1, TC.PRAECANTIO, 1).setStatsEnergetic(Steel.mBoilingPoint-200, Steel.mBoilingPoint+500),
	AstralSilver			= alloymachore	( 8676, "Astral Silver"			, SET_SHINY				, 230, 230, 255, 255).add(MAGICAL, CENTRIFUGE).setQuality(10.0F,	 64,  2)																						.setMcfg( 2, Ag				, 2*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 1, TC.PRAECANTIO, 1),
	Midasium				= alloymachore	( 8677, "Midasium"				, SET_SHINY				, 255, 200,  40, 255).add(MAGICAL, CENTRIFUGE, WASHING_MERCURY).setQuality(12.0F,	 64,  2)																		.setMcfg( 2, Au				, 2*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 2, TC.PRAECANTIO, 1),
	Mithril					= alloymachore	( 8678, "Mithril"				, SET_SHINY				, 255, 255, 210, 255).add(MAGICAL, CENTRIFUGE, WASHING_MERCURY).setQuality(14.0F,	 64,  3)																		.setMcfg( 2, Pt				, 2*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.LUCRUM, 3, TC.PRAECANTIO, 1),
	Thaumium				= alloymachine	( 8679, "Thaumium"				, SET_METALLIC			, 150, 100, 200, 255).add(MAGICAL).setQuality(12.0F,	 256,  3)																									.setMcfg( 1, Fe				, 1*U, Ma				, 1*U)																									.aspects(TC.METALLUM, 2, TC.PRAECANTIO, 1).setStatsEnergetic(Fe.mMeltingPoint+500, Fe.mBoilingPoint+1000),
	DarkThaumium			= alloymachine	( 8680, "Dark Thaumium"			, SET_METALLIC			, 100,  75,  75, 255).add(MAGICAL).setQuality(12.0F,	 512,  3)																																																																,
	VoidMetal				= alloymachine	( 8681,	"Void Metal"			, SET_METALLIC			,  30,  10,  30, 100).add(MAGICAL).setQuality(12.0F,	2048,  4).addIdenticalNames("Void")																																																										.aspects(TC.METALLUM, 7, TC.TENEBRAE, 8, TC.VACUOS, 8, TC.ALIENIS, 2, TC.HERBA, 1).setStatsEnergetic(3000, 5000),
	Osmiridium				= alloymachine	( 8682,	"Osmiridium"			, SET_METALLIC			, 100, 100, 255, 255).add().setQuality(8.0F, 3000, 4)																																																																			,
	Sunnarium				= alloymachine	( 8683,	"Sunnarium"				, SET_SHINY				, 255, 255,   0, 255).add()																																																																										,
	NaquadahAlloy			= alloymachine	( 8684,	"Naquadah Alloy"		, SET_METALLIC			,  40,  40,  40, 255).add().setQuality(8.0F, 5120,  5)																																																																			.aspects(TC.METALLUM, 4, TC.NEBRISUM, 1),
	ChromiumDioxide 		= alloy			( 8685, "Chromium Dioxide"		, SET_DULL				,  10,  20,  10, 255).add(ELECTROLYSER)																																.setMcfg( 1, Cr				, 1*U, O 				, 2*U)																									.aspects(TC.METALLUM, 2, TC.MACHINA, 1).setQuality(11.0F,  256,  3).setMeltingPoint(650),
	VanadiumGallium			= alloy			( 8686, "Vanadium-Gallium"		, SET_SHINY				, 128, 128, 140, 255).add(BRITTLE, CENTRIFUGE)														 																.setMcfg( 0, V 				, 3*U, Ga				, 1*U)																									.makeIntoSimpleAlloy(),
	YttriumBariumCuprate	= alloy			( 8687, "Yttrium-Barium-Cuprate", SET_METALLIC			,  80,  64,  70, 255).add(BRITTLE)																																	.setMcfg( 0, Y 				, 1*U, Ba				, 2*U, Cu				, 3*U, O				, 7*U)													.setMeltingPoint(1200),
	NiobiumNitride			= alloy			( 8688, "Niobium Nitride"		, SET_DULL				,  29,  41,  29, 255).add(BRITTLE, ELECTROLYSER)			 								 																		.setMcfg( 0, Nb				, 1*U, N 				, 1*U)																									.setMeltingPoint(2573), // Anti-Reflective Material
	NiobiumTitanium			= alloy			( 8689, "Niobium Titanium"		, SET_DULL				,  29,  29,  41, 255).add(BRITTLE, CENTRIFUGE)														  																.setMcfg( 0, Nb				, 1*U, Ti				, 1*U)																									.makeIntoSimpleAlloy(),
	AluminiumBrass			= alloymachine	( 8700,	"Aluminium Brass"		, SET_METALLIC			, 256, 256, 256, 255).add().setQuality(6.0F, 64, 2).addIdenticalNames("AluminumBrass")																																																											.aspects(TC.STRONTIO, 1),
	Aredrite				= metalore		( 8701,	"Aredrite"				, SET_NONE				, 255, 255,   0, 255).add().setQuality(6.0F, 64,  2)																																																																			.aspects(TC.STRONTIO, 1),
	Alumite					= alloymachine	( 8702, "Alumite"				, SET_METALLIC			, 256, 256, 256, 255).add(CENTRIFUGE).setQuality( 1.5F,	 64,  0)																									.setMcfg( 0, Al				, 5*U, WroughtIron		, 2*U, Lava				, 2*U)																			.aspects(TC.STRONTIO, 2).makeIntoSimpleAlloy(),
	Manyullyn				= alloymachine	( 8703, "Manyullyn"				, SET_METALLIC			, 256, 256, 256, 255).add(CENTRIFUGE).setQuality( 2.0F,	 96,  1)																									.setMcfg( 0, Co				, 1*U, Aredrite			, 1*U)																									.aspects(TC.STRONTIO, 2).makeIntoSimpleAlloy(),
	VibraniumSteel			= alloymachine	( 8704, "Vibranium Steel"		, SET_SHINY				,  40,  24,  50, 255).add(CENTRIFUGE).setQuality(  50.0F, 2048, 10)																									.setMcfg( 0, Vb				, 1*U, Steel			, 3*U)																									.aspects(TC.SENSUS, 1, TC.VITREUS, 1, TC.METALLUM, 1),
	VibraniumSilver			= alloy			( 8705, "Vibranium Silver"		, SET_SHINY				, 240, 240, 255, 255).add(CENTRIFUGE).setQuality( 100.0F,  512,  9)																									.setMcfg( 0, Vb				, 1*U, Ag				, 3*U)																									.aspects(TC.SENSUS, 1, TC.VITREUS, 1, TC.LUCRUM, 1),
	Vibramantium			= alloymachine	( 8706, "Vibramantium"			, SET_SHINY				, 250, 250, 250, 255).add(CENTRIFUGE).setQuality(1000.0F, 5120, 15)																									.setMcfg( 0, Vb				, 1*U, Ad				, 3*U)																									.aspects(TC.SENSUS, 1, TC.VITREUS, 1, TC.PRAECANTIO, 1, TC.METALLUM, 1),
	
	Enderium				= alloy			( 8710,	"Enderium"				, SET_DULL				,  89, 145, 135, 255).add(MAGICAL).setQuality( 8.0F,	256,  3)																																																																.aspects(TC.METALLUM, 2, TC.ALIENIS, 1),
	ElectrumFlux			= alloy			( 8711,	"Electrum Flux"			, SET_SHINY				, 255, 255, 120, 255).add(MAGICAL).setQuality(16.0F,	512,  3)																																																																,
	InfusedGold				= alloymachore	( 8712,	"Infused Gold"			, SET_SHINY				, 255, 200,  60, 255).add(HIDDEN).setQuality(12.0F,	 64,  3)																																																																	,
	GlowstoneRefined		= alloy			( 8713,	"GlowstoneRefined"		, SET_ROUGH				, 255, 255,   0, 255).add(HIDDEN).setLocal("Refined Glowstone").addIdenticalNames("RefinedGlowstone")																																																							,
	ObsidianRefined			= alloy			( 8714,	"ObsidianRefined"		, SET_ROUGH				,  80,  50, 100, 255).add(HIDDEN).setLocal("Refined Obsidian").addIdenticalNames("RefinedObsidian")																																																								,
	
	Desh					= metalmachore	( 8750,	"Desh"					, SET_DULL				,  40,  40,  40, 255).setQuality( 1.0F,   1280,  3).add()																																																																		,
	Duranium				= alloymachine	( 8751,	"Duranium"				, SET_METALLIC			, 256, 256, 256, 255).setQuality( 8.0F,   1280,  4).add()																																																																		,
	Tritanium				= alloymachine	( 8752,	"Tritanium"				, SET_METALLIC			, 256, 256, 256, 255).setQuality( 6.0F,   2560,  4).add()																																																																		.aspects(TC.METALLUM, 2, TC.ORDO, 2);
	
	public static final OreDictMaterial
	Pumice					= create		( 9000, "Pumice"				, SET_DULL				, 230, 185, 185, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE)													  												.setMcfg( 0, Stone			, 1*U)																															,
	Diatomite				= create		( 9001, "Diatomite"				, SET_DULL				, 225, 225, 225, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE)																									.setMcfg( 0, Flint			, 8*U, BandedIron		, 1*U, Sapphire			, 1*U)																			,
	BasalticMineralSand		= create		( 9003, "Basaltic Mineral Sand"	, SET_SAND				,  40,  50,  40, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_GOLD)																	.setMcfg( 0, Magnetite		, 1*U, Basalt			, 1*U)																									.aspects(TC.METALLUM, 2, TC.MAGNETO, 1),
	GraniticMineralSand		= create		( 9004, "Granitic Mineral Sand"	, SET_SAND				,  40,  60,  60, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE, ELECTROMAGNETIC_SEPARATION_GOLD)																	.setMcfg( 0, Magnetite		, 1*U, GraniteBlack		, 1*U)																									.aspects(TC.METALLUM, 2, TC.MAGNETO, 1),
	GarnetSand				= create		( 9005, "Garnet Sand"			, SET_SAND				, 200, 100,   0, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE)																									.setMcfg( 0, GarnetRed		, 1*U, GarnetYellow		, 1*U)																									,
	QuartzSand				= create		( 9006, "Quartz Sand"			, SET_SAND				, 200, 200, 200, 255).add(G_DUST_ORES, CENTRIFUGE, DECOMPOSABLE, QUARTZ).setSmelting(Si, U/9)																		.setMcfg( 0, CertusQuartz	, 1*U, Quartzite		, 1*U)																									;
	
	/** Technical Materials, which are only there for Recipes and such. */
	public static class TECH {
		public static final OreDictMaterial
		Brick						= invalid("Brick"						).add(DONT_SHOW_THIS_COMPONENT).add(STONE, BRITTLE).setMeltingPoint(2000),
		Organic						= invalid("Organic"						).add(DONT_SHOW_THIS_COMPONENT),
		Crystal						= invalid("Crystal"						).add(DONT_SHOW_THIS_COMPONENT).add(BRITTLE, CRYSTAL),
		Quartz						= invalid("Quartz"						).add(DONT_SHOW_THIS_COMPONENT).add(BRITTLE, CRYSTAL, QUARTZ),
		Unknown						= invalid("Unknown"						).add(DONT_SHOW_THIS_COMPONENT),
		Cobblestone					= invalid("Cobblestone"					).add(DONT_SHOW_THIS_COMPONENT).add(UNRECYCLABLE),
		
		Voltage8					= invalid("Voltage8"					).add(DONT_SHOW_THIS_COMPONENT),
		Voltage32					= invalid("Voltage32"					).add(DONT_SHOW_THIS_COMPONENT),
		Voltage128					= invalid("Voltage128"					).add(DONT_SHOW_THIS_COMPONENT),
		Voltage512					= invalid("Voltage512"					).add(DONT_SHOW_THIS_COMPONENT),
		Voltage2048					= invalid("Voltage2048"					).add(DONT_SHOW_THIS_COMPONENT),
		Voltage8192					= invalid("Voltage8192"					).add(DONT_SHOW_THIS_COMPONENT),
		Voltage32768				= invalid("Voltage32768"				).add(DONT_SHOW_THIS_COMPONENT),
		Voltage131072				= invalid("Voltage131072"				).add(DONT_SHOW_THIS_COMPONENT),
		Voltage524288				= invalid("Voltage524288"				).add(DONT_SHOW_THIS_COMPONENT),
		Voltage2147483647			= invalid("Voltage2147483647"			).add(DONT_SHOW_THIS_COMPONENT),
		
		DiamondIndustrial			= invalid("DiamondIndustrial"			).setAllToTheOutputOf(Diamond	).addReRegistrations(Diamond).add(AUTO_BLACKLIST).setLocal("Industrial Diamond"),
		
		AnyWood						= invalid("Any Wood"					).setLocal("Wood"		).setAllToTheOutputOf(Wood		).add(WOOD, FLAMMABLE				).addReRegistrationToThis(Wood, WoodSealed, Peanutwood),
		AnyStone					= invalid("Any Stone"					).setLocal("Stone"		).setAllToTheOutputOf(Stone		).add(STONE, BRITTLE, UNRECYCLABLE	).addReRegistrationToThis(Stone, Concrete),
		AnyIron						= invalid("Any Iron"					).setLocal("Iron"		).setAllToTheOutputOf(Fe		).add(SMITHABLE, MELTING			).addReRegistrationToThis(Fe, WroughtIron, PigIron, DeepIron, ShadowIron, MeteoricIron),
		AnyIronSteel				= invalid("Any Iron-Steel"				).setLocal("Steel"		).setAllToTheOutputOf(Steel		).add(SMITHABLE, MELTING			).addReRegistrationToThis(Steel, ShadowSteel, MeteoricSteel, HSLA),
		AnyCopper					= invalid("Any Copper"					).setLocal("Copper"		).setAllToTheOutputOf(Cu		).add(SMITHABLE, MELTING			).addReRegistrationToThis(Cu, AnnealedCopper),
		AnySilicon					= invalid("Any Silicon"					).setLocal("Silicon"	).setAllToTheOutputOf(Si		).add(SMITHABLE, MELTING			).addReRegistrationToThis(Si),
		
		AnySteel					= invalid("Any Steel"					).add(SMITHABLE, MELTING, DONT_SHOW_THIS_COMPONENT).addReRegistrationToThis(AnyIronSteel, Steeleaf, Knightmetal, FierySteel, BlackSteel, RedSteel, BlueSteel, DamascusSteel, TungstenSteel, StainlessSteel),
		AnyBronze					= invalid("Any Bronze"					).add(SMITHABLE, MELTING, DONT_SHOW_THIS_COMPONENT).addReRegistrationToThis(Bronze),
		AnyMetal					= invalid("Any Metal"					).add(SMITHABLE, MELTING, DONT_SHOW_THIS_COMPONENT).addReRegistrationToThis(AnyBronze, AnyCopper, AnyIron, AnySteel);
		
		static {
			for (OreDictMaterial tMaterial : ALL_MATERIALS_REGISTERED_HERE) {
				tMaterial.mHandleMaterial = (tMaterial.contains(BURNING)?Blaze:tMaterial.getMass()>Tc.getMass()*2?TungstenSteel:tMaterial.getMass()>Tc.getMass()?Steel:Wood);
				if (tMaterial.mReRegistrations.contains(AnyIron) || tMaterial.mReRegistrations.contains(AnyIronSteel)) {
					tMaterial.mAchievementsForCreation.add(AchievementList.openInventory);
					tMaterial.mAchievementsForCreation.add(AchievementList.mineWood);
					tMaterial.mAchievementsForCreation.add(AchievementList.buildWorkBench);
					tMaterial.mAchievementsForCreation.add(AchievementList.buildPickaxe);
					tMaterial.mAchievementsForCreation.add(AchievementList.buildFurnace);
					tMaterial.mAchievementsForCreation.add(AchievementList.acquireIron);
				}
			}
			
			Diamond.mAchievementsForCreation.add(AchievementList.openInventory);
			Diamond.mAchievementsForCreation.add(AchievementList.mineWood);
			Diamond.mAchievementsForCreation.add(AchievementList.buildWorkBench);
			Diamond.mAchievementsForCreation.add(AchievementList.buildPickaxe);
			Diamond.mAchievementsForCreation.add(AchievementList.buildFurnace);
			Diamond.mAchievementsForCreation.add(AchievementList.acquireIron);
			Diamond.mAchievementsForCreation.add(AchievementList.diamonds);
			
			Wood.mHandleMaterial = Stone.mHandleMaterial = Fe.mHandleMaterial = Au.mHandleMaterial = Diamond.mHandleMaterial = Thaumium.mHandleMaterial = Wood;
			Desh.mHandleMaterial = Desh;
			Vb.mHandleMaterial = VibraniumSteel.mHandleMaterial = VibraniumSilver.mHandleMaterial = VibraniumSteel;
			Vibramantium.mHandleMaterial = Vibramantium;
			VoidMetal.mHandleMaterial = InfusedAir.mHandleMaterial = InfusedBalance.mHandleMaterial = InfusedDull.mHandleMaterial = InfusedEarth.mHandleMaterial = InfusedEntropy.mHandleMaterial = InfusedFire.mHandleMaterial = InfusedOrder.mHandleMaterial = InfusedVis.mHandleMaterial = InfusedVis.mHandleMaterial = Loader.isModLoaded(ModIDs.TC)?Thaumium:DamascusSteel;
			
			// Too lazy to look up proper Materials, so I made everything Stone.
			Rhyolite				.setAllToTheOutputOf(Stone);
			Andesite				.setAllToTheOutputOf(Stone);
			Gabbro					.setAllToTheOutputOf(Stone);
			Komatiite				.setAllToTheOutputOf(Stone);
			Siltstone				.setAllToTheOutputOf(Stone);
			Dacite					.setAllToTheOutputOf(Stone);
			Gneiss					.setAllToTheOutputOf(Stone);
			Eclogite				.setAllToTheOutputOf(Stone);
			Blueschist				.setAllToTheOutputOf(Stone);
			Greenschist				.setAllToTheOutputOf(Stone);
			Migmatite				.setAllToTheOutputOf(Stone);
			Limestone				.setAllToTheOutputOf(Stone);
			Shale					.setAllToTheOutputOf(Stone);
			Greywacke				.setAllToTheOutputOf(Stone);
			Chert					.setAllToTheOutputOf(Stone);
			
			LiveRoot				.add(MORTAR);
			IronWood				.add(MORTAR);
			Steeleaf				.add(MORTAR);
			Wheat					.add(MORTAR);
			Paper					.add(MORTAR);
			Coal					.add(MORTAR);
			Charcoal				.add(MORTAR);
			Lignite					.add(MORTAR);
			Pb						.add(MORTAR);
			Sn						.add(MORTAR);
			SolderingAlloy			.add(MORTAR);
			Au						.add(MORTAR);
			Ag						.add(MORTAR);
			Fe						.add(MORTAR);
			IronMagnetic			.add(MORTAR);
			HSLA					.add(MORTAR);
			Steel					.add(MORTAR);
			SteelMagnetic			.add(MORTAR);
			Zn						.add(MORTAR);
			Sb						.add(MORTAR);
			Cu						.add(MORTAR);
			AnnealedCopper			.add(MORTAR);
			Bronze					.add(MORTAR);
			Ni						.add(MORTAR);
			Invar					.add(MORTAR);
			Brass					.add(MORTAR);
			WroughtIron				.add(MORTAR);
			Electrum				.add(MORTAR);
			Clay					.add(MORTAR);
			
			
			Amber					.setOreMultiplier( 2);
			InfusedAir				.setOreMultiplier( 2);
			InfusedFire				.setOreMultiplier( 2);
			InfusedEarth			.setOreMultiplier( 2);
			InfusedWater			.setOreMultiplier( 2);
			InfusedEntropy			.setOreMultiplier( 2);
			InfusedOrder			.setOreMultiplier( 2);
			InfusedVis				.setOreMultiplier( 2);
			InfusedDull				.setOreMultiplier( 2);
			InfusedBalance			.setOreMultiplier( 2);
			Salt					.setOreMultiplier( 2);
			RockSalt				.setOreMultiplier( 2);
			Cassiterite				.setOreMultiplier( 2);
			CassiteriteSand			.setOreMultiplier( 2);
			NetherQuartz			.setOreMultiplier( 2);
			CertusQuartz			.setOreMultiplier( 2);
			ChargedCertusQuartz		.setOreMultiplier( 2);
			Monazite				.setOreMultiplier( 2);
			Phosphorus				.setOreMultiplier( 3);
			Saltpeter				.setOreMultiplier( 3);
			Apatite					.setOreMultiplier( 4);
			Nikolite				.setOreMultiplier( 4);
			Teslatite				.setOreMultiplier( 4);
			Redstone				.setOreMultiplier( 4);
			Glowstone				.setOreMultiplier( 4);
			Lapis					.setOreMultiplier( 5);
			Sodalite				.setOreMultiplier( 5);
			Lazurite				.setOreMultiplier( 5);
			
			Plastic					.setEnchantmentForTools(Enchantment.knockback, 1);
			Rubber					.setEnchantmentForTools(Enchantment.knockback, 2);
			InfusedAir				.setEnchantmentForTools(Enchantment.knockback, 2);
			
			IronWood				.setEnchantmentForTools(Enchantment.fortune, 1);
			Steeleaf				.setEnchantmentForTools(Enchantment.fortune, 2);
			Midasium				.setEnchantmentForTools(Enchantment.fortune, 2);
			Mithril					.setEnchantmentForTools(Enchantment.fortune, 3);
			Vinteum					.setEnchantmentForTools(Enchantment.fortune, 1);
			Thaumium				.setEnchantmentForTools(Enchantment.fortune, 2);
			DarkThaumium			.setEnchantmentForTools(Enchantment.fortune, 3);
			Magic					.setEnchantmentForTools(Enchantment.fortune, 3);
			InfusedWater			.setEnchantmentForTools(Enchantment.fortune, 3);
			VibraniumSteel			.setEnchantmentForTools(Enchantment.fortune, 4);
			Vb						.setEnchantmentForTools(Enchantment.fortune, 5);
			
			Flint					.setEnchantmentForTools(Enchantment.fireAspect, 1);
			Obsidian				.setEnchantmentForTools(Enchantment.fireAspect, 2);
			DarkIron				.setEnchantmentForTools(Enchantment.fireAspect, 2);
			Firestone				.setEnchantmentForTools(Enchantment.fireAspect, 3);
			FierySteel				.setEnchantmentForTools(Enchantment.fireAspect, 3);
			Pyrotheum				.setEnchantmentForTools(Enchantment.fireAspect, 3);
			Blaze					.setEnchantmentForTools(Enchantment.fireAspect, 3);
			InfusedFire				.setEnchantmentForTools(Enchantment.fireAspect, 3);
			
			Ad						.setEnchantmentForTools(Enchantment.silkTouch, 1);
			Force					.setEnchantmentForTools(Enchantment.silkTouch, 1);
			Amber					.setEnchantmentForTools(Enchantment.silkTouch, 1);
			EnderPearl				.setEnchantmentForTools(Enchantment.silkTouch, 1);
			Enderium				.setEnchantmentForTools(Enchantment.silkTouch, 1);
			NetherStar				.setEnchantmentForTools(Enchantment.silkTouch, 1);
			InfusedOrder			.setEnchantmentForTools(Enchantment.silkTouch, 1);
			InfusedBalance			.setEnchantmentForTools(Enchantment.silkTouch, 1);
			Vibramantium			.setEnchantmentForTools(Enchantment.silkTouch, 1);
			
			BlackBronze				.setEnchantmentForTools(Enchantment.smite, 2);
			Gold					.setEnchantmentForTools(Enchantment.smite, 3);
			RoseGold				.setEnchantmentForTools(Enchantment.smite, 4);
			Platinum				.setEnchantmentForTools(Enchantment.smite, 5);
			InfusedVis				.setEnchantmentForTools(Enchantment.smite, 5);
			
			Pb						.setEnchantmentForTools(Enchantment.baneOfArthropods, 2);
			Ni						.setEnchantmentForTools(Enchantment.baneOfArthropods, 2);
			Invar					.setEnchantmentForTools(Enchantment.baneOfArthropods, 3);
			Sb						.setEnchantmentForTools(Enchantment.baneOfArthropods, 3);
			BatteryAlloy			.setEnchantmentForTools(Enchantment.baneOfArthropods, 4);
			Bi						.setEnchantmentForTools(Enchantment.baneOfArthropods, 4);
			BismuthBronze			.setEnchantmentForTools(Enchantment.baneOfArthropods, 5);
			InfusedEarth			.setEnchantmentForTools(Enchantment.baneOfArthropods, 5);
			
			Fe						.setEnchantmentForTools(Enchantment.sharpness, 1);
			Ice						.setEnchantmentForTools(Enchantment.sharpness, 1);
			Bronze					.setEnchantmentForTools(Enchantment.sharpness, 1);
			Brass					.setEnchantmentForTools(Enchantment.sharpness, 2);
			HSLA					.setEnchantmentForTools(Enchantment.sharpness, 2);
			Steel					.setEnchantmentForTools(Enchantment.sharpness, 2);
			WroughtIron				.setEnchantmentForTools(Enchantment.sharpness, 2);
			StainlessSteel			.setEnchantmentForTools(Enchantment.sharpness, 3);
			Knightmetal				.setEnchantmentForTools(Enchantment.sharpness, 3);
			ShadowIron				.setEnchantmentForTools(Enchantment.sharpness, 3);
			ShadowSteel				.setEnchantmentForTools(Enchantment.sharpness, 4);
			BlackSteel				.setEnchantmentForTools(Enchantment.sharpness, 4);
			RedSteel				.setEnchantmentForTools(Enchantment.sharpness, 4);
			BlueSteel				.setEnchantmentForTools(Enchantment.sharpness, 5);
			DamascusSteel			.setEnchantmentForTools(Enchantment.sharpness, 5);
			InfusedEntropy			.setEnchantmentForTools(Enchantment.sharpness, 5);
			Glass					.setEnchantmentForTools(Enchantment.sharpness, 5);
			
			InfusedAir				.setEnchantmentForArmors(Enchantment.respiration, 3);
			
			InfusedFire				.setEnchantmentForArmors(Enchantment.featherFalling, 4);
			
			Steeleaf				.setEnchantmentForArmors(Enchantment.protection, 2);
			Knightmetal				.setEnchantmentForArmors(Enchantment.protection, 1);
			InfusedEarth			.setEnchantmentForArmors(Enchantment.protection, 4);
			VibraniumSilver			.setEnchantmentForArmors(Enchantment.protection, 6);
			VibraniumSteel			.setEnchantmentForArmors(Enchantment.protection, 8);
			Vibramantium			.setEnchantmentForArmors(Enchantment.protection,10);
			Vb						.setEnchantmentForArmors(Enchantment.protection,10);
			
			InfusedEntropy			.setEnchantmentForArmors(Enchantment.thorns, 3);
			
			InfusedWater			.setEnchantmentForArmors(Enchantment.aquaAffinity, 1);
			IronWood				.setEnchantmentForArmors(Enchantment.aquaAffinity, 1);
			
			InfusedOrder			.setEnchantmentForArmors(Enchantment.projectileProtection, 4);
			
			Obsidian				.setEnchantmentForArmors(Enchantment.blastProtection, 3);
			InfusedDull				.setEnchantmentForArmors(Enchantment.blastProtection, 4);

			InfusedVis				.setEnchantmentForArmors(Enchantment.protection, 4);
			InfusedBalance			.setEnchantmentForArmors(Enchantment.protection, 4);
			
			/*
			Stone					.setSpecialEffect(SpecialToolEffect.Crushing, 1);
			GraniteRed				.setSpecialEffect(SpecialToolEffect.Crushing, 2);
			GraniteBlack			.setSpecialEffect(SpecialToolEffect.Crushing, 2);
			Adamantium				.setSpecialEffect(SpecialToolEffect.Crushing, 3);
			
			DeepIron				.setSpecialEffect(SpecialToolEffect.Magnetic, 1);
			MeteoricIron			.setSpecialEffect(SpecialToolEffect.Magnetic, 2);
			MeteoricSteel			.setSpecialEffect(SpecialToolEffect.Magnetic, 3);
			IronMagnetic			.setSpecialEffect(SpecialToolEffect.Magnetic, 1);
			SteelMagnetic			.setSpecialEffect(SpecialToolEffect.Magnetic, 2);
			NeodymiumMagnetic		.setSpecialEffect(SpecialToolEffect.Magnetic, 3);
			
			FryingOilHot			.setHeatDamage(1.0F);
			Lava					.setHeatDamage(3.0F);
			Firestone				.setHeatDamage(5.0F);
			Pyrotheum				.setHeatDamage(5.0F);
			*/
			
			Chalcopyrite			.addOreByProducts(Pyrite				, Co					, Cd					, Au			);
			Sphalerite				.addOreByProducts(GarnetYellow			, Cd					, Ga					, Zn			);
			MeteoricIron			.addOreByProducts(Fe					, Ni					, Ir					, Pt			);
			Cu						.addOreByProducts(Co					, Au					, Ni					, Co_60			);
			Ni						.addOreByProducts(Co					, Pt					, Fe					, Co_60			);
			Pentlandite				.addOreByProducts(Fe					, S						, Co					, Co_60			);
			YellowLimonite			.addOreByProducts(Ni					, BrownLimonite			, Co					, Co_60			);
			Pitchblende				.addOreByProducts(Th					, U_238					, Pu					, U_235			);
			Uraninite				.addOreByProducts(U_238					, Th					, Pb					, U_235			);
			Pu						.addOreByProducts(U_238					, Pb					, Pu_241				, Pu_243		);
			GlauconiteSand			.addOreByProducts(Na					, Al					, Fe					);
			Glauconite				.addOreByProducts(Na					, Al					, Fe					);
			Vermiculite				.addOreByProducts(Fe					, Al					, Mg					);
			FullersEarth			.addOreByProducts(Al					, Si					, Mg					);
			Bentonite				.addOreByProducts(Al					, Ca					, Mg					);
			Galena					.addOreByProducts(S						, Ag					, Pb					);
			Lapis					.addOreByProducts(Lazurite				, Sodalite				, Pyrite				);
			Pyrite					.addOreByProducts(S						, Phosphorus			, Fe					);
			GarnetRed				.addOreByProducts(Spessartine			, Pyrope				, Almandine				);
			GarnetYellow			.addOreByProducts(Andradite				, Grossular				, Uvarovite				);
			Cooperite				.addOreByProducts(Pd					, Ni					, Ir					);
			Cinnabar				.addOreByProducts(Redstone				, S						, Glowstone				);
			Tantalite				.addOreByProducts(Mn					, Nb					, Ta					);
			Pollucite				.addOreByProducts(Cs					, Al					, Rb					);
			Chrysotile				.addOreByProducts(Asbestos				, Si					, Mg					);
			Asbestos				.addOreByProducts(Asbestos				, Si					, Mg					);
			U_238					.addOreByProducts(Pb					, Pu					, Th					);
			Scheelite				.addOreByProducts(Mn					, Mo					, Ca					);
			Tungstate				.addOreByProducts(Mn					, Ag					, Li					);
			Bauxite					.addOreByProducts(Grossular				, Ti					, Ga					);
			QuartzSand				.addOreByProducts(CertusQuartz			, Quartzite				, Barite				);
			Quartzite				.addOreByProducts(CertusQuartz			, Barite				, ChargedCertusQuartz	);
			CertusQuartz			.addOreByProducts(Quartzite				, ChargedCertusQuartz	, Barite				);
			ChargedCertusQuartz		.addOreByProducts(CertusQuartz			, Quartzite				, Barite				);
			Redstone				.addOreByProducts(Cinnabar				, RareEarth				, Glowstone				);
			Monazite				.addOreByProducts(Th					, Nd					, RareEarth				);
			Forcicium				.addOreByProducts(Th					, Nd					, RareEarth				);
			Forcillium				.addOreByProducts(Th					, Nd					, RareEarth				);
			Malachite				.addOreByProducts(Cu					, BrownLimonite			, Calcite				);
			BrownLimonite			.addOreByProducts(Malachite				, YellowLimonite		);
			Nd						.addOreByProducts(Monazite				, RareEarth				);
			Bastnasite				.addOreByProducts(Nd					, RareEarth				);
			Glowstone				.addOreByProducts(Redstone				, Au					);
			Zn						.addOreByProducts(Sn					, Ga					);
			W						.addOreByProducts(Mn					, Mo					);
			Diatomite				.addOreByProducts(BandedIron			, Sapphire				);
			Fe						.addOreByProducts(Ni					, Sn					);
			Lepidolite				.addOreByProducts(Li					, Cs					);
			Au						.addOreByProducts(Cu					, Ni					);
			Sn						.addOreByProducts(Fe					, Zn					);
			Sb						.addOreByProducts(Zn					, Fe					);
			Ag						.addOreByProducts(Pb					, S						);
			Pb						.addOreByProducts(Ag					, S						);
			Th						.addOreByProducts(U_238					, Pb					);
			Electrum				.addOreByProducts(Au					, Ag					);
			Bronze					.addOreByProducts(Cu					, Sn					);
			Brass					.addOreByProducts(Cu					, Zn					);
			Coal					.addOreByProducts(Lignite				, Th					);
			Ilmenite				.addOreByProducts(Fe					, Ti					);
			Mn						.addOreByProducts(Cr					, Fe					);
			Sapphire				.addOreByProducts(Al					, GreenSapphire			);
			GreenSapphire			.addOreByProducts(Al					, Sapphire				);
			Pt						.addOreByProducts(Ni					, Ir					);
			Emerald					.addOreByProducts(Be					, Al					);
			Olivine					.addOreByProducts(Pyrope				, Mg					);
			Cr						.addOreByProducts(Fe					, Mg					);
			Chromite				.addOreByProducts(Fe					, Mg					);
			Tetrahedrite			.addOreByProducts(Sb					, Zn					);
			GarnetSand				.addOreByProducts(GarnetRed				, GarnetYellow			);
			Magnetite				.addOreByProducts(Fe					, Au					);
			GraniticMineralSand		.addOreByProducts(GraniteBlack			, Magnetite				);
			BasalticMineralSand		.addOreByProducts(Basalt				, Magnetite				);
			Basalt					.addOreByProducts(Olivine				, DarkAsh				);
			Celestine				.addOreByProducts(Sr					, S						);
			VanadiumMagnetite		.addOreByProducts(Magnetite				, V						);
			Lazurite				.addOreByProducts(Sodalite				, Lapis					);
			Sodalite				.addOreByProducts(Lazurite				, Lapis					);
			Spodumene				.addOreByProducts(Al					, Li					);
			Ruby					.addOreByProducts(Cr					, GarnetRed				);
			Phosphorus				.addOreByProducts(Apatite				, Phosphate				);
			Ir						.addOreByProducts(Pt					, Os					);
			Pyrope					.addOreByProducts(GarnetRed				, Mg					);
			Almandine				.addOreByProducts(GarnetRed				, Al					);
			Spessartine				.addOreByProducts(GarnetRed				, Mn					);
			Andradite				.addOreByProducts(GarnetYellow			, Fe					);
			Grossular				.addOreByProducts(GarnetYellow			, Ca					);
			Uvarovite				.addOreByProducts(GarnetYellow			, Cr					);
			Calcite					.addOreByProducts(Andradite				, Malachite				);
			Cobaltite				.addOreByProducts(Co					, Co_60					);
			Co						.addOreByProducts(Cobaltite				, Co_60					);
			Co_60					.addOreByProducts(Cobaltite				, Co					);
			Nq_528					.addOreByProducts(Nq					, Nq_522				);
			Nq_522					.addOreByProducts(Nq					);
			Nq						.addOreByProducts(Nq_528				);
			Pyrolusite				.addOreByProducts(Mn					);
			Molybdenite				.addOreByProducts(Mo					);
			Stibnite				.addOreByProducts(Sb					);
			Garnierite				.addOreByProducts(Ni					);
			Lignite					.addOreByProducts(Coal					);
			Diamond					.addOreByProducts(Graphite				);
			Be						.addOreByProducts(Emerald				);
			Apatite					.addOreByProducts(Phosphorus			);
			Nikolite				.addOreByProducts(Diamond				);
			Teslatite				.addOreByProducts(Diamond				);
			Magnesite				.addOreByProducts(Mg					);
			NetherQuartz			.addOreByProducts(Netherrack			);
			PigIron					.addOreByProducts(Fe					);
			DeepIron				.addOreByProducts(Fe					);
			ShadowIron				.addOreByProducts(Fe					);
			DarkIron				.addOreByProducts(Fe					);
			MeteoricIron			.addOreByProducts(Fe					);
			Steel					.addOreByProducts(Fe					);
			Mithril					.addOreByProducts(Pt					);
			Midasium				.addOreByProducts(Au					);
			AstralSilver			.addOreByProducts(Ag					);
			Graphite				.addOreByProducts(C						);
			Netherrack				.addOreByProducts(S						);
			Flint					.addOreByProducts(Obsidian				);
			S						.addOreByProducts(S						);
			Saltpeter				.addOreByProducts(Saltpeter				);
			Endstone				.addOreByProducts(He_3					);
			Os						.addOreByProducts(Ir					);
			Mg						.addOreByProducts(Olivine				);
			Al						.addOreByProducts(Bauxite				);
			Ti						.addOreByProducts(Almandine				);
			Obsidian				.addOreByProducts(Olivine				);
			Ash						.addOreByProducts(C						);
			DarkAsh					.addOreByProducts(C						);
			Redrock					.addOreByProducts(Clay					);
			Marble					.addOreByProducts(Calcite				);
			Clay					.addOreByProducts(Clay					);
			Cassiterite				.addOreByProducts(Sn					);
			CassiteriteSand			.addOreByProducts(Sn					);
			GraniteBlack			.addOreByProducts(Biotite				);
			GraniteRed				.addOreByProducts(PotassiumFeldspar		);
			Phosphate				.addOreByProducts(P						);
			P						.addOreByProducts(Phosphate				);
			Tanzanite				.addOreByProducts(Opal					);
			Opal					.addOreByProducts(Tanzanite				);
			Jasper					.addOreByProducts(FoolsRuby				);
			FoolsRuby				.addOreByProducts(Jasper				);
			Topaz					.addOreByProducts(BlueTopaz				);
			BlueTopaz				.addOreByProducts(Topaz					);
			Niter					.addOreByProducts(Saltpeter				);
			Si						.addOreByProducts(SiliconDioxide		);
			Salt					.addOreByProducts(RockSalt				);
			RockSalt				.addOreByProducts(Salt					);
			
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(YellowLimonite		, 4*U/ 1), OM.stack(DarkAsh			,   U/ 1)));
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(BrownLimonite			, 4*U/ 1), OM.stack(DarkAsh			,   U/ 1)));
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(Chalcopyrite			, 4*U/ 1), OM.stack(DarkAsh			,   U/ 1))); // TODO
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(BandedIron			, 5*U/ 2), OM.stack(DarkAsh			, 2*U/ 2), OM.stack(Calcite	, 2*U/ 2)));
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 3, OM.stack(Magnetite				, 7*U/ 3), OM.stack(DarkAsh			, 3*U/ 3)));
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 3, OM.stack(VanadiumMagnetite		,14*U/ 3), OM.stack(DarkAsh			, 3*U/ 3))); // TODO
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 3, OM.stack(BasalticMineralSand	,14*U/ 3), OM.stack(DarkAsh			, 3*U/ 3)));
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 3, OM.stack(GraniticMineralSand	,14*U/ 3), OM.stack(DarkAsh			, 3*U/ 3)));
			
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(YellowLimonite		, 8*U/ 2), OM.stack(C				,   U/ 2)));
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(BrownLimonite			, 8*U/ 2), OM.stack(C				,   U/ 2)));
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(Chalcopyrite			, 8*U/ 2), OM.stack(C				,   U/ 2))); // TODO
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(BandedIron			,10*U/ 4), OM.stack(C				, 2*U/ 4), OM.stack(Calcite	, 2*U/ 4)));
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 6, OM.stack(Magnetite				,14*U/ 6), OM.stack(C				, 3*U/ 6)));
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 6, OM.stack(VanadiumMagnetite		,28*U/ 6), OM.stack(C				, 3*U/ 6))); // TODO
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 6, OM.stack(BasalticMineralSand	,28*U/ 6), OM.stack(C				, 3*U/ 6)));
			Fe				.addAlloyingRecipe(new OreDictConfigurationComponent( 6, OM.stack(GraniticMineralSand	,28*U/ 6), OM.stack(C				, 3*U/ 6)));
			
			SterlingSilver	.addAlloyingRecipe(new OreDictConfigurationComponent( 5, OM.stack(AnnealedCopper		, 1*U/ 5), OM.stack(Ag				, 4*U/ 5)));
			RoseGold		.addAlloyingRecipe(new OreDictConfigurationComponent( 5, OM.stack(AnnealedCopper		, 1*U/ 5), OM.stack(Au				, 4*U/ 5)));
			Brass			.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(AnnealedCopper		, 3*U/ 4), OM.stack(Zn				, 1*U/ 4)));
			Bronze			.addAlloyingRecipe(new OreDictConfigurationComponent( 4, OM.stack(AnnealedCopper		, 3*U/ 4), OM.stack(Sn				, 1*U/ 4)));
			BlackBronze		.addAlloyingRecipe(new OreDictConfigurationComponent( 5, OM.stack(AnnealedCopper		, 3*U/ 5), OM.stack(Electrum		, 2*U/ 5)));
			RedAlloy		.addAlloyingRecipe(new OreDictConfigurationComponent( 1, OM.stack(AnnealedCopper		, 1*U/ 1), OM.stack(Redstone		, 4*U/ 1)));
			Cupronickel		.addAlloyingRecipe(new OreDictConfigurationComponent( 2, OM.stack(AnnealedCopper		, 1*U/ 2), OM.stack(Ni				, 1*U/ 2)));
			
			
			
			String tMakeSteel =
			  "In order to make Steel you just need to melt Iron or Wrought Iron in a Smelting Crucible and give some Carbon (1 per 50 Iron), Graphite (1 per 25 Iron) or Diamonds (1 per 100 Iron at "+Diamond.mMeltingPoint+"K) to it.";
			String tMakeWroughtIron =
			  "Wrought Iron is created by heating up Iron until " + WroughtIron.mMeltingPoint + " Kelvin to dissolve most unwanted impurities.";
			String tMakeAnnealedCopper =
			  "Annealed Copper is created by heating up Copper until " + AnnealedCopper.mMeltingPoint + " Kelvin to dissolve most unwanted impurities.";
			
			Cu.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Copper is a Material, which is needed in almost every electrical Device. But not only that, it is also used to make Bronze, Brass, Cupronickel and some other Alloys."
			, tMakeAnnealedCopper
			};
			
			AnnealedCopper.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Annealed Copper is just Copper cleaned from impurities. Therefore some of Coppers natural properties are better than in unpurified Copper."
			, tMakeAnnealedCopper
			};
			
			Fe.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Iron, the most needed Material in Minecraft, unless you have an Iron Titan or something. Nearly every Mod needs it in order to proceed with its Machines. It exists in many Shapes and Forms, such as"
			, "Compressed Iron, Wrought Iron, Steel, HSLA Steel, Dark Iron, Deep Iron, Meteoric Iron, Conductive Iron, Electrical Steel, Pulsating Iron, Mutated Iron, Shadow Iron, Ironwood and many many more things."
			, "It is a very useful Material as it is required to make Steel, the Stuff almost everything Technological is made of. You can also make Wrought Iron, to craft Tools slightly better than Iron Tools."
			, tMakeWroughtIron
			, tMakeSteel
			};
			
			WroughtIron.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Wrought Iron, a Material that is worth as much as Iron. It is just a step between Iron itself and Steel, and is just a cleaner Version of Iron. It is also a slightly better Tool Material than Iron."
			, tMakeWroughtIron
			, tMakeSteel
			};
			
			Steel.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Steel, the Material Standard Machinery is made of. Cheap, abundant and common, as it only requires Iron and some source of Carbon. There are many different Ways of making Steel supplied by many Mods."
			, tMakeSteel
			};
			
			C.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Carbon. Not much to say about it. It is used to make Steel. Uhhm, maybe it has some other uses explained in a later Edition of this Book."
			, tMakeSteel
			};
			
			Diamond.mDescription = DiamondIndustrial.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Diamonds, the shiny Stuff every Minecrafter looks for. Some people may even create living Horses made of Diamonds."
			, tMakeSteel
			};
			
			Graphite.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Graphite is a Source of Carbon. It can also be used to make Steel directly."
			, tMakeSteel
			};
			
			DamascusSteel.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Damascus Steel is a very rare kind of Steel. Rare meaning actually nobody knows how to produce it anymore as they are all dead. Maybe ask a Zombie or a Skeleton on how to make it?"
			, "Because of that, it can only be found in old Structures such as Dungeons. Sometimes Villagers stumble upon this rare Steel or things made of it and take it home to give it to their Blacksmith."
			, "It is a very good Tool Material and even surpasses Blue Steel in its Quality. It has Vanadium and Tungsten impurities which improve the stability of the Carbides when Forging at higher Temperatures."
			};
			
			Craponite.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Craponite is a very exquisite Material used only in the fanciest Jewelry. It looks best in combination with Peridot. That is why Teleshopping Channels often sell Peridot Craponite Earrings and alike."
			};
			
			Ir.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Iridium, a very rare Metal used in only the most advanced Technology. Its properties are very versatile and used in many advanced Devices."
			, "Iridium is a Metal that can stabilise a Tesseract, so that it can transfer much larger Amounts of Matter and Energy."
			, "As Weapon it is very useful in order to kill Shapeshifters, since it is highly Toxic to them. Now, if there were Shapeshifters in our World this Information would be more useful."
			};
			
			Tc.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Technetium, named after Gregorius Techneticies, is the first Element of the Periodic System, which has no stable Isotope. Its usage is mostly limited to Medical Applications."
			, "Fun Fact:\nEvery Element, which has no Elemental Stats assigned to, will automatically default to the Elemental Stats of Technetium."
			};
			
			Mcg.mDescription = new String[] {
			// ======================================================================================================================================================================================================== 
			  "Mc-Guffium, the most useful Material of them all. It can be used for everything, as long as its underlying Science is too obscure to be understood."
			, "Possible Applications are: Time Travel, Infinite Impossibility Drives, World Piece, World Domination, World War, World War II, World War IV, Curing all Diseases, Stopping Global Warming,"
			, "Controlling the Weather, Turning everything into Gold, Turning you into a God, Killing a God, Making Dark Matter an Energy Source, Tesseracts, Wormholes, Black Holes, Bending Space and Time, Tardis,"
			, "Ascension to a higher Level of existence, Invisibility, Killing Chuck Norris, Cloning, Making Profit of collecting Underpants, Making the Impossible possible, Love Rays, Hate Rays, Perpetuum Mobiles,"
			, "Making you able to hold and use Thors Hammer, and ofcourse baking delicious Cookies."
			};
			
			Nq.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "This extremly rare Material is called Naquadah. Its properties make it very useful in highly advanced Technology."
			, "It is a Superconductor even at very high Temperatures, what makes it seeming Ideal for electric Wiring as long as the amount of transferred Electricity is not too large."
			, "Another property of it is that it emits Gamma Radiation when supplied with enough Electricity, due to creating Positrons out of said Energy, which then collide with Electrons very quickly."
			, "However that property makes it very unstable and can result in a Nuclear Explosion when not regulated properly, especially with molten Naquadah."
			};
			
			Nq_528.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "This is a heavy Isotope of Naquadah mainly used to generate Energy, or to create Bombs."
			, "Due to being less stable than regular Naquadah, supplying this Isotope with Electricity results in a very strong Nuclear Reaction, which when regulated properly can result in a constant Energy Source."
			, "The instability increases when this Isotope is molten, resulting in a larger Energy Output when used in Reactors."
			, "Its instability is also often used to make Naquadah Bombs, which are much more destructive than the regular Plutonium Nukes. It doesn't make any difference to use the molten or the solid Isotope"
			, "for Bombs, since its explosion heats up the whole Bomb far above the melting Point anyways."
			};
			
			Nq_522.mDescription = new String[] {
			// ========================================================================================================================================================================================================
			  "Naquadria, a light and extremely unstable Isotope of Naquadah. It can be used for Reactors as well as Bombs and is much stronger than heavy Naquadah."
			, "It is created when Naquadah is receiving too large amounts of Gamma Radiation. During its creation it emits more Gamma Radiation turning adjacent Naquadah into Naquadria too."
			, "The Natural origin of this Isotope is a Gamma Ray Burst, which hits a Naquadah Vein directly. Planets to which that happens are then turning into a Desert due to the enormous Heat created by the Ray."
			, "Gamma Bombs which are exploding far enough away from a Naquadah Vein, or just a strong enough Gamma Ray, can also trigger a Chain Reaction turing Naquadah into Naquadria."
			, "However most of the time a Gamma Ray hits a Naquadah Vein, it just explodes, what is the main Factor of Naquadria being very rare. Also Naquadria turns back into Naquadah after a few thousand years."
			};
		}
	}
	
	/** The "I don't care" Section, everything I don't want to do anything with right now, is right here. Just to make the Material Finder shut up about them. But I do see potential uses in some of these Materials. */
	public static class UNUSED {
		public static final OreDictMaterial
		Haderoth					= unused("Haderoth"						).setQuality( 6.0F,	 64,  2).add(G_INGOT_ORES),
		Hematite					= unused("Hematite"						).add(G_INGOT_ORES),
		Hepatizon					= unused("Hepatizon"					).setQuality( 6.0F,	 64,  2).add(G_INGOT_ORES),
		Ignatius					= unused("Ignatius"						).add(G_DUST),
		Infernal					= unused("Infernal"						),
		Infuscolium					= unused("Infuscolium"					).setQuality( 6.0F,	 64,  2).add(G_INGOT_ORES),
		Inolashite					= unused("Inolashite"					).setQuality( 6.0F,	128,  3).add(G_INGOT_ORES),
		Invisium					= unused("Invisium"						).add(G_DUST),
		Kalendrite					= unused("Kalendrite"					).add(G_DUST),
		Lemurite					= unused("Lemurite"						).add(G_DUST),
		Lodestone					= unused("Lodestone"					).add(G_DUST_ORES),
		Luminite					= unused("Luminite"						).add(G_DUST_ORES),
		Magma						= unused("Magma"						),
		Mawsitsit					= unused("Mawsitsit"					).add(G_DUST),
		Mercassium					= unused("Mercassium"					).setQuality( 6.0F,	 64,  1).add(G_INGOT_ORES),
		Meteorite					= unused("Meteorite"					).add(G_DUST_ORES),
		Meutoite					= unused("Meutoite"						).add(G_DUST_ORES),
		Mimichite					= unused("Mimichite"					).add(G_GEM_ORES, CRYSTAL, BRITTLE),
		Moonstone					= unused("Moonstone"					).add(G_DUST_ORES).aspects(TC.VITREUS, 1, TC.ALIENIS, 1),
		Nether						= unused("Nether"						),
		ObsidianFlux				= unused("Obsidian Flux"				).add(G_INGOT),
		Onyx						= unused("Onyx"							).add(G_DUST),
		Orichalcum					= unused("Orichalcum"					).setQuality( 6.0F,	 64,  3).add(G_INGOT_ORES),
		Osmonium					= unused("Osmonium"						).setQuality( 6.0F,	 64,  1).add(G_INGOT_ORES),
		Oureclase					= unused("Oureclase"					).setQuality( 6.0F,	 64,  1).add(G_INGOT_ORES),
		Painite						= unused("Painite"						),
		Petroleum					= unused("Petroleum"					).add(G_DUST_ORES),
		Pewter						= unused("Pewter"						),
		Phoenixite					= unused("Phoenixite"					).setQuality( 6.0F,	 64,  1).add(G_INGOT_ORES),
		Potash						= unused("Potash"						),
		Prometheum					= unused("Prometheum"					).setQuality( 6.0F,	 64,  1).add(G_INGOT_ORES),
		Quicklime					= unused("Quicklime"					).add(G_DUST),
		Randomite					= unused("Randomite"					).add(G_DUST_ORES),
		Rubracium					= unused("Rubracium"					).add(G_DUST_ORES),
		RyuDragonRyder				= unused("RyuDragonRyder"				),
		Sanguinite					= unused("Sanguinite"					).add(G_INGOT_ORES),
		Spinel						= unused("Spinel"						),
		Starconium					= unused("Starconium"					).add(G_INGOT_ORES),
		Sugilite					= unused("Sugilite"						).add(G_DUST),
		Sunstone					= unused("Sunstone"						).add(G_DUST_ORES).aspects(TC.VITREUS, 1, TC.ALIENIS, 1),
		Tar							= unused("Tar"							),
		Tartarite					= unused("Tartarite"					).add(G_INGOT_ORES),
		Tapazite					= unused("Tapazite"						).add(G_DUST),
		Thyrium						= unused("Thyrium"						).add(G_INGOT_ORES),
		Tourmaline					= unused("Tourmaline"					).add(G_DUST),
		Turquoise					= unused("Turquoise"					).add(G_DUST),
		Vis							= unused("Vis"							).add(DECOMPOSABLE).setMcfg( 0, Ma, 1*U).aspects(TC.AURAM, 2, TC.PRAECANTIO, 1),
		Voidstone					= unused("Voidstone"					).aspects(TC.VITREUS, 1, TC.VACUOS, 1),
		Vulcanite					= unused("Vulcanite"					).setQuality( 6.0F,	 64,  2).add(G_INGOT_ORES),
		Vyroxeres					= unused("Vyroxeres"					).setQuality( 6.0F,	 64,  2).add(G_INGOT_ORES),
		Wimalite					= unused("Wimalite"						).add(G_DUST_ORES),
		Yellorite					= unused("Yellorite"					).add(G_DUST_ORES),
		Yellorium					= unused("Yellorium"					).add(G_INGOT),
		Zectium						= unused("Zectium"						).add(G_INGOT_ORES),
		Adamite						= unused("Adamite"						).add(G_DUST_ORES),
		Adluorite					= unused("Adluorite"					).add(G_DUST_ORES),
		Agate						= unused("Agate"						).add(G_DUST),
		Alduorite					= unused("Alduorite"					).add(G_DUST_ORES),
		Ammonium					= unused("Ammonium"						).add(G_DUST),
		Amordrine					= unused("Amordrine"					).setQuality( 6.0F,   64,  2).add(G_INGOT_ORES),
		Angmallen					= unused("Angmallen"					).setQuality( 6.0F,   64,  2).add(G_INGOT_ORES),
		Ardite						= unused("Ardite"						).setQuality( 6.0F,   64,  2).add(G_INGOT_ORES),
		Atlarus						= unused("Atlarus"						).setQuality( 6.0F,   64,  2).add(G_INGOT_ORES),
		Bitumen						= unused("Bitumen"						).add(G_DUST_ORES),
		Black						= unused("Black"						),
		Bloodstone					= unused("Bloodstone"					).add(G_DUST),
		Blutonium					= unused("Blutonium"					).add(G_INGOT_ORES),
		Carmot						= unused("Carmot"						).setQuality( 6.0F,	 64,  2).add(G_INGOT_ORES),
		Celenegil					= unused("Celenegil"					).setQuality( 6.0F,	 64,  2).add(G_INGOT_ORES),
		Ceruclase					= unused("Ceruclase"					).add(G_INGOT_ORES),
		Citrine						= unused("Citrine"						).add(G_DUST),
		Chimerite					= unused("Chimerite"					).add(G_DUST),
		Coral						= unused("Coral"						).add(G_DUST),
		Chrysocolla					= unused("Chrysocolla"					).add(G_DUST),
		CrystalFlux					= unused("Crystal Flux"					).add(G_GEM, CRYSTAL, BRITTLE),
		Cyanite						= unused("Cyanite"						).add(G_DUST),
		DarkStone					= unused("Dark Stone"					).add(G_DUST),
		Demonite					= unused("Demonite"						).add(G_DUST),
		Desichalkos					= unused("Desichalkos"					).setQuality( 6.0F,   1280,  3).add(G_INGOT_ORES),
		Draconic					= unused("Draconic"						).add(G_DUST),
		EnderiumBase				= unused("Enderium Base"				).setQuality( 8.0F,	256,  3).add(G_INGOT, MAGICAL).aspects(TC.METALLUM, 2, TC.ALIENIS, 1),
		Energized					= unused("Energized"					),
		Eximite						= unused("Eximite"						).setQuality( 6.0F,	 64,  2).add(G_INGOT_ORES),
		Fluorite					= unused("Fluorite"						).add(G_DUST_ORES),
		Leather						= unused("Leather"						).add(FLAMMABLE).setBurning(Ash, U/9).setSmelting(Ash, U/9),
		Unstable					= unused("Unstable" 					).aspects(TC.PERDITIO, 4).addIdenticalNames("Unstableingot"),
		Reinforced					= unused("Reinforced"					),
		Mud							= unused("Mud"					   		),
		LimePure					= unused("LimePure"				   		).setLocal("Pure Lime"),
		Peat						= unused("Peat"					   		).aspects(TC.POTENTIA, 2, TC.IGNIS, 2),
		TNT							= unused("TNT"					   		).add(EXPLOSIVE, FLAMMABLE).aspects(TC.PERDITIO, 3, TC.IGNIS, 1),
		Cluster						= unused("Cluster"				   		),
		Red							= unused("Red"							),
		Antimatter					= unused("Antimatter"					).add(ANTIMATTER),
		Sweet						= unused("Sweet"						),
		Gelatine					= unused("Gelatine"						),
		TarPitch					= unused("Tar Pitch"					),
		Carborundum					= unused("Carborundum"					),
		Satinspar					= unused("Satinspar"					),
		Selenite					= unused("Selenite"						),
		PetrifiedWood				= unused("Petrified Wood"				),
		Jet							= unused("Jet"							),
		Microcline					= unused("Microcline"					),
		Serpentine					= unused("Serpentine"					),
		Sylvite						= unused("Sylvite"						),
		Flux						= unused("Flux"							),
		Goshen						= unused("Goshen"						),
		Joshen						= unused("Joshen"						),
		Itarius						= unused("Itarius"						),
		Legendary					= unused("Legendary"					),
		MutatedIron					= unused("Mutated Iron"					),
		Witheria					= unused("Witheria"						),
		RedstoneAlloy				= unused("Redstone Alloy"				),
		OsmiumTetroxide				= unused("Osmium Tetroxide"				),
		NitricAcid					= unused("Nitric Acid"					),
		RubberTreeSap				= unused("Rubber Tree Sap"				),
		AquaRegia					= unused("Aqua Regia"					),
		SolutionBlueVitriol			= unused("SolutionBlueVitriol"			).setLocal("Blue Vitriol Solution"),
		SolutionNickelSulfate		= unused("SolutionNickelSulfate"		).setLocal("Nickel Sulfate Solution"),
		Signalum					= unused("Signalum"						),
		Lumium						= unused("Lumium"						),
		PhasedIron					= unused("Phased Iron"					).add(G_INGOT),
		PhasedGold					= unused("Phased Gold"					).add(G_INGOT),
		Soularium					= unused("Soularium"					),
		Endium						= unused("Endium"						),
		Draconium					= unused("Draconium"					).add(G_INGOT),
		DraconiumAwakened			= unused("DraconiumAwakened"			).add(G_INGOT).setLocal("Awakened Draconium"),
		Prismarine					= unused("Prismarine"					),
		GraveyardDirt				= unused("Graveyard Dirt"				),
		DarkSteel					= unused("Dark Steel"					).add(G_INGOT),
		IrridantUranium				= unused("Irridant Uranium"				).add(G_INGOT),
		IridiumIron					= unused("IridiumIron"					).add(G_INGOT).setLocal("Iridiron"),
		IridiumIronReinforced		= unused("IridiumIronReinforced"		).add(G_INGOT).setLocal("Reinforced Iridiron"),
		IrridantReinforced			= unused("IrridantReinforced"			).add(G_INGOT),
		IronSharp					= unused("IronSharp"					).add(G_INGOT).setLocal("Sharp Iron"),
		Cocaine						= unused("Cocaine"						),
		Vile						= unused("Vile"							),
		Dull						= unused("Dull"							),
		Dark						= unused("Dark"							),
		Ectoplasm					= unused("Ectoplasm"					),
		Soulium						= unused("Soulium"						),
		Terrasteel					= unused("Terrasteel"					).add(G_INGOT, MAGICAL),
		TerrasteelAlloyRaw			= unused("TerrasteelAlloyRaw"			).add(G_INGOT, MAGICAL).setLocal("Raw Terrasteel Alloy").addIdenticalNames("RawTerrasteelAlloy"),
		TerrasteelAlloyStrengthened	= unused("TerrasteelAlloyStrengthened"	).add(G_INGOT, MAGICAL).setLocal("Strengthened Terrasteel Alloy").addIdenticalNames("StrengthenedTerrasteelAlloy"),
		ConductiveIron				= unused("Conductive Iron"				).add(G_INGOT),
		ElectricalSteel				= unused("Electrical Steel"				).add(G_INGOT), // SiFe9 or something like that.
		EnergeticAlloy				= unused("Energetic Alloy"				),
		VibrantAlloy				= unused("Vibrant Alloy"				),
		PulsatingIron				= unused("Pulsating Iron"				).add(G_INGOT),
		Rutile						= unused("Rutile"						), // TiO2
		Fluix						= unused("Fluix"						).add(CRYSTAL, BRITTLE, QUARTZ, CRYSTALLISABLE), // AE
		Manasteel					= unused("Manasteel"					),
		Tennantite					= unused("Tennantite"					),
		Alfium						= unused("Alfium"						),
		Ryu							= unused("Ryu"							),
		Mutation					= unused("Mutation"						),
		Aquamarine					= unused("Aquamarine"					),
		Ender						= unused("Ender"						),
		ElvenElementium				= unused("Elven Elementium"				).add(MAGICAL),
		EnrichedCopper				= unused("Enriched Copper"				),
		DiamondCopper				= unused("Diamond Copper"				),
		SodiumPeroxide				= unused("Sodium Peroxide"				),
		IridiumSodiumOxide			= unused("Iridium Sodium Oxide"			),
		PlatinumGroupSludge			= unused("Platinum Group Sludge"		),
		Fairy						= unused("Fairy"						),
		Ludicrite					= unused("Ludicrite"					),
		Pokefennium					= unused("Pokefennium"					);
	}
}